searchData={"items":[{"type":"module","title":"Hermes.Client","doc":"High-level DSL for defining MCP (Model Context Protocol) clients.\n\nThis module provides an Ecto-like interface for creating MCP clients with minimal boilerplate.\nBy using this module, you get a fully functional MCP client with automatic supervision,\ntransport management, and all standard MCP operations.","ref":"Hermes.Client.html"},{"type":"module","title":"Usage - Hermes.Client","doc":"Define a client module:\n\n    defmodule MyApp.AnthropicClient do\n      use Hermes.Client,\n        name: \"MyApp\",\n        version: \"1.0.0\",\n        protocol_version: \"2024-11-05\",\n        capabilities: [:roots, {:sampling, list_changed?: true}]\n    end\n\nAdd it to your supervision tree:\n\n    children = [\n      {MyApp.AnthropicClient, \n       transport: {:stdio, command: \"uvx\", args: [\"mcp-server-anthropic\"]}}\n    ]\n\nUse the client:\n\n    {:ok, tools} = MyApp.AnthropicClient.list_tools()\n    {:ok, result} = MyApp.AnthropicClient.call_tool(\"search\", %{query: \"elixir\"})","ref":"Hermes.Client.html#module-usage"},{"type":"module","title":"Options - Hermes.Client","doc":"The `use` macro accepts the following required options:\n\n  * `:name` - The client name to advertise to the server (string)\n  * `:version` - The client version (string)\n  * `:protocol_version` - The MCP protocol version (string)\n  * `:capabilities` - List of client capabilities (see below)","ref":"Hermes.Client.html#module-options"},{"type":"module","title":"Capabilities - Hermes.Client","doc":"Capabilities can be specified as:\n\n  * Atoms: `:roots`, `:sampling`\n  * Tuples with options: `{:roots, list_changed?: true}`\n  * Maps for custom capabilities: `%{\"custom\" => %{\"feature\" => true}}`","ref":"Hermes.Client.html#module-capabilities"},{"type":"module","title":"Transport Configuration - Hermes.Client","doc":"When starting the client, you must provide transport configuration:\n\n  * `{:stdio, command: \"cmd\", args: [\"arg1\", \"arg2\"]}`\n  * `{:sse, base_url: \"http://localhost:8000\"}`\n  * `{:websocket, url: \"ws://localhost:8000/ws\"}`\n  * `{:streamable_http, url: \"http://localhost:8000/mcp\"}`","ref":"Hermes.Client.html#module-transport-configuration"},{"type":"module","title":"Process Naming - Hermes.Client","doc":"By default, the client process is registered with the module name.\nYou can override this with the `:name` option in `child_spec` or `start_link`:\n\n    # Custom atom name\n    {MyApp.AnthropicClient, name: :my_custom_client, transport: ...}\n    \n    # For distributed systems with registries (e.g., Horde)\n    {MyApp.AnthropicClient,\n     name: {:via, Horde.Registry, {MyCluster, \"client_1\"}},\n     transport_name: {:via, Horde.Registry, {MyCluster, \"transport_1\"}},\n     transport: ...}\n\nWhen using via tuples or other non-atom names, you must explicitly provide\nthe `:transport_name` option. For atom names, the transport is automatically\nnamed as `Module.concat(ClientName, \"Transport\")`.","ref":"Hermes.Client.html#module-process-naming"},{"type":"macro","title":"Hermes.Client.__using__/1","doc":"Generates an MCP client module with all necessary functions.\n\nThis macro is used via the `use` directive and accepts the following options:\n\n  * `:name` - Client name (required, string)\n  * `:version` - Client version (required, string)  \n  * `:protocol_version` - MCP protocol version (required, string)\n  * `:capabilities` - List of capabilities (optional, defaults to empty list)\n\nThe macro generates:\n\n  * `child_spec/1` - For supervision tree integration\n  * `start_link/1` - To start the client\n  * All MCP operation functions (ping, list_tools, call_tool, etc.)","ref":"Hermes.Client.html#__using__/1"},{"type":"type","title":"Hermes.Client.capabilities/0","doc":"","ref":"Hermes.Client.html#t:capabilities/0"},{"type":"type","title":"Hermes.Client.capability/0","doc":"","ref":"Hermes.Client.html#t:capability/0"},{"type":"type","title":"Hermes.Client.capability_opts/0","doc":"","ref":"Hermes.Client.html#t:capability_opts/0"},{"type":"macro","title":"Hermes.Client.is_client_capability/1","doc":"Guard to check if an atom is a valid client capability.","ref":"Hermes.Client.html#is_client_capability/1"},{"type":"macro","title":"Hermes.Client.is_supported_capability/2","doc":"Guard to check if a capability is supported by checking map keys.","ref":"Hermes.Client.html#is_supported_capability/2"},{"type":"module","title":"Hermes.Client.Base","doc":"A GenServer implementation of an MCP (Model Context Protocol) client.\n\nThis module handles the client-side implementation of the MCP protocol,\nincluding initialization, request/response handling, and maintaining\nprotocol state.\n\n>","ref":"Hermes.Client.Base.html"},{"type":"module","title":"Notes - Hermes.Client.Base","doc":">\n> For initialization and setup, check our [Installation & Setup](./installation.html) and\n> the [Client Usage](./client_usage.html) guides for reference.","ref":"Hermes.Client.Base.html#module-notes"},{"type":"function","title":"Hermes.Client.Base.add_root/4","doc":"Adds a root directory to the client's roots list.","ref":"Hermes.Client.Base.html#add_root/4"},{"type":"function","title":"Parameters - Hermes.Client.Base.add_root/4","doc":"* `client` - The client process\n  * `uri` - The URI of the root directory (must start with \"file://\")\n  * `name` - Optional human-readable name for the root\n  * `opts` - Additional options\n    * `:timeout` - Request timeout in milliseconds","ref":"Hermes.Client.Base.html#add_root/4-parameters"},{"type":"function","title":"Examples - Hermes.Client.Base.add_root/4","doc":"iex> Hermes.Client.add_root(client, \"file:///home/user/project\", \"My Project\")\n    :ok","ref":"Hermes.Client.Base.html#add_root/4-examples"},{"type":"function","title":"Hermes.Client.Base.call_tool/4","doc":"Calls a tool on the server.","ref":"Hermes.Client.Base.html#call_tool/4"},{"type":"function","title":"Options - Hermes.Client.Base.call_tool/4","doc":"* `:timeout` - Request timeout in milliseconds\n  * `:progress` - Progress tracking options\n    * `:token` - A unique token to track progress (string or integer)\n    * `:callback` - A function to call when progress updates are received","ref":"Hermes.Client.Base.html#call_tool/4-options"},{"type":"function","title":"Hermes.Client.Base.cancel_all_requests/3","doc":"Cancels all pending requests.","ref":"Hermes.Client.Base.html#cancel_all_requests/3"},{"type":"function","title":"Parameters - Hermes.Client.Base.cancel_all_requests/3","doc":"* `client` - The client process\n  * `reason` - Optional reason for cancellation (defaults to \"client_cancelled\")","ref":"Hermes.Client.Base.html#cancel_all_requests/3-parameters"},{"type":"function","title":"Returns - Hermes.Client.Base.cancel_all_requests/3","doc":"* `{:ok, requests}` - A list of the Request structs that were cancelled\n  * `{:error, reason}` - If an error occurred","ref":"Hermes.Client.Base.html#cancel_all_requests/3-returns"},{"type":"function","title":"Hermes.Client.Base.cancel_request/4","doc":"Cancels an in-progress request.","ref":"Hermes.Client.Base.html#cancel_request/4"},{"type":"function","title":"Parameters - Hermes.Client.Base.cancel_request/4","doc":"* `client` - The client process\n  * `request_id` - The ID of the request to cancel\n  * `reason` - Optional reason for cancellation","ref":"Hermes.Client.Base.html#cancel_request/4-parameters"},{"type":"function","title":"Returns - Hermes.Client.Base.cancel_request/4","doc":"* `:ok` if the cancellation was successful\n  * `{:error, reason}` if an error occurred\n  * `{:not_found, request_id}` if the request ID was not found","ref":"Hermes.Client.Base.html#cancel_request/4-returns"},{"type":"type","title":"Hermes.Client.Base.capabilities/0","doc":"MCP client capabilities\n\n- `:roots` - Capabilities related to the roots resource\n  - `:listChanged` - Whether the client can handle listChanged notifications\n- `:sampling` - Capabilities related to sampling\n\nMCP describes these client capabilities on it [specification](https://spec.modelcontextprotocol.io/specification/2024-11-05/client/)","ref":"Hermes.Client.Base.html#t:capabilities/0"},{"type":"function","title":"Hermes.Client.Base.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Hermes.Client.Base.html#child_spec/1"},{"type":"function","title":"Hermes.Client.Base.clear_roots/2","doc":"Clears all root directories.","ref":"Hermes.Client.Base.html#clear_roots/2"},{"type":"function","title":"Parameters - Hermes.Client.Base.clear_roots/2","doc":"* `client` - The client process\n  * `opts` - Additional options\n    * `:timeout` - Request timeout in milliseconds","ref":"Hermes.Client.Base.html#clear_roots/2-parameters"},{"type":"function","title":"Examples - Hermes.Client.Base.clear_roots/2","doc":"iex> Hermes.Client.clear_roots(client)\n    :ok","ref":"Hermes.Client.Base.html#clear_roots/2-examples"},{"type":"type","title":"Hermes.Client.Base.client_info/0","doc":"MCP client metadata info\n\n- `:name` - The name of the client (required)\n- `:version` - The version of the client","ref":"Hermes.Client.Base.html#t:client_info/0"},{"type":"function","title":"Hermes.Client.Base.close/1","doc":"Closes the client connection and terminates the process.","ref":"Hermes.Client.Base.html#close/1"},{"type":"function","title":"Hermes.Client.Base.complete/4","doc":"Requests autocompletion suggestions for prompt arguments or resource URIs.","ref":"Hermes.Client.Base.html#complete/4"},{"type":"function","title":"Parameters - Hermes.Client.Base.complete/4","doc":"* `client` - The client process\n  * `ref` - Reference to what is being completed (required)\n    * For prompts: `%{\"type\" => \"ref/prompt\", \"name\" => prompt_name}`\n    * For resources: `%{\"type\" => \"ref/resource\", \"uri\" => resource_uri}`\n  * `argument` - The argument being completed (required)\n    * `%{\"name\" => arg_name, \"value\" => current_value}`\n  * `opts` - Additional options\n    * `:timeout` - Request timeout in milliseconds\n    * `:progress` - Progress tracking options\n      * `:token` - A unique token to track progress (string or integer)\n      * `:callback` - A function to call when progress updates are received","ref":"Hermes.Client.Base.html#complete/4-parameters"},{"type":"function","title":"Returns - Hermes.Client.Base.complete/4","doc":"Returns `{:ok, response}` with completion suggestions if successful, or `{:error, reason}` if an error occurs.\n\nThe response result contains a \"completion\" object with:\n* `values` - List of completion suggestions (maximum 100)\n* `total` - Optional total number of matching items\n* `hasMore` - Boolean indicating if more results are available","ref":"Hermes.Client.Base.html#complete/4-returns"},{"type":"function","title":"Examples - Hermes.Client.Base.complete/4","doc":"# Get completion for a prompt argument\n    ref = %{\"type\" => \"ref/prompt\", \"name\" => \"code_review\"}\n    argument = %{\"name\" => \"language\", \"value\" => \"py\"}\n    {:ok, response} = Hermes.Client.complete(client, ref, argument)\n    \n    # Access the completion values\n    values = get_in(Response.unwrap(response), [\"completion\", \"values\"])","ref":"Hermes.Client.Base.html#complete/4-examples"},{"type":"function","title":"Hermes.Client.Base.get_prompt/4","doc":"Gets a specific prompt from the server.","ref":"Hermes.Client.Base.html#get_prompt/4"},{"type":"function","title":"Options - Hermes.Client.Base.get_prompt/4","doc":"* `:timeout` - Request timeout in milliseconds\n  * `:progress` - Progress tracking options\n    * `:token` - A unique token to track progress (string or integer)\n    * `:callback` - A function to call when progress updates are received","ref":"Hermes.Client.Base.html#get_prompt/4-options"},{"type":"function","title":"Hermes.Client.Base.get_schema/1","doc":"","ref":"Hermes.Client.Base.html#get_schema/1"},{"type":"function","title":"Hermes.Client.Base.get_server_capabilities/2","doc":"Gets the server's capabilities as reported during initialization.\n\nReturns `nil` if the client has not been initialized yet.","ref":"Hermes.Client.Base.html#get_server_capabilities/2"},{"type":"function","title":"Hermes.Client.Base.get_server_info/2","doc":"Gets the server's information as reported during initialization.\n\nReturns `nil` if the client has not been initialized yet.","ref":"Hermes.Client.Base.html#get_server_info/2"},{"type":"function","title":"Hermes.Client.Base.list_prompts/2","doc":"Lists available prompts from the server.","ref":"Hermes.Client.Base.html#list_prompts/2"},{"type":"function","title":"Options - Hermes.Client.Base.list_prompts/2","doc":"* `:cursor` - Pagination cursor for continuing a previous request\n  * `:timeout` - Request timeout in milliseconds\n  * `:progress` - Progress tracking options\n    * `:token` - A unique token to track progress (string or integer)\n    * `:callback` - A function to call when progress updates are received","ref":"Hermes.Client.Base.html#list_prompts/2-options"},{"type":"function","title":"Hermes.Client.Base.list_resources/2","doc":"Lists available resources from the server.","ref":"Hermes.Client.Base.html#list_resources/2"},{"type":"function","title":"Options - Hermes.Client.Base.list_resources/2","doc":"* `:cursor` - Pagination cursor for continuing a previous request\n  * `:timeout` - Request timeout in milliseconds\n  * `:progress` - Progress tracking options\n    * `:token` - A unique token to track progress (string or integer)\n    * `:callback` - A function to call when progress updates are received","ref":"Hermes.Client.Base.html#list_resources/2-options"},{"type":"function","title":"Hermes.Client.Base.list_roots/2","doc":"Gets a list of all root directories.","ref":"Hermes.Client.Base.html#list_roots/2"},{"type":"function","title":"Parameters - Hermes.Client.Base.list_roots/2","doc":"* `client` - The client process\n  * `opts` - Additional options\n    * `:timeout` - Request timeout in milliseconds","ref":"Hermes.Client.Base.html#list_roots/2-parameters"},{"type":"function","title":"Examples - Hermes.Client.Base.list_roots/2","doc":"iex> Hermes.Client.list_roots(client)\n    [%{uri: \"file:///home/user/project\", name: \"My Project\"}]","ref":"Hermes.Client.Base.html#list_roots/2-examples"},{"type":"function","title":"Hermes.Client.Base.list_tools/2","doc":"Lists available tools from the server.","ref":"Hermes.Client.Base.html#list_tools/2"},{"type":"function","title":"Options - Hermes.Client.Base.list_tools/2","doc":"* `:cursor` - Pagination cursor for continuing a previous request\n  * `:timeout` - Request timeout in milliseconds\n  * `:progress` - Progress tracking options\n    * `:token` - A unique token to track progress (string or integer)\n    * `:callback` - A function to call when progress updates are received","ref":"Hermes.Client.Base.html#list_tools/2-options"},{"type":"type","title":"Hermes.Client.Base.log_callback/0","doc":"Log callback function type.\n\nCalled when log message notifications are received from the server.","ref":"Hermes.Client.Base.html#t:log_callback/0"},{"type":"type","title":"Parameters - Hermes.Client.Base.log_callback/0","doc":"- `level` - Log level as a string (e.g., \"debug\", \"info\", \"warning\", \"error\")\n  - `data` - Log message data, typically a map with message details\n  - `logger` - Optional logger name identifying the source","ref":"Hermes.Client.Base.html#t:log_callback/0-parameters"},{"type":"type","title":"Returns - Hermes.Client.Base.log_callback/0","doc":"- The return value is ignored","ref":"Hermes.Client.Base.html#t:log_callback/0-returns"},{"type":"function","title":"Hermes.Client.Base.merge_capabilities/3","doc":"Merges additional capabilities into the client's capabilities.","ref":"Hermes.Client.Base.html#merge_capabilities/3"},{"type":"type","title":"Hermes.Client.Base.option/0","doc":"MCP client initialization options\n\n- `:name` - Following the `GenServer` patterns described on \"Name registration\".\n- `:transport` - The MCP transport options\n- `:client_info` - Information about the client\n- `:capabilities` - Client capabilities to advertise to the MCP server\n- `:protocol_version` - Protocol version to use (defaults to \"2024-11-05\")\n\nAny other option support by `GenServer`.","ref":"Hermes.Client.Base.html#t:option/0"},{"type":"function","title":"Hermes.Client.Base.parse_options/1","doc":"","ref":"Hermes.Client.Base.html#parse_options/1"},{"type":"function","title":"Hermes.Client.Base.parse_options!/1","doc":"","ref":"Hermes.Client.Base.html#parse_options!/1"},{"type":"function","title":"Hermes.Client.Base.ping/2","doc":"Sends a ping request to the server to check connection health. Returns `:pong` if successful.","ref":"Hermes.Client.Base.html#ping/2"},{"type":"function","title":"Options - Hermes.Client.Base.ping/2","doc":"* `:timeout` - Request timeout in milliseconds (default: 30s)\n  * `:progress` - Progress tracking options\n    * `:token` - A unique token to track progress (string or integer)\n    * `:callback` - A function to call when progress updates are received","ref":"Hermes.Client.Base.html#ping/2-options"},{"type":"type","title":"Hermes.Client.Base.progress_callback/0","doc":"Progress callback function type.\n\nCalled when progress notifications are received for a specific progress token.","ref":"Hermes.Client.Base.html#t:progress_callback/0"},{"type":"type","title":"Parameters - Hermes.Client.Base.progress_callback/0","doc":"- `progress_token` - String or integer identifier for the progress operation\n  - `progress` - Current progress value\n  - `total` - Total expected value (nil if unknown)","ref":"Hermes.Client.Base.html#t:progress_callback/0-parameters"},{"type":"type","title":"Returns - Hermes.Client.Base.progress_callback/0","doc":"- The return value is ignored","ref":"Hermes.Client.Base.html#t:progress_callback/0-returns"},{"type":"function","title":"Hermes.Client.Base.read_resource/3","doc":"Reads a specific resource from the server.","ref":"Hermes.Client.Base.html#read_resource/3"},{"type":"function","title":"Options - Hermes.Client.Base.read_resource/3","doc":"* `:timeout` - Request timeout in milliseconds\n  * `:progress` - Progress tracking options\n    * `:token` - A unique token to track progress (string or integer)\n    * `:callback` - A function to call when progress updates are received","ref":"Hermes.Client.Base.html#read_resource/3-options"},{"type":"function","title":"Hermes.Client.Base.register_log_callback/3","doc":"Registers a callback function to be called when log messages are received.","ref":"Hermes.Client.Base.html#register_log_callback/3"},{"type":"function","title":"Parameters - Hermes.Client.Base.register_log_callback/3","doc":"* `client` - The client process\n  * `callback` - A function that takes three arguments: level, data, and logger name\n\nThe callback function will be called whenever a log message notification is received.","ref":"Hermes.Client.Base.html#register_log_callback/3-parameters"},{"type":"function","title":"Hermes.Client.Base.register_progress_callback/4","doc":"Registers a callback function to be called when progress notifications are received\nfor the specified progress token.","ref":"Hermes.Client.Base.html#register_progress_callback/4"},{"type":"function","title":"Parameters - Hermes.Client.Base.register_progress_callback/4","doc":"* `client` - The client process\n  * `progress_token` - The progress token to watch for (string or integer)\n  * `callback` - A function that takes three arguments: progress_token, progress, and total\n\nThe callback function will be called whenever a progress notification with the\nmatching token is received.","ref":"Hermes.Client.Base.html#register_progress_callback/4-parameters"},{"type":"function","title":"Hermes.Client.Base.remove_root/3","doc":"Removes a root directory from the client's roots list.","ref":"Hermes.Client.Base.html#remove_root/3"},{"type":"function","title":"Parameters - Hermes.Client.Base.remove_root/3","doc":"* `client` - The client process\n  * `uri` - The URI of the root directory to remove\n  * `opts` - Additional options\n    * `:timeout` - Request timeout in milliseconds","ref":"Hermes.Client.Base.html#remove_root/3-parameters"},{"type":"function","title":"Examples - Hermes.Client.Base.remove_root/3","doc":"iex> Hermes.Client.remove_root(client, \"file:///home/user/project\")\n    :ok","ref":"Hermes.Client.Base.html#remove_root/3-examples"},{"type":"type","title":"Hermes.Client.Base.root/0","doc":"Root directory specification.\n\nRepresents a root directory that the client has access to.","ref":"Hermes.Client.Base.html#t:root/0"},{"type":"type","title":"Fields - Hermes.Client.Base.root/0","doc":"- `:uri` - File URI for the root directory (e.g., \"file:///home/user/project\")\n  - `:name` - Optional human-readable name for the root","ref":"Hermes.Client.Base.html#t:root/0-fields"},{"type":"function","title":"Hermes.Client.Base.send_batch/2","doc":"Sends a batch of operations to the server.","ref":"Hermes.Client.Base.html#send_batch/2"},{"type":"function","title":"Parameters - Hermes.Client.Base.send_batch/2","doc":"* `client` - The client process\n  * `operations` - List of Operation structs to send as a batch","ref":"Hermes.Client.Base.html#send_batch/2-parameters"},{"type":"function","title":"Returns - Hermes.Client.Base.send_batch/2","doc":"* `{:ok, results}` - A list of results in the same order as the operations.\n    Each result is either `{:ok, response}` or `{:error, error}`.\n  * `{:error, reason}` - If the entire batch fails (e.g., transport error)","ref":"Hermes.Client.Base.html#send_batch/2-returns"},{"type":"function","title":"Example - Hermes.Client.Base.send_batch/2","doc":"operations = [\n      Operation.new(%{method: \"ping\", params: %{}}),\n      Operation.new(%{method: \"tools/list\", params: %{}})\n    ]\n    \n    {:ok, results} = Hermes.Client.send_batch(client, operations)\n    # results is a list: [{:ok, :pong}, {:ok, tools_response}]","ref":"Hermes.Client.Base.html#send_batch/2-example"},{"type":"function","title":"Hermes.Client.Base.send_progress/5","doc":"Sends a progress notification to the server for a long-running operation.","ref":"Hermes.Client.Base.html#send_progress/5"},{"type":"function","title":"Parameters - Hermes.Client.Base.send_progress/5","doc":"* `client` - The client process\n  * `progress_token` - The progress token provided in the original request (string or integer)\n  * `progress` - The current progress value (number)\n  * `total` - The optional total value for the operation (number)\n\nReturns `:ok` if notification was sent successfully, or `{:error, reason}` otherwise.","ref":"Hermes.Client.Base.html#send_progress/5-parameters"},{"type":"function","title":"Hermes.Client.Base.set_log_level/2","doc":"Sets the minimum log level for the server to send log messages.","ref":"Hermes.Client.Base.html#set_log_level/2"},{"type":"function","title":"Parameters - Hermes.Client.Base.set_log_level/2","doc":"* `client` - The client process\n  * `level` - The minimum log level (debug, info, notice, warning, error, critical, alert, emergency)\n\nReturns {:ok, result} if successful, {:error, reason} otherwise.","ref":"Hermes.Client.Base.html#set_log_level/2-parameters"},{"type":"function","title":"Hermes.Client.Base.start_link/1","doc":"Starts a new MCP client process.","ref":"Hermes.Client.Base.html#start_link/1"},{"type":"type","title":"Hermes.Client.Base.t/0","doc":"","ref":"Hermes.Client.Base.html#t:t/0"},{"type":"type","title":"Hermes.Client.Base.transport/0","doc":"MCP client transport options\n\n- `:layer` - The transport layer to use, either `Hermes.Transport.STDIO`, `Hermes.Transport.SSE`, `Hermes.Transport.WebSocket`, or `Hermes.Transport.StreamableHTTP` (required)\n- `:name` - The transport optional custom name","ref":"Hermes.Client.Base.html#t:transport/0"},{"type":"function","title":"Hermes.Client.Base.unregister_log_callback/2","doc":"Unregisters a previously registered log callback.","ref":"Hermes.Client.Base.html#unregister_log_callback/2"},{"type":"function","title":"Parameters - Hermes.Client.Base.unregister_log_callback/2","doc":"* `client` - The client process\n  * `callback` - The callback function to unregister","ref":"Hermes.Client.Base.html#unregister_log_callback/2-parameters"},{"type":"function","title":"Hermes.Client.Base.unregister_progress_callback/3","doc":"Unregisters a previously registered progress callback for the specified token.","ref":"Hermes.Client.Base.html#unregister_progress_callback/3"},{"type":"function","title":"Parameters - Hermes.Client.Base.unregister_progress_callback/3","doc":"* `client` - The client process\n  * `progress_token` - The progress token to stop watching (string or integer)","ref":"Hermes.Client.Base.html#unregister_progress_callback/3-parameters"},{"type":"module","title":"Hermes.Client.Operation","doc":"Represents an operation to be performed by the MCP client.\n\nThis struct encapsulates all information about a client API call:\n- `method` - The MCP method to call\n- `params` - The parameters to send to the server\n- `progress_opts` - Progress tracking options (optional)\n- `timeout` - The timeout for this specific operation (default: 30 seconds)","ref":"Hermes.Client.Operation.html"},{"type":"function","title":"Hermes.Client.Operation.new/1","doc":"Creates a new operation struct.","ref":"Hermes.Client.Operation.html#new/1"},{"type":"function","title":"Parameters - Hermes.Client.Operation.new/1","doc":"* `attrs` - Map containing the operation attributes\n    * `:method` - The MCP method name (required)\n    * `:params` - The parameters to send to the server (required)\n    * `:progress_opts` - Progress tracking options (optional)\n    * `:timeout` - The timeout for this operation in milliseconds (optional, defaults to 30s)","ref":"Hermes.Client.Operation.html#new/1-parameters"},{"type":"type","title":"Hermes.Client.Operation.progress_options/0","doc":"","ref":"Hermes.Client.Operation.html#t:progress_options/0"},{"type":"type","title":"Hermes.Client.Operation.t/0","doc":"","ref":"Hermes.Client.Operation.html#t:t/0"},{"type":"module","title":"Hermes.Client.Request","doc":"Represents a pending request in the MCP client.\n\nThis struct encapsulates all information about an in-progress request:\n- `id` - The unique request ID\n- `method` - The MCP method being called\n- `from` - The GenServer caller reference\n- `timer_ref` - Reference to the request-specific timeout timer\n- `start_time` - When the request started (monotonic time in milliseconds)\n- `batch_id` - The batch ID if this request is part of a batch (nil for single requests)","ref":"Hermes.Client.Request.html"},{"type":"function","title":"Hermes.Client.Request.elapsed_time/1","doc":"Calculates the elapsed time for a request in milliseconds.","ref":"Hermes.Client.Request.html#elapsed_time/1"},{"type":"function","title":"Hermes.Client.Request.new/1","doc":"Creates a new request struct.","ref":"Hermes.Client.Request.html#new/1"},{"type":"function","title":"Parameters - Hermes.Client.Request.new/1","doc":"* `attrs` - Map containing the request attributes\n    * `:id` - The unique request ID\n    * `:method` - The MCP method name\n    * `:from` - The GenServer caller reference\n    * `:timer_ref` - Reference to the request-specific timeout timer\n    * `:batch_id` - Optional batch ID if part of a batch request","ref":"Hermes.Client.Request.html#new/1-parameters"},{"type":"type","title":"Hermes.Client.Request.t/0","doc":"","ref":"Hermes.Client.Request.html#t:t/0"},{"type":"module","title":"Hermes.Client.Supervisor","doc":"Supervisor for MCP client processes.\n\nThis module manages the lifecycle of both the MCP client and its associated\ntransport process. It uses a `:one_for_all` strategy, meaning if either the\nclient or transport crashes, both are restarted together to maintain consistency.\n\nThe supervisor automatically:\n- Starts the appropriate transport based on configuration\n- Starts the client with a reference to the transport\n- Ensures proper initialization order (transport first, then client)\n- Handles process naming for both client and transport","ref":"Hermes.Client.Supervisor.html"},{"type":"module","title":"Process Naming - Hermes.Client.Supervisor","doc":"- Client process: Uses the module name or custom `:name` option\n- Transport process: Named as `Module.concat(ClientName, \"Transport\")`","ref":"Hermes.Client.Supervisor.html#module-process-naming"},{"type":"module","title":"Transport Configuration - Hermes.Client.Supervisor","doc":"Supports all Hermes transport types:\n- `:stdio` - For command-line MCP servers\n- `:sse` - For Server-Sent Events transports\n- `:websocket` - For WebSocket connections\n- `:streamable_http` - For streaming HTTP transports","ref":"Hermes.Client.Supervisor.html#module-transport-configuration"},{"type":"function","title":"Hermes.Client.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Hermes.Client.Supervisor.html#child_spec/1"},{"type":"function","title":"Hermes.Client.Supervisor.start_link/2","doc":"Starts the client supervisor.","ref":"Hermes.Client.Supervisor.html#start_link/2"},{"type":"function","title":"Arguments - Hermes.Client.Supervisor.start_link/2","doc":"* `client_module` - The client module using `Hermes.Client`\n  * `opts` - Supervisor options including:\n    * `:name` - Optional custom name for the client process\n    * `:transport` - Transport configuration (required)\n    * `:transport_name` - Optional custom name for the transport process\n    * `:client_info` - Client identification info\n    * `:capabilities` - Client capabilities map\n    * `:protocol_version` - MCP protocol version","ref":"Hermes.Client.Supervisor.html#start_link/2-arguments"},{"type":"function","title":"Examples - Hermes.Client.Supervisor.start_link/2","doc":"# Simple usage with module names\n    Hermes.Client.Supervisor.start_link(MyApp.MCPClient, \n      transport: {:stdio, command: \"mcp\", args: [\"server\"]},\n      client_info: %{\"name\" => \"MyApp\", \"version\" => \"1.0.0\"},\n      capabilities: %{\"roots\" => %{}},\n      protocol_version: \"2024-11-05\"\n    )\n    \n    # With custom names (e.g., for distributed systems)\n    Hermes.Client.Supervisor.start_link(MyApp.MCPClient,\n      name: {:via, Horde.Registry, {MyCluster, \"client_1\"}},\n      transport_name: {:via, Horde.Registry, {MyCluster, \"transport_1\"}},\n      transport: {:stdio, command: \"mcp\", args: [\"server\"]},\n      client_info: %{\"name\" => \"MyApp\", \"version\" => \"1.0.0\"},\n      capabilities: %{\"roots\" => %{}},\n      protocol_version: \"2024-11-05\"\n    )","ref":"Hermes.Client.Supervisor.html#start_link/2-examples"},{"type":"type","title":"Hermes.Client.Supervisor.transport_config/0","doc":"","ref":"Hermes.Client.Supervisor.html#t:transport_config/0"},{"type":"module","title":"Hermes.MCP.Error","doc":"Fluent API for building MCP protocol errors.\n\nThis module provides a semantic interface for creating errors that comply with\nthe MCP specification and JSON-RPC 2.0 error standards.","ref":"Hermes.MCP.Error.html"},{"type":"module","title":"Error Categories - Hermes.MCP.Error","doc":"- **Protocol Errors**: Standard JSON-RPC errors (parse, invalid request, etc.)\n- **Transport Errors**: Connection and communication issues\n- **Resource Errors**: MCP-specific resource handling errors\n- **Execution Errors**: Tool and operation execution failures","ref":"Hermes.MCP.Error.html#module-error-categories"},{"type":"module","title":"Examples - Hermes.MCP.Error","doc":"# Protocol errors\n    Hermes.MCP.Error.protocol(:parse_error)\n    Hermes.MCP.Error.protocol(:method_not_found, %{method: \"unknown\"})\n\n    # Transport errors\n    Hermes.MCP.Error.transport(:connection_refused)\n    Hermes.MCP.Error.transport(:timeout, %{elapsed_ms: 30000})\n\n    # Resource errors\n    Hermes.MCP.Error.resource(:not_found, %{uri: \"file:///missing.txt\"})\n\n    # Execution errors with custom messages\n    Hermes.MCP.Error.execution(\"Database connection failed\", %{retries: 3})\n\n    # Converting from JSON-RPC\n    Hermes.MCP.Error.from_json_rpc(%{\"code\" => -32700, \"message\" => \"Parse error\"})","ref":"Hermes.MCP.Error.html#module-examples"},{"type":"function","title":"Hermes.MCP.Error.build_json_rpc/2","doc":"","ref":"Hermes.MCP.Error.html#build_json_rpc/2"},{"type":"function","title":"Hermes.MCP.Error.execution/2","doc":"Creates an execution error with a custom message.\n\nUsed for tool execution failures and domain-specific errors.","ref":"Hermes.MCP.Error.html#execution/2"},{"type":"function","title":"Examples - Hermes.MCP.Error.execution/2","doc":"iex> Hermes.MCP.Error.execution(\"Database connection failed\")\n    %Hermes.MCP.Error{code: -32000, reason: :execution_error, message: \"Database connection failed\", data: %{}}\n\n    iex> Hermes.MCP.Error.execution(\"API rate limit exceeded\", %{retry_after: 60})\n    %Hermes.MCP.Error{code: -32000, reason: :execution_error, message: \"API rate limit exceeded\", data: %{retry_after: 60}}","ref":"Hermes.MCP.Error.html#execution/2-examples"},{"type":"function","title":"Hermes.MCP.Error.from_json_rpc/1","doc":"Creates an error from a JSON-RPC error object.","ref":"Hermes.MCP.Error.html#from_json_rpc/1"},{"type":"function","title":"Examples - Hermes.MCP.Error.from_json_rpc/1","doc":"iex> Hermes.MCP.Error.from_json_rpc(%{\"code\" => -32700, \"message\" => \"Parse error\"})\n    %Hermes.MCP.Error{code: -32700, reason: :parse_error, message: \"Parse error\", data: %{}}\n\n    iex> Hermes.MCP.Error.from_json_rpc(%{\"code\" => -32002, \"message\" => \"Not found\", \"data\" => %{\"uri\" => \"/file\"}})\n    %Hermes.MCP.Error{code: -32002, reason: :resource_not_found, message: \"Not found\", data: %{\"uri\" => \"/file\"}}","ref":"Hermes.MCP.Error.html#from_json_rpc/1-examples"},{"type":"function","title":"Hermes.MCP.Error.protocol/2","doc":"Creates a protocol-level error.\n\nThese are standard JSON-RPC errors that occur during message parsing and validation.","ref":"Hermes.MCP.Error.html#protocol/2"},{"type":"function","title":"Supported Reasons - Hermes.MCP.Error.protocol/2","doc":"- `:parse_error` - Invalid JSON was received\n- `:invalid_request` - The JSON is not a valid Request object\n- `:method_not_found` - The method does not exist\n- `:invalid_params` - Invalid method parameters\n- `:internal_error` - Internal JSON-RPC error","ref":"Hermes.MCP.Error.html#protocol/2-supported-reasons"},{"type":"function","title":"Examples - Hermes.MCP.Error.protocol/2","doc":"iex> Hermes.MCP.Error.protocol(:parse_error)\n    %Hermes.MCP.Error{code: -32700, reason: :parse_error, message: \"Parse error\", data: %{}}\n\n    iex> Hermes.MCP.Error.protocol(:method_not_found, %{method: \"foo\"})\n    %Hermes.MCP.Error{code: -32601, reason: :method_not_found, message: \"Method not found\", data: %{method: \"foo\"}}","ref":"Hermes.MCP.Error.html#protocol/2-examples"},{"type":"function","title":"Hermes.MCP.Error.resource/2","doc":"Creates a resource-specific error.\n\nUsed for MCP resource operations.","ref":"Hermes.MCP.Error.html#resource/2"},{"type":"function","title":"Examples - Hermes.MCP.Error.resource/2","doc":"iex> Hermes.MCP.Error.resource(:not_found, %{uri: \"file:///missing.txt\"})\n    %Hermes.MCP.Error{code: -32002, reason: :resource_not_found, message: \"Resource not found\", data: %{uri: \"file:///missing.txt\"}}","ref":"Hermes.MCP.Error.html#resource/2-examples"},{"type":"type","title":"Hermes.MCP.Error.t/0","doc":"","ref":"Hermes.MCP.Error.html#t:t/0"},{"type":"function","title":"Hermes.MCP.Error.to_json_rpc/2","doc":"Encodes the error as a JSON-RPC error response.","ref":"Hermes.MCP.Error.html#to_json_rpc/2"},{"type":"function","title":"Examples - Hermes.MCP.Error.to_json_rpc/2","doc":"iex> error = Hermes.MCP.Error.protocol(:parse_error)\n    iex> {:ok, encoded} = Hermes.MCP.Error.to_json_rpc(error, \"req-123\")\n    iex> String.contains?(encoded, \"Parse error\")\n    true","ref":"Hermes.MCP.Error.html#to_json_rpc/2-examples"},{"type":"function","title":"Hermes.MCP.Error.transport/2","doc":"Creates a transport-level error.\n\nUsed for network, connection, and communication failures.","ref":"Hermes.MCP.Error.html#transport/2"},{"type":"function","title":"Examples - Hermes.MCP.Error.transport/2","doc":"iex> Hermes.MCP.Error.transport(:connection_refused)\n    %Hermes.MCP.Error{code: -32000, reason: :connection_refused, message: \"Connection Refused\", data: %{}}\n\n    iex> Hermes.MCP.Error.transport(:timeout, %{elapsed_ms: 5000})\n    %Hermes.MCP.Error{code: -32000, reason: :timeout, message: \"Timeout\", data: %{elapsed_ms: 5000}}","ref":"Hermes.MCP.Error.html#transport/2-examples"},{"type":"module","title":"Hermes.MCP.Message","doc":"Handles parsing and validation of MCP (Model Context Protocol) messages using the Peri library.\n\nThis module provides functions to parse and validate MCP messages based on the Model Context Protocol schema","ref":"Hermes.MCP.Message.html"},{"type":"function","title":"Hermes.MCP.Message.batch_schema/1","doc":"","ref":"Hermes.MCP.Message.html#batch_schema/1"},{"type":"function","title":"Hermes.MCP.Message.batch_schema!/1","doc":"","ref":"Hermes.MCP.Message.html#batch_schema!/1"},{"type":"function","title":"Hermes.MCP.Message.build_error/2","doc":"Builds an error message map without encoding to JSON.\n\nThis is useful for batch processing where we need the raw map.","ref":"Hermes.MCP.Message.html#build_error/2"},{"type":"function","title":"Parameters - Hermes.MCP.Message.build_error/2","doc":"* `error` - The error map with code, message, and optional data\n  * `id` - The error response ID","ref":"Hermes.MCP.Message.html#build_error/2-parameters"},{"type":"function","title":"Examples - Hermes.MCP.Message.build_error/2","doc":"iex> Message.build_error(%{\"code\" => -32600, \"message\" => \"Invalid Request\"}, \"req_123\")\n    %{\"jsonrpc\" => \"2.0\", \"error\" => %{\"code\" => -32600, \"message\" => \"Invalid Request\"}, \"id\" => \"req_123\"}","ref":"Hermes.MCP.Message.html#build_error/2-examples"},{"type":"function","title":"Hermes.MCP.Message.build_notification/2","doc":"Builds a notification message map without encoding to JSON.\n\nThis is useful for batch processing where we need the raw map.","ref":"Hermes.MCP.Message.html#build_notification/2"},{"type":"function","title":"Parameters - Hermes.MCP.Message.build_notification/2","doc":"* `method` - The notification method\n  * `params` - The notification parameters","ref":"Hermes.MCP.Message.html#build_notification/2-parameters"},{"type":"function","title":"Examples - Hermes.MCP.Message.build_notification/2","doc":"iex> Message.build_notification(\"notifications/message\", %{\"level\" => \"info\", \"data\" => \"test\"})\n    %{\"jsonrpc\" => \"2.0\", \"method\" => \"notifications/message\", \"params\" => %{\"level\" => \"info\", \"data\" => \"test\"}}","ref":"Hermes.MCP.Message.html#build_notification/2-examples"},{"type":"function","title":"Hermes.MCP.Message.build_response/2","doc":"Builds a response message map without encoding to JSON.\n\nThis is useful for batch processing where we need the raw map.","ref":"Hermes.MCP.Message.html#build_response/2"},{"type":"function","title":"Parameters - Hermes.MCP.Message.build_response/2","doc":"* `result` - The result data\n  * `id` - The response ID","ref":"Hermes.MCP.Message.html#build_response/2-parameters"},{"type":"function","title":"Examples - Hermes.MCP.Message.build_response/2","doc":"iex> Message.build_response(%{\"value\" => 42}, \"req_123\")\n    %{\"jsonrpc\" => \"2.0\", \"result\" => %{\"value\" => 42}, \"id\" => \"req_123\"}","ref":"Hermes.MCP.Message.html#build_response/2-examples"},{"type":"function","title":"Hermes.MCP.Message.decode/1","doc":"Decodes raw data (possibly containing multiple messages) into JSON-RPC messages.\n\nHandles both single messages and batch messages (arrays).\n\nReturns either:\n- `{:ok, messages}` where messages is a list of parsed JSON-RPC messages\n- `{:error, reason}` if parsing fails","ref":"Hermes.MCP.Message.html#decode/1"},{"type":"function","title":"Hermes.MCP.Message.encode_batch/2","doc":"Encodes a batch of JSON-RPC messages.\n\nThis function uses Peri schema validation and always returns a JSON array.","ref":"Hermes.MCP.Message.html#encode_batch/2"},{"type":"function","title":"Parameters - Hermes.MCP.Message.encode_batch/2","doc":"* `messages` - A list of complete JSON-RPC message maps\n  * `batch_schema` - Optional Peri schema for batch validation (defaults to :batch_schema)\n\nReturns `{:ok, encoded_batch}` if successful, or `{:error, reason}` if validation fails.","ref":"Hermes.MCP.Message.html#encode_batch/2-parameters"},{"type":"function","title":"Hermes.MCP.Message.encode_error/2","doc":"Encodes an error message to a JSON-RPC 2.0 compliant string.\n\nReturns the encoded string with a newline character appended.","ref":"Hermes.MCP.Message.html#encode_error/2"},{"type":"function","title":"Hermes.MCP.Message.encode_log_message/3","doc":"Encodes a log message notification to be sent to the client.","ref":"Hermes.MCP.Message.html#encode_log_message/3"},{"type":"function","title":"Parameters - Hermes.MCP.Message.encode_log_message/3","doc":"* `level` - The log level (debug, info, notice, warning, error, critical, alert, emergency)\n  * `data` - The data to be logged (any JSON-serializable value)\n  * `logger` - Optional name of the logger issuing the message\n\nReturns the encoded notification string with a newline character appended.","ref":"Hermes.MCP.Message.html#encode_log_message/3-parameters"},{"type":"function","title":"Hermes.MCP.Message.encode_notification/1","doc":"Encodes a notification message to a JSON-RPC 2.0 compliant string.\n\nReturns the encoded string with a newline character appended.","ref":"Hermes.MCP.Message.html#encode_notification/1"},{"type":"function","title":"Hermes.MCP.Message.encode_notification/2","doc":"Encodes a notification message using a custom schema.","ref":"Hermes.MCP.Message.html#encode_notification/2"},{"type":"function","title":"Parameters - Hermes.MCP.Message.encode_notification/2","doc":"* `notification` - The notification map containing method and params\n  * `schema` - The Peri schema to use for validation\n\nReturns the encoded string with a newline character appended.","ref":"Hermes.MCP.Message.html#encode_notification/2-parameters"},{"type":"function","title":"Hermes.MCP.Message.encode_progress_notification/2","doc":"Encodes a progress notification message to a JSON-RPC 2.0 compliant string.","ref":"Hermes.MCP.Message.html#encode_progress_notification/2"},{"type":"function","title":"Parameters - Hermes.MCP.Message.encode_progress_notification/2","doc":"* `params` - Map containing progress parameters:\n    * `\"progressToken\"` - The token that was provided in the original request (string or integer)\n    * `\"progress\"` - The current progress value (number) \n    * `\"total\"` - Optional total value for the operation (number)\n    * `\"message\"` - Optional descriptive message (string, for 2025-03-26)\n  * `params_schema` - Optional Peri schema for params validation (defaults to @progress_notif_params_schema)\n\nReturns the encoded string with a newline character appended.","ref":"Hermes.MCP.Message.html#encode_progress_notification/2-parameters"},{"type":"function","title":"Hermes.MCP.Message.encode_progress_notification/3","doc":"Legacy function for progress notifications with individual parameters.\n\n**Deprecated**: Prefer using `encode_progress_notification/2` with a params map.\n\nThis function will be removed in a future release. Update your code to use the newer function:\n\n    encode_progress_notification(%{\n      \"progressToken\" => progress_token,\n      \"progress\" => progress,\n      \"total\" => total\n    })","ref":"Hermes.MCP.Message.html#encode_progress_notification/3"},{"type":"function","title":"Hermes.MCP.Message.encode_request/2","doc":"Encodes a request message to a JSON-RPC 2.0 compliant string.\n\nReturns the encoded string with a newline character appended.","ref":"Hermes.MCP.Message.html#encode_request/2"},{"type":"function","title":"Hermes.MCP.Message.encode_request/3","doc":"Encodes a request message using a custom schema.","ref":"Hermes.MCP.Message.html#encode_request/3"},{"type":"function","title":"Parameters - Hermes.MCP.Message.encode_request/3","doc":"* `request` - The request map containing method and params\n  * `id` - The request ID\n  * `schema` - The Peri schema to use for validation\n\nReturns the encoded string with a newline character appended.","ref":"Hermes.MCP.Message.html#encode_request/3-parameters"},{"type":"function","title":"Hermes.MCP.Message.encode_response/2","doc":"Encodes a response message to a JSON-RPC 2.0 compliant string.\n\nReturns the encoded string with a newline character appended.","ref":"Hermes.MCP.Message.html#encode_response/2"},{"type":"function","title":"Hermes.MCP.Message.encode_response/3","doc":"Encodes a response message using a custom schema.","ref":"Hermes.MCP.Message.html#encode_response/3"},{"type":"function","title":"Parameters - Hermes.MCP.Message.encode_response/3","doc":"* `response` - The response map containing result\n  * `id` - The response ID\n  * `schema` - The Peri schema to use for validation\n\nReturns the encoded string with a newline character appended.","ref":"Hermes.MCP.Message.html#encode_response/3-parameters"},{"type":"function","title":"Hermes.MCP.Message.error_schema/1","doc":"","ref":"Hermes.MCP.Message.html#error_schema/1"},{"type":"function","title":"Hermes.MCP.Message.error_schema!/1","doc":"","ref":"Hermes.MCP.Message.html#error_schema!/1"},{"type":"function","title":"Hermes.MCP.Message.get_schema/1","doc":"","ref":"Hermes.MCP.Message.html#get_schema/1"},{"type":"macro","title":"Hermes.MCP.Message.is_error/1","doc":"Guard to determine if a JSON-RPC message is an error.\n\nA message is considered an error if it contains both \"error\" and \"id\" fields.","ref":"Hermes.MCP.Message.html#is_error/1"},{"type":"macro","title":"Examples - Hermes.MCP.Message.is_error/1","doc":"iex> message = %{\"jsonrpc\" => \"2.0\", \"error\" => %{\"code\" => -32600}, \"id\" => 1}\n    iex> is_error(message)\n    true","ref":"Hermes.MCP.Message.html#is_error/1-examples"},{"type":"macro","title":"Hermes.MCP.Message.is_initialize/1","doc":"Guard to check if a request is an initialize request.","ref":"Hermes.MCP.Message.html#is_initialize/1"},{"type":"macro","title":"Examples - Hermes.MCP.Message.is_initialize/1","doc":"iex> message = %{\"jsonrpc\" => \"2.0\", \"method\" => \"initialize\", \"id\" => 1, \"params\" => %{}}\n    iex> is_initialize(message)\n    true","ref":"Hermes.MCP.Message.html#is_initialize/1-examples"},{"type":"macro","title":"Hermes.MCP.Message.is_initialize_lifecycle/1","doc":"Guard to check if a message is part of the initialization lifecycle.\n\nThis includes both the initialize request and the notifications/initialized notification.","ref":"Hermes.MCP.Message.html#is_initialize_lifecycle/1"},{"type":"macro","title":"Examples - Hermes.MCP.Message.is_initialize_lifecycle/1","doc":"iex> init_request = %{\"jsonrpc\" => \"2.0\", \"method\" => \"initialize\", \"id\" => 1, \"params\" => %{}}\n    iex> is_initialize_lifecycle(init_request)\n    true\n\n    iex> init_notification = %{\"jsonrpc\" => \"2.0\", \"method\" => \"notifications/initialized\"}\n    iex> is_initialize_lifecycle(init_notification)\n    true\n\n    iex> other_message = %{\"jsonrpc\" => \"2.0\", \"method\" => \"tools/list\", \"id\" => 2}\n    iex> is_initialize_lifecycle(other_message)\n    false","ref":"Hermes.MCP.Message.html#is_initialize_lifecycle/1-examples"},{"type":"macro","title":"Hermes.MCP.Message.is_notification/1","doc":"Guard to determine if a JSON-RPC message is a notification.\n\nA message is considered a notification if it contains a \"method\" field but no \"id\" field.","ref":"Hermes.MCP.Message.html#is_notification/1"},{"type":"macro","title":"Examples - Hermes.MCP.Message.is_notification/1","doc":"iex> message = %{\"jsonrpc\" => \"2.0\", \"method\" => \"notification\"}\n    iex> is_notification(message)\n    true\n\n    iex> request = %{\"jsonrpc\" => \"2.0\", \"method\" => \"ping\", \"id\" => 1}\n    iex> is_notification(request)\n    false","ref":"Hermes.MCP.Message.html#is_notification/1-examples"},{"type":"macro","title":"Hermes.MCP.Message.is_ping/1","doc":"Guard to check if a request is a ping request.","ref":"Hermes.MCP.Message.html#is_ping/1"},{"type":"macro","title":"Examples - Hermes.MCP.Message.is_ping/1","doc":"iex> message = %{\"jsonrpc\" => \"2.0\", \"method\" => \"ping\", \"id\" => 1}\n    iex> is_ping(message)\n    true","ref":"Hermes.MCP.Message.html#is_ping/1-examples"},{"type":"macro","title":"Hermes.MCP.Message.is_request/1","doc":"Guard to determine if a JSON-RPC message is a request.\n\nA message is considered a request if it contains both \"method\" and \"id\" fields.","ref":"Hermes.MCP.Message.html#is_request/1"},{"type":"macro","title":"Examples - Hermes.MCP.Message.is_request/1","doc":"iex> message = %{\"jsonrpc\" => \"2.0\", \"method\" => \"ping\", \"id\" => 1}\n    iex> is_request(message)\n    true\n\n    iex> notification = %{\"jsonrpc\" => \"2.0\", \"method\" => \"notification\"}\n    iex> is_request(notification)\n    false","ref":"Hermes.MCP.Message.html#is_request/1-examples"},{"type":"macro","title":"Hermes.MCP.Message.is_response/1","doc":"Guard to determine if a JSON-RPC message is a response.\n\nA message is considered a response if it contains both \"result\" and \"id\" fields.","ref":"Hermes.MCP.Message.html#is_response/1"},{"type":"macro","title":"Examples - Hermes.MCP.Message.is_response/1","doc":"iex> message = %{\"jsonrpc\" => \"2.0\", \"result\" => %{}, \"id\" => 1}\n    iex> is_response(message)\n    true","ref":"Hermes.MCP.Message.html#is_response/1-examples"},{"type":"function","title":"Hermes.MCP.Message.mcp_message_schema/1","doc":"","ref":"Hermes.MCP.Message.html#mcp_message_schema/1"},{"type":"function","title":"Hermes.MCP.Message.mcp_message_schema!/1","doc":"","ref":"Hermes.MCP.Message.html#mcp_message_schema!/1"},{"type":"function","title":"Hermes.MCP.Message.notification_schema/1","doc":"","ref":"Hermes.MCP.Message.html#notification_schema/1"},{"type":"function","title":"Hermes.MCP.Message.notification_schema!/1","doc":"","ref":"Hermes.MCP.Message.html#notification_schema!/1"},{"type":"function","title":"Hermes.MCP.Message.progress_params_schema/0","doc":"Returns the standard progress notification parameters schema for 2024-11-05.","ref":"Hermes.MCP.Message.html#progress_params_schema/0"},{"type":"function","title":"Hermes.MCP.Message.progress_params_schema_2025/0","doc":"Returns the progress notification parameters schema for 2025-03-26 (with message field).","ref":"Hermes.MCP.Message.html#progress_params_schema_2025/0"},{"type":"function","title":"Hermes.MCP.Message.request_schema/1","doc":"","ref":"Hermes.MCP.Message.html#request_schema/1"},{"type":"function","title":"Hermes.MCP.Message.request_schema!/1","doc":"","ref":"Hermes.MCP.Message.html#request_schema!/1"},{"type":"function","title":"Hermes.MCP.Message.response_schema/1","doc":"","ref":"Hermes.MCP.Message.html#response_schema/1"},{"type":"function","title":"Hermes.MCP.Message.response_schema!/1","doc":"","ref":"Hermes.MCP.Message.html#response_schema!/1"},{"type":"function","title":"Hermes.MCP.Message.validate_message/1","doc":"Validates a decoded JSON message to ensure it complies with the MCP schema.","ref":"Hermes.MCP.Message.html#validate_message/1"},{"type":"module","title":"Hermes.MCP.Response","doc":"Represents successful responses in the MCP protocol.\n\nThis module provides a wrapper around JSON-RPC responses, handling\ndomain-specific error semantics for MCP's \"isError\" field in results.","ref":"Hermes.MCP.Response.html"},{"type":"module","title":"Response Structure - Hermes.MCP.Response","doc":"Each response includes:\n- `result`: The response data from the server\n- `id`: The request ID this response is associated with\n- `is_error`: Boolean flag indicating if this is a domain-level error","ref":"Hermes.MCP.Response.html#module-response-structure"},{"type":"module","title":"Domain vs. Protocol Errors - Hermes.MCP.Response","doc":"The MCP protocol distinguishes between two types of errors:\n\n1. Protocol errors: Standard JSON-RPC errors with error codes (handled by `Hermes.MCP.Error`)\n2. Domain errors: Valid responses that indicate application-level errors with `isError: true`\n\nThis module specifically handles domain errors, which are successful at the protocol level\nbut indicate failures at the application level.","ref":"Hermes.MCP.Response.html#module-domain-vs-protocol-errors"},{"type":"module","title":"Examples - Hermes.MCP.Response","doc":"```elixir\n# Create from a JSON-RPC response\nresponse = Hermes.MCP.Response.from_json_rpc(%{\"result\" => %{\"data\" => \"value\"}, \"id\" => \"req_123\"})\n\n# Check if the response is successful or has a domain error\nif Hermes.MCP.Response.success?(response) do\n  # Handle success\nelse\n  # Handle domain error\nend\n\n# Unwrap the response to get the result or error\ncase Hermes.MCP.Response.unwrap(response) do\n  {:ok, result} -> # Handle success\n  {:error, error} -> # Handle domain error\nend\n```","ref":"Hermes.MCP.Response.html#module-examples"},{"type":"function","title":"Hermes.MCP.Response.error?/1","doc":"Checks if the response has a domain error.","ref":"Hermes.MCP.Response.html#error?/1"},{"type":"function","title":"Examples - Hermes.MCP.Response.error?/1","doc":"iex> response = Hermes.MCP.Response.from_json_rpc(%{\"result\" => %{\"data\" => \"value\"}, \"id\" => \"req_123\"})\n    iex> Hermes.MCP.Response.error?(response)\n    false\n    \n    iex> error_response = Hermes.MCP.Response.from_json_rpc(%{\"result\" => %{\"isError\" => true}, \"id\" => \"req_123\"})\n    iex> Hermes.MCP.Response.error?(error_response)\n    true","ref":"Hermes.MCP.Response.html#error?/1-examples"},{"type":"function","title":"Hermes.MCP.Response.from_json_rpc/1","doc":"Creates a Response struct from a JSON-RPC response.\n\nAutomatically detects domain errors by checking for the \"isError\" field.","ref":"Hermes.MCP.Response.html#from_json_rpc/1"},{"type":"function","title":"Parameters - Hermes.MCP.Response.from_json_rpc/1","doc":"* `response` - A map containing the JSON-RPC response","ref":"Hermes.MCP.Response.html#from_json_rpc/1-parameters"},{"type":"function","title":"Examples - Hermes.MCP.Response.from_json_rpc/1","doc":"iex> Hermes.MCP.Response.from_json_rpc(%{\"result\" => %{}, \"id\" => \"req_123\"})\n    %Hermes.MCP.Response{result: %{}, id: \"req_123\", is_error: false}\n    \n    iex> Hermes.MCP.Response.from_json_rpc(%{\"result\" => %{\"isError\" => true}, \"id\" => \"req_123\"})\n    %Hermes.MCP.Response{result: %{\"isError\" => true}, id: \"req_123\", is_error: true}","ref":"Hermes.MCP.Response.html#from_json_rpc/1-examples"},{"type":"function","title":"Hermes.MCP.Response.get_id/1","doc":"Gets the request ID associated with this response.","ref":"Hermes.MCP.Response.html#get_id/1"},{"type":"function","title":"Examples - Hermes.MCP.Response.get_id/1","doc":"iex> response = Hermes.MCP.Response.from_json_rpc(%{\"result\" => %{}, \"id\" => \"req_123\"})\n    iex> Hermes.MCP.Response.get_id(response)\n    \"req_123\"","ref":"Hermes.MCP.Response.html#get_id/1-examples"},{"type":"function","title":"Hermes.MCP.Response.get_result/1","doc":"Gets the result data from the response.\n\nThis function returns the raw result regardless of whether it represents\na success or domain error.","ref":"Hermes.MCP.Response.html#get_result/1"},{"type":"function","title":"Examples - Hermes.MCP.Response.get_result/1","doc":"iex> response = Hermes.MCP.Response.from_json_rpc(%{\"result\" => %{\"data\" => \"value\"}, \"id\" => \"req_123\"})\n    iex> Hermes.MCP.Response.get_result(response)\n    %{\"data\" => \"value\"}","ref":"Hermes.MCP.Response.html#get_result/1-examples"},{"type":"function","title":"Hermes.MCP.Response.success?/1","doc":"Checks if the response is successful (no domain error).","ref":"Hermes.MCP.Response.html#success?/1"},{"type":"function","title":"Examples - Hermes.MCP.Response.success?/1","doc":"iex> response = Hermes.MCP.Response.from_json_rpc(%{\"result\" => %{\"data\" => \"value\"}, \"id\" => \"req_123\"})\n    iex> Hermes.MCP.Response.success?(response)\n    true\n    \n    iex> error_response = Hermes.MCP.Response.from_json_rpc(%{\"result\" => %{\"isError\" => true}, \"id\" => \"req_123\"})\n    iex> Hermes.MCP.Response.success?(error_response)\n    false","ref":"Hermes.MCP.Response.html#success?/1-examples"},{"type":"type","title":"Hermes.MCP.Response.t/0","doc":"","ref":"Hermes.MCP.Response.html#t:t/0"},{"type":"function","title":"Hermes.MCP.Response.unwrap/1","doc":"Unwraps the response, returning the raw result.\n\nReturns the raw result data regardless of whether it represents\na success or domain error.","ref":"Hermes.MCP.Response.html#unwrap/1"},{"type":"function","title":"Examples - Hermes.MCP.Response.unwrap/1","doc":"iex> response = Hermes.MCP.Response.from_json_rpc(%{\"result\" => %{\"data\" => \"value\"}, \"id\" => \"req_123\"})\n    iex> Hermes.MCP.Response.unwrap(response)\n    %{\"data\" => \"value\"}\n    \n    iex> error_response = Hermes.MCP.Response.from_json_rpc(%{\"result\" => %{\"isError\" => true, \"reason\" => \"not_found\"}, \"id\" => \"req_123\"})\n    iex> Hermes.MCP.Response.unwrap(error_response)\n    %{\"isError\" => true, \"reason\" => \"not_found\"}","ref":"Hermes.MCP.Response.html#unwrap/1-examples"},{"type":"module","title":"Hermes.Protocol","doc":"Protocol version management and feature validation for MCP.\n\nThis module handles protocol version compatibility, feature detection,\nand transport validation following the MCP specification.","ref":"Hermes.Protocol.html"},{"type":"module","title":"Protocol Versions - Hermes.Protocol","doc":"- `2024-11-05`: Initial stable release with SSE and basic features\n- `2025-03-26`: Enhanced version with Streamable HTTP, authorization, batching, and extended features","ref":"Hermes.Protocol.html#module-protocol-versions"},{"type":"module","title":"Examples - Hermes.Protocol","doc":"iex> Hermes.Protocol.validate_transport(\"2024-11-05\", Hermes.Transport.SSE)\n    :ok\n\n    iex> Hermes.Protocol.validate_transport(\"2024-11-05\", Hermes.Transport.StreamableHTTP)\n    {:error, %Hermes.MCP.Error{reason: :incompatible_transport}}\n\n    iex> Hermes.Protocol.supports_feature?(\"2025-03-26\", :json_rpc_batching)\n    true","ref":"Hermes.Protocol.html#module-examples"},{"type":"function","title":"Hermes.Protocol.compatible_transports/2","doc":"Returns transport modules that support a protocol version.","ref":"Hermes.Protocol.html#compatible_transports/2"},{"type":"function","title":"Parameters - Hermes.Protocol.compatible_transports/2","doc":"* `version` - The protocol version\n  * `transport_modules` - List of transport modules to check","ref":"Hermes.Protocol.html#compatible_transports/2-parameters"},{"type":"function","title":"Examples - Hermes.Protocol.compatible_transports/2","doc":"iex> transports = [Hermes.Transport.STDIO, Hermes.Transport.SSE]\n    iex> Hermes.Protocol.compatible_transports(\"2024-11-05\", transports)\n    [Hermes.Transport.STDIO, Hermes.Transport.SSE]","ref":"Hermes.Protocol.html#compatible_transports/2-examples"},{"type":"function","title":"Hermes.Protocol.fallback_version/0","doc":"Returns the fallback protocol version for compatibility.","ref":"Hermes.Protocol.html#fallback_version/0"},{"type":"function","title":"Examples - Hermes.Protocol.fallback_version/0","doc":"iex> Hermes.Protocol.fallback_version()\n    \"2024-11-05\"","ref":"Hermes.Protocol.html#fallback_version/0-examples"},{"type":"type","title":"Hermes.Protocol.feature/0","doc":"","ref":"Hermes.Protocol.html#t:feature/0"},{"type":"function","title":"Hermes.Protocol.get_features/1","doc":"Returns the set of features supported by a protocol version.","ref":"Hermes.Protocol.html#get_features/1"},{"type":"function","title":"Parameters - Hermes.Protocol.get_features/1","doc":"* `version` - The protocol version","ref":"Hermes.Protocol.html#get_features/1-parameters"},{"type":"function","title":"Examples - Hermes.Protocol.get_features/1","doc":"iex> features = Hermes.Protocol.get_features(\"2025-03-26\")\n    iex> MapSet.member?(features, :json_rpc_batching)\n    true","ref":"Hermes.Protocol.html#get_features/1-examples"},{"type":"function","title":"Hermes.Protocol.latest_version/0","doc":"Returns the latest supported protocol version.","ref":"Hermes.Protocol.html#latest_version/0"},{"type":"function","title":"Examples - Hermes.Protocol.latest_version/0","doc":"iex> Hermes.Protocol.latest_version()\n    \"2025-03-26\"","ref":"Hermes.Protocol.html#latest_version/0-examples"},{"type":"function","title":"Hermes.Protocol.negotiate_version/2","doc":"Negotiates protocol version between client and server versions.\n\nReturns the best compatible version or an error if incompatible.","ref":"Hermes.Protocol.html#negotiate_version/2"},{"type":"function","title":"Parameters - Hermes.Protocol.negotiate_version/2","doc":"* `client_version` - The client's preferred protocol version\n  * `server_version` - The server's supported protocol version","ref":"Hermes.Protocol.html#negotiate_version/2-parameters"},{"type":"function","title":"Examples - Hermes.Protocol.negotiate_version/2","doc":"iex> Hermes.Protocol.negotiate_version(\"2025-03-26\", \"2025-03-26\")\n    {:ok, \"2025-03-26\"}\n\n    iex> Hermes.Protocol.negotiate_version(\"2025-03-26\", \"2024-11-05\")\n    {:ok, \"2024-11-05\"}\n\n    iex> Hermes.Protocol.negotiate_version(\"2024-11-05\", \"1.0.0\")\n    {:error, %Hermes.MCP.Error{reason: :incompatible_versions}}","ref":"Hermes.Protocol.html#negotiate_version/2-examples"},{"type":"function","title":"Hermes.Protocol.supported_versions/0","doc":"Returns all supported protocol versions.","ref":"Hermes.Protocol.html#supported_versions/0"},{"type":"function","title":"Examples - Hermes.Protocol.supported_versions/0","doc":"iex> Hermes.Protocol.supported_versions()\n    [\"2024-11-05\", \"2025-03-26\"]","ref":"Hermes.Protocol.html#supported_versions/0-examples"},{"type":"function","title":"Hermes.Protocol.supports_feature?/2","doc":"Checks if a feature is supported by a protocol version.","ref":"Hermes.Protocol.html#supports_feature?/2"},{"type":"function","title":"Parameters - Hermes.Protocol.supports_feature?/2","doc":"* `version` - The protocol version\n  * `feature` - The feature to check","ref":"Hermes.Protocol.html#supports_feature?/2-parameters"},{"type":"function","title":"Examples - Hermes.Protocol.supports_feature?/2","doc":"iex> Hermes.Protocol.supports_feature?(\"2025-03-26\", :json_rpc_batching)\n    true\n\n    iex> Hermes.Protocol.supports_feature?(\"2024-11-05\", :authorization)\n    false","ref":"Hermes.Protocol.html#supports_feature?/2-examples"},{"type":"function","title":"Hermes.Protocol.validate_client_config/3","doc":"Validates client configuration for protocol compatibility.\n\nThis function checks if the client configuration is compatible with\nthe specified protocol version, including transport and capabilities.","ref":"Hermes.Protocol.html#validate_client_config/3"},{"type":"function","title":"Parameters - Hermes.Protocol.validate_client_config/3","doc":"* `version` - The protocol version\n  * `transport_module` - The transport module being used\n  * `capabilities` - The client capabilities","ref":"Hermes.Protocol.html#validate_client_config/3-parameters"},{"type":"function","title":"Examples - Hermes.Protocol.validate_client_config/3","doc":"iex> capabilities = %{\"resources\" => %{}, \"tools\" => %{}}\n    iex> Hermes.Protocol.validate_client_config(\"2024-11-05\", Hermes.Transport.SSE, capabilities)\n    :ok","ref":"Hermes.Protocol.html#validate_client_config/3-examples"},{"type":"function","title":"Hermes.Protocol.validate_transport/2","doc":"Validates if a transport is compatible with a protocol version.","ref":"Hermes.Protocol.html#validate_transport/2"},{"type":"function","title":"Parameters - Hermes.Protocol.validate_transport/2","doc":"* `version` - The protocol version\n  * `transport_module` - The transport module to validate","ref":"Hermes.Protocol.html#validate_transport/2-parameters"},{"type":"function","title":"Examples - Hermes.Protocol.validate_transport/2","doc":"iex> Hermes.Protocol.validate_transport(\"2024-11-05\", Hermes.Transport.SSE)\n    :ok\n\n    iex> Hermes.Protocol.validate_transport(\"2024-11-05\", Hermes.Transport.StreamableHTTP)\n    {:error, %Hermes.MCP.Error{reason: :incompatible_transport}}","ref":"Hermes.Protocol.html#validate_transport/2-examples"},{"type":"function","title":"Hermes.Protocol.validate_version/1","doc":"Validates if a protocol version is supported.","ref":"Hermes.Protocol.html#validate_version/1"},{"type":"function","title":"Parameters - Hermes.Protocol.validate_version/1","doc":"* `version` - The protocol version to validate","ref":"Hermes.Protocol.html#validate_version/1-parameters"},{"type":"function","title":"Examples - Hermes.Protocol.validate_version/1","doc":"iex> Hermes.Protocol.validate_version(\"2024-11-05\")\n    :ok\n\n    iex> Hermes.Protocol.validate_version(\"1.0.0\")\n    {:error, %Hermes.MCP.Error{reason: :unsupported_protocol_version}}","ref":"Hermes.Protocol.html#validate_version/1-examples"},{"type":"type","title":"Hermes.Protocol.version/0","doc":"","ref":"Hermes.Protocol.html#t:version/0"},{"type":"behaviour","title":"Hermes.Server","doc":"Build MCP servers that extend language model capabilities.\n\nMCP servers are specialized processes that provide three core primitives to AI assistants:\n**Resources** (contextual data like files or schemas), **Tools** (actions the model can invoke),\nand **Prompts** (user-selectable templates). They operate in a secure, isolated architecture\nwhere clients maintain 1:1 connections with servers, enabling composable functionality while\nmaintaining strict security boundaries.","ref":"Hermes.Server.html"},{"type":"behaviour","title":"Quick Start - Hermes.Server","doc":"Create a server in three steps:\n\n    defmodule MyServer do\n      use Hermes.Server,\n        name: \"my-server\",\n        version: \"1.0.0\",\n        capabilities: [:tools]\n\n      component MyServer.Calculator\n    end\n\n    defmodule MyServer.Calculator do\n      use Hermes.Server.Component, type: :tool\n\n      def definition do\n        %{\n          name: \"add\",\n          description: \"Add two numbers\",\n          input_schema: %{\n            type: \"object\",\n            properties: %{\n              a: %{type: \"number\"},\n              b: %{type: \"number\"}\n            }\n          }\n        }\n      end\n\n      def call(%{\"a\" => a, \"b\" => b}), do: {:ok, a + b}\n    end\n\n    # Start your server\n    {:ok, _pid} = Hermes.Server.start_link(MyServer, [], transport: :stdio)\n\nYour server is now a living process that AI assistants can connect to, discover available\ntools, and execute calculations through a secure protocol boundary.","ref":"Hermes.Server.html#module-quick-start"},{"type":"behaviour","title":"Capabilities - Hermes.Server","doc":"Declare what your server can do:\n\n- **`:tools`** - Execute functions with structured inputs and outputs\n- **`:resources`** - Provide data that models can read (files, APIs, databases)\n- **`:prompts`** - Offer reusable templates for common interactions\n- **`:logging`** - Allow clients to configure verbosity levels\n\nConfigure capabilities with options:\n\n    use Hermes.Server,\n      capabilities: [\n        :tools,\n        {:resources, subscribe?: true},      # Enable resource update subscriptions\n        {:prompts, list_changed?: true}      # Notify when prompts change\n      ]","ref":"Hermes.Server.html#module-capabilities"},{"type":"behaviour","title":"Components - Hermes.Server","doc":"Register tools, resources, and prompts as components:\n\n    component MyServer.FileReader           # Auto-named as \"file_reader\"\n    component MyServer.ApiClient, name: \"api\"   # Custom name\n\nComponents are modules that implement specific behaviors (`Hermes.Server.Component`)\nand are automatically discovered by clients through the protocol.","ref":"Hermes.Server.html#module-components"},{"type":"behaviour","title":"Server Lifecycle - Hermes.Server","doc":"Your server follows a predictable lifecycle with callbacks you can hook into:\n\n1. **`init/2`** - Set up initial state when the server starts\n2. **`handle_request/2`** - Process MCP protocol requests from clients\n3. **`handle_notification/2`** - React to one-way client messages\n4. **`handle_info/2`** - Bridge external events into MCP notifications\n\nMost protocol handling is automatic - you typically only implement `init/2` for setup\nand occasionally override other callbacks for custom behavior.","ref":"Hermes.Server.html#module-server-lifecycle"},{"type":"macro","title":"Hermes.Server.component/2","doc":"Registers a component (tool, prompt, or resource) with the server.","ref":"Hermes.Server.html#component/2"},{"type":"macro","title":"Examples - Hermes.Server.component/2","doc":"# Register with auto-derived name\n    component MyServer.Tools.Calculator\n    \n    # Register with custom name\n    component MyServer.Tools.FileManager, name: \"files\"","ref":"Hermes.Server.html#component/2-examples"},{"type":"callback","title":"Hermes.Server.handle_call/3","doc":"Handles synchronous calls to the server process.\n\nThis optional callback allows you to handle custom synchronous calls made to your\nMCP server process using `GenServer.call/2`. This is useful for implementing\nadministrative functions, status queries, or any synchronous operations that\nneed to interact with the server's internal state.\n\nThe callback follows standard GenServer semantics and should return appropriate\nreply tuples. If not implemented, the Base module provides a default implementation\nthat handles standard MCP operations.","ref":"Hermes.Server.html#c:handle_call/3"},{"type":"callback","title":"Hermes.Server.handle_cast/2","doc":"Handles asynchronous casts to the server process.\n\nThis optional callback allows you to handle custom asynchronous messages sent to your\nMCP server process using `GenServer.cast/2`. This is useful for fire-and-forget\noperations, background tasks, or any asynchronous operations that don't require\nan immediate response.\n\nThe callback follows standard GenServer semantics. If not implemented, the Base\nmodule provides a default implementation that handles standard MCP operations.","ref":"Hermes.Server.html#c:handle_cast/2"},{"type":"callback","title":"Hermes.Server.handle_info/2","doc":"Handles non-MCP messages sent to the server process.\n\nWhile `handle_request` and `handle_notification` deal with MCP protocol messages,\nthis callback handles everything else - timer events, messages from other processes,\nsystem signals, and any custom inter-process communication your server needs.\n\nThis is particularly useful for servers that need to react to external events\n(like file system changes or database updates) and notify connected clients through\nMCP notifications. Think of it as the bridge between your Elixir application's\ninternal events and the MCP protocol's notification system.","ref":"Hermes.Server.html#c:handle_info/2"},{"type":"callback","title":"Hermes.Server.handle_notification/2","doc":"Handles incoming MCP notifications from clients.\n\nNotifications are one-way messages in the MCP protocol - the client informs the server\nabout events or state changes without expecting a response. This fire-and-forget pattern\nis perfect for status updates, progress tracking, and lifecycle events.\n\n**Standard MCP Notifications from Clients:**\n- `notifications/initialized` - Client signals it's ready after successful initialization\n- `notifications/cancelled` - Client requests cancellation of an in-progress operation\n- `notifications/progress` - Client reports progress on a long-running operation\n- `notifications/roots/list_changed` - Client's available filesystem roots have changed\n\nUnlike requests, notifications never receive responses. Any errors during processing\nare typically logged but not communicated back to the client. This makes notifications\nideal for optional features like progress tracking where delivery isn't guaranteed.\n\nThe server processes these notifications to update its internal state, trigger side effects,\nor coordinate with other parts of the system. When using `use Hermes.Server`, basic\nnotification handling is provided, but you'll often want to override this callback\nto handle progress updates or cancellations specific to your server's operations.","ref":"Hermes.Server.html#c:handle_notification/2"},{"type":"callback","title":"Hermes.Server.handle_prompt_get/3","doc":"Handles a prompt get request.\n\nThis callback is invoked when a client requests a specific prompt template. It receives\nthe prompt name, any arguments to fill into the template, and the current frame.\n\nThis callback handles both module-based components (registered with `component`) and\nruntime components (registered with `Frame.register_prompt/3`). For module-based prompts,\nthe framework automatically generates pattern-matched clauses during compilation.","ref":"Hermes.Server.html#c:handle_prompt_get/3"},{"type":"callback","title":"Hermes.Server.handle_request/2","doc":"Low-level handler for any MCP request.\n\nThis is an advanced callback that gives you complete control over request handling.\nWhen implemented, it bypasses the automatic routing to `handle_tool_call/3`,\n`handle_resource_read/2`, and `handle_prompt_get/3` and all other requests that are\nhandled internally, like `tools/list` and `logging/setLevel`.\n\nUse this when you need to:\n- Implement custom request methods beyond the standard MCP protocol\n- Add middleware-like processing before requests reach specific handlers\n- Override the framework's default request routing behavior\n\nNote: If you implement this callback, you become responsible for handling ALL\nMCP requests, including standard protocol methods like `tools/list`, `resources/list`, etc.\nConsider using the specific callbacks instead unless you need this level of control.","ref":"Hermes.Server.html#c:handle_request/2"},{"type":"callback","title":"Hermes.Server.handle_resource_read/2","doc":"Handles a resource read request.\n\nThis callback is invoked when a client requests to read a specific resource. It receives\nthe resource URI and the current frame. Developer's implementation should retrieve and return\nthe resource content.\n\nThis callback handles both module-based components (registered with `component`) and\nruntime components (registered with `Frame.register_resource/3`). For module-based resources,\nthe framework automatically generates pattern-matched clauses during compilation.","ref":"Hermes.Server.html#c:handle_resource_read/2"},{"type":"callback","title":"Hermes.Server.handle_tool_call/3","doc":"Handles a tool call request.\n\nThis callback is invoked when a client calls a specific tool. It receives the tool name,\nthe arguments provided by the client, and the current frame. Developers's implementation should\nexecute the tool's logic and return the result.\n\nThis callback handles both module-based components (registered with `component`) and\nruntime components (registered with `Frame.register_tool/3`). For module-based tools,\nthe framework automatically generates pattern-matched clauses during compilation.","ref":"Hermes.Server.html#c:handle_tool_call/3"},{"type":"callback","title":"Hermes.Server.init/2","doc":"Called after a client requests a `initialize` request.\n\nThis callback is invoked while the MCP handshake starts and so the client may not sent\nthe `notifications/initialized` message yet. For checking if the notification was already sent\nand the MCP handshare was successfully completed, you can call the `initialized?/1` function.\n\nIt receives the client's information and\nthe current frame, allowing you to perform client-specific setup, validate capabilities,\nor prepare resources based on the connected client.\n\nThe client_info parameter contains details about the connected client including its\nname, version, and any additional metadata. Use this to tailor your server's behavior\nto specific client implementations or versions.","ref":"Hermes.Server.html#c:init/2"},{"type":"function","title":"Hermes.Server.initialized?/1","doc":"Checks if the MCP session has been initialized.\n\nReturns true if the client has completed the initialization handshake and sent\nthe `notifications/initialized` message. This is useful for guarding operations\nthat require an active session.","ref":"Hermes.Server.html#initialized?/1"},{"type":"function","title":"Examples - Hermes.Server.initialized?/1","doc":"def handle_info(:check_status, frame) do\n      if Hermes.Server.initialized?(frame) do\n        # Perform operations requiring initialized session\n        {:noreply, frame}\n      else\n        # Wait for initialization\n        {:noreply, frame}\n      end\n    end","ref":"Hermes.Server.html#initialized?/1-examples"},{"type":"type","title":"Hermes.Server.mcp_error/0","doc":"","ref":"Hermes.Server.html#t:mcp_error/0"},{"type":"type","title":"Hermes.Server.notification/0","doc":"","ref":"Hermes.Server.html#t:notification/0"},{"type":"type","title":"Hermes.Server.request/0","doc":"","ref":"Hermes.Server.html#t:request/0"},{"type":"type","title":"Hermes.Server.response/0","doc":"","ref":"Hermes.Server.html#t:response/0"},{"type":"callback","title":"Hermes.Server.server_capabilities/0","doc":"Declares the server's capabilities during initialization.\n\nThis callback tells clients what features your server supports - which types of resources\nit can provide, what tools it can execute, whether it supports logging configuration, etc.\nThe capabilities you declare here directly impact which requests the client will send.\n\nWhen using `use Hermes.Server` with the `capabilities` option, this callback is automatically\nimplemented based on your configuration. The macro analyzes your registered components and\nbuilds the appropriate capability map, so you rarely need to implement this manually.","ref":"Hermes.Server.html#c:server_capabilities/0"},{"type":"type","title":"Hermes.Server.server_capabilities/0","doc":"","ref":"Hermes.Server.html#t:server_capabilities/0"},{"type":"callback","title":"Hermes.Server.server_info/0","doc":"Provides the server's identity information during initialization.\n\nThis callback is called during the MCP handshake to identify your server to connecting clients.\nThe information returned here helps clients understand which server they're talking to and\nensures version compatibility.\n\nWhen using `use Hermes.Server`, this callback is automatically implemented using the\n`name` and `version` options you provide. You only need to implement this manually if\nyou require dynamic server information based on runtime conditions.","ref":"Hermes.Server.html#c:server_info/0"},{"type":"type","title":"Hermes.Server.server_info/0","doc":"","ref":"Hermes.Server.html#t:server_info/0"},{"type":"callback","title":"Hermes.Server.supported_protocol_versions/0","doc":"Specifies which MCP protocol versions this server can speak.\n\nProtocol version negotiation ensures client and server can communicate effectively.\nDuring initialization, the client and server agree on a mutually supported version.\nThis callback returns the list of versions your server understands, typically in\norder of preference from newest to oldest.\n\nWhen using `use Hermes.Server`, this is automatically implemented with sensible defaults\ncovering current and recent protocol versions. Override only if you need to restrict\nor extend version support for specific compatibility requirements.","ref":"Hermes.Server.html#c:supported_protocol_versions/0"},{"type":"callback","title":"Hermes.Server.terminate/2","doc":"Cleans up when the server process terminates.\n\nThis optional callback is invoked when the server process is about to terminate.\nIt allows you to perform cleanup operations, close connections, save state,\nor release resources before the process exits.\n\nThe callback receives the termination reason and the current frame. Any return\nvalue is ignored. If not implemented, the Base module provides a default\nimplementation that logs the termination event.","ref":"Hermes.Server.html#c:terminate/2"},{"type":"module","title":"Hermes.Server.Component","doc":"High-level API for defining MCP server components (tools, prompts, resources).\n\nThis module provides a simplified way to define server components with automatic\nschema validation using Peri, reducing boilerplate while maintaining type safety.","ref":"Hermes.Server.Component.html"},{"type":"module","title":"Component Types - Hermes.Server.Component","doc":"- **Tools**: Functions that can be invoked by clients with parameters\n- **Prompts**: Templates that generate messages based on arguments\n- **Resources**: Data providers identified by URIs","ref":"Hermes.Server.Component.html#module-component-types"},{"type":"module","title":"Usage - Hermes.Server.Component","doc":"","ref":"Hermes.Server.Component.html#module-usage"},{"type":"module","title":"Tool Example - Hermes.Server.Component","doc":"defmodule MyServer.Tools.Calculator do\n      @moduledoc \"Performs arithmetic operations on two numbers\"\n      \n      use Hermes.Server.Component, type: :tool\n      \n      schema %{\n        operation: {:required, {:enum, [\"add\", \"subtract\", \"multiply\", \"divide\"]}},\n        a: {:required, :float},\n        b: {:required, :float}\n      }\n      \n      @impl true\n      def execute(%{operation: \"add\", a: a, b: b}, frame) do\n        {:ok, a + b, frame}\n      end\n      \n      @impl true\n      def execute(%{operation: \"divide\", a: a, b: 0}, frame) do\n        {:error, \"Cannot divide by zero\"}\n      end\n    end","ref":"Hermes.Server.Component.html#module-tool-example"},{"type":"module","title":"Prompt Example - Hermes.Server.Component","doc":"defmodule MyServer.Prompts.CodeReview do\n      @moduledoc \"Generate code review prompts for various programming languages\"\n      \n      use Hermes.Server.Component, type: :prompt\n      \n      schema %{\n        language: {:required, :string},\n        code: {:required, :string},\n        focus_areas: {:string, {:default, \"general quality\"}}\n      }\n      \n      @impl true\n      def get_messages(%{language: lang, code: code, focus_areas: focus}, frame) do\n        messages = [\n          %{\n            \"role\" => \"user\",\n            \"content\" => %{\n              \"type\" => \"text\",\n              \"text\" => \"Review this #{lang} code focusing on #{focus}...\"\n            }\n          }\n        ]\n        {:ok, messages, frame}\n      end\n    end","ref":"Hermes.Server.Component.html#module-prompt-example"},{"type":"module","title":"Resource Example - Hermes.Server.Component","doc":"defmodule MyServer.Resources.Config do\n      @moduledoc \"Application configuration file\"\n      \n      use Hermes.Server.Component, \n        type: :resource,\n        uri: \"file:///config/app.json\",\n        mime_type: \"application/json\"\n      \n      @impl true\n      def read(_params, frame) do\n        case File.read(\"config/app.json\") do\n          {:ok, content} -> {:ok, content, frame}\n          {:error, reason} -> {:error, \"Failed to read config: #{inspect(reason)}\"}\n        end\n      end\n    end\n    \n    # Resource with query parameters\n    defmodule MyServer.Resources.UserData do\n      @moduledoc \"User data with filtering support\"\n      \n      use Hermes.Server.Component,\n        type: :resource,\n        uri: \"users://data\"\n      \n      schema %{\n        user_id: {:required, :string},\n        include_metadata: {:boolean, {:default, false}}\n      }\n      \n      @impl true\n      def read(%{user_id: id, include_metadata: meta?}, frame) do\n        data = fetch_user_data(id, meta?)\n        {:ok, Jason.encode!(data), frame}\n      end\n    end","ref":"Hermes.Server.Component.html#module-resource-example"},{"type":"module","title":"Schema Validation - Hermes.Server.Component","doc":"Components can define parameter schemas using the `schema/1` macro, which leverages\nPeri for validation. The schema is automatically:\n- Validated before callback execution\n- Converted to JSON Schema for the MCP protocol\n- Used to generate helpful error messages","ref":"Hermes.Server.Component.html#module-schema-validation"},{"type":"module","title":"Automatic Features - Hermes.Server.Component","doc":"- **Description**: Extracted from `@moduledoc`\n- **Validation**: Parameters/arguments validated before execution\n- **Error Handling**: Detailed validation errors with paths\n- **JSON Schema**: Automatic conversion for protocol compliance","ref":"Hermes.Server.Component.html#module-automatic-features"},{"type":"function","title":"Hermes.Server.Component.component?/1","doc":"Checks if a module is a valid component.","ref":"Hermes.Server.Component.html#component?/1"},{"type":"function","title":"Parameters - Hermes.Server.Component.component?/1","doc":"* `module` - The module atom to check","ref":"Hermes.Server.Component.html#component?/1-parameters"},{"type":"function","title":"Returns - Hermes.Server.Component.component?/1","doc":"* `true` if the module uses `Hermes.Server.Component`\n  * `false` otherwise","ref":"Hermes.Server.Component.html#component?/1-returns"},{"type":"function","title":"Examples - Hermes.Server.Component.component?/1","doc":"iex> defmodule MyTool do\n    ...>   use Hermes.Server.Component, type: :tool\n    ...> end\n    iex> Hermes.Server.Component.component?(MyTool)\n    true\n\n    iex> defmodule NotAComponent do\n    ...>   def hello, do: :world\n    ...> end\n    iex> Hermes.Server.Component.component?(NotAComponent)\n    false","ref":"Hermes.Server.Component.html#component?/1-examples"},{"type":"macro","title":"Hermes.Server.Component.field/3","doc":"Defines a field with metadata for JSON Schema generation.\n\nSupports both simple fields and nested objects with their own fields.","ref":"Hermes.Server.Component.html#field/3"},{"type":"macro","title":"Examples - Hermes.Server.Component.field/3","doc":"# Simple field\n    field :email, {:required, :string}, format: \"email\", description: \"User's email address\"\n    field :age, :integer, description: \"Age in years\"\n    \n    # Nested field\n    field :user do\n      field :name, {:required, :string}\n      field :email, :string, format: \"email\"\n    end\n\n    # Nested field with metadata\n    field :profile, description: \"User profile information\" do\n      field :bio, :string, description: \"Short biography\"\n      field :avatar_url, :string, format: \"uri\"\n    end","ref":"Hermes.Server.Component.html#field/3-examples"},{"type":"function","title":"Hermes.Server.Component.get_description/1","doc":"Extracts the description from a component module's moduledoc.","ref":"Hermes.Server.Component.html#get_description/1"},{"type":"function","title":"Parameters - Hermes.Server.Component.get_description/1","doc":"* `module` - The component module atom","ref":"Hermes.Server.Component.html#get_description/1-parameters"},{"type":"function","title":"Returns - Hermes.Server.Component.get_description/1","doc":"* The module's `@moduledoc` content as a string\n  * Empty string if no moduledoc is defined","ref":"Hermes.Server.Component.html#get_description/1-returns"},{"type":"function","title":"Examples - Hermes.Server.Component.get_description/1","doc":"iex> defmodule MyTool do\n    ...>   @moduledoc \"A helpful tool\"\n    ...>   use Hermes.Server.Component, type: :tool\n    ...> end\n    iex> Hermes.Server.Component.get_description(MyTool)\n    \"A helpful tool\"","ref":"Hermes.Server.Component.html#get_description/1-examples"},{"type":"function","title":"Hermes.Server.Component.get_type/1","doc":"Gets the component type (:tool, :prompt, or :resource).","ref":"Hermes.Server.Component.html#get_type/1"},{"type":"function","title":"Parameters - Hermes.Server.Component.get_type/1","doc":"* `module` - The component module atom","ref":"Hermes.Server.Component.html#get_type/1-parameters"},{"type":"function","title":"Returns - Hermes.Server.Component.get_type/1","doc":"* `:tool` - If the module is a tool component\n  * `:prompt` - If the module is a prompt component\n  * `:resource` - If the module is a resource component","ref":"Hermes.Server.Component.html#get_type/1-returns"},{"type":"function","title":"Examples - Hermes.Server.Component.get_type/1","doc":"iex> defmodule MyTool do\n    ...>   use Hermes.Server.Component, type: :tool\n    ...> end\n    iex> Hermes.Server.Component.get_type(MyTool)\n    :tool","ref":"Hermes.Server.Component.html#get_type/1-examples"},{"type":"macro","title":"Hermes.Server.Component.schema/1","doc":"Defines the parameter schema for the component.\n\nThe schema uses Peri's validation DSL and is automatically validated\nbefore the component's callback is executed.","ref":"Hermes.Server.Component.html#schema/1"},{"type":"macro","title":"Examples - Hermes.Server.Component.schema/1","doc":"schema do\n      %{\n        query: {:required, :string},\n        limit: {:integer, {:default, 10}},\n        filters: %{\n          status: {:enum, [\"active\", \"inactive\", \"pending\"]},\n          created_after: :datetime\n        }\n      }\n    end\n\n    # With field metadata for JSON Schema (no braces needed!)\n    schema do\n      field(:email, {:required, :string}, format: \"email\", description: \"User's email address\")\n      field(:age, :integer, description: \"Age in years\")\n      field :address, description: \"User's address\" do\n        field(:street, {:required, :string})\n        field(:city, :string)\n        field(:country, :string, description: \"ISO 3166-1 alpha-2 code\")\n      end\n    end","ref":"Hermes.Server.Component.html#schema/1-examples"},{"type":"behaviour","title":"Hermes.Server.Component.Prompt","doc":"Defines the behaviour for MCP prompts.\n\nPrompts are reusable templates that generate messages based on provided arguments.\nThey help standardize common interactions and can be customized with parameters.","ref":"Hermes.Server.Component.Prompt.html"},{"type":"behaviour","title":"Example - Hermes.Server.Component.Prompt","doc":"defmodule MyServer.Prompts.CodeReview do\n      @behaviour Hermes.Server.Behaviour.Prompt\n      \n      alias Hermes.Server.Frame\n      \n      @impl true\n      def name, do: \"code_review\"\n      \n      @impl true\n      def description do\n        \"Generate a code review prompt for the given programming language and code\"\n      end\n      \n      @impl true\n      def arguments do\n        [\n          %{\n            \"name\" => \"language\",\n            \"description\" => \"The programming language of the code\",\n            \"required\" => true\n          },\n          %{\n            \"name\" => \"code\",\n            \"description\" => \"The code to review\",\n            \"required\" => true\n          },\n          %{\n            \"name\" => \"focus_areas\",\n            \"description\" => \"Specific areas to focus on (e.g., performance, security)\",\n            \"required\" => false\n          }\n        ]\n      end\n      \n      @impl true\n      def get_messages(%{\"language\" => lang, \"code\" => code} = args, frame) do\n        focus = Map.get(args, \"focus_areas\", \"general quality\")\n        \n        messages = [\n          %{\n            \"role\" => \"user\",\n            \"content\" => %{\n              \"type\" => \"text\",\n              \"text\" => \"\"\"\n              Please review the following #{lang} code, focusing on #{focus}:\n              \n              ```#{lang}\n              #{code}\n              ```\n              \n              Provide constructive feedback on:\n              1. Code quality and readability\n              2. Potential bugs or issues\n              3. Performance considerations\n              4. Best practices for #{lang}\n              \"\"\"\n            }\n          }\n        ]\n        \n        # Can track prompt usage\n        new_frame = Frame.assign(frame, :last_prompt_used, \"code_review\")\n        \n        {:ok, messages, new_frame}\n      end\n    end","ref":"Hermes.Server.Component.Prompt.html#module-example"},{"type":"type","title":"Hermes.Server.Component.Prompt.argument_def/0","doc":"","ref":"Hermes.Server.Component.Prompt.html#t:argument_def/0"},{"type":"callback","title":"Hermes.Server.Component.Prompt.arguments/0","doc":"Returns the list of arguments this prompt accepts.\n\nEach argument should include:\n- `\"name\"` - The argument name\n- `\"description\"` - What the argument is for\n- `\"required\"` - Whether the argument is required (optional, defaults to false)","ref":"Hermes.Server.Component.Prompt.html#c:arguments/0"},{"type":"callback","title":"Example - Hermes.Server.Component.Prompt.arguments/0","doc":"[\n      %{\n        \"name\" => \"topic\",\n        \"description\" => \"The topic to generate content about\",\n        \"required\" => true\n      },\n      %{\n        \"name\" => \"tone\",\n        \"description\" => \"The tone of voice (formal, casual, etc.)\",\n        \"required\" => false\n      }\n    ]","ref":"Hermes.Server.Component.Prompt.html#c:arguments/0-example"},{"type":"type","title":"Hermes.Server.Component.Prompt.arguments/0","doc":"","ref":"Hermes.Server.Component.Prompt.html#t:arguments/0"},{"type":"callback","title":"Hermes.Server.Component.Prompt.get_messages/2","doc":"Generates messages based on the provided arguments.","ref":"Hermes.Server.Component.Prompt.html#c:get_messages/2"},{"type":"callback","title":"Parameters - Hermes.Server.Component.Prompt.get_messages/2","doc":"- `args` - The arguments provided by the client\n- `frame` - The server frame containing context and state","ref":"Hermes.Server.Component.Prompt.html#c:get_messages/2-parameters"},{"type":"callback","title":"Return Values - Hermes.Server.Component.Prompt.get_messages/2","doc":"- `{:ok, messages}` - Messages generated successfully, frame unchanged\n- `{:ok, messages, new_frame}` - Messages generated with frame updates\n- `{:error, reason}` - Failed to generate messages","ref":"Hermes.Server.Component.Prompt.html#c:get_messages/2-return-values"},{"type":"callback","title":"Message Format - Hermes.Server.Component.Prompt.get_messages/2","doc":"Messages should follow the MCP message format:\n\n    %{\n      \"role\" => \"user\" | \"assistant\",\n      \"content\" => %{\n        \"type\" => \"text\",\n        \"text\" => \"The message content\"\n      }\n    }\n\nMultiple messages can be returned to create a conversation context.","ref":"Hermes.Server.Component.Prompt.html#c:get_messages/2-message-format"},{"type":"type","title":"Hermes.Server.Component.Prompt.message/0","doc":"","ref":"Hermes.Server.Component.Prompt.html#t:message/0"},{"type":"type","title":"Hermes.Server.Component.Prompt.t/0","doc":"","ref":"Hermes.Server.Component.Prompt.html#t:t/0"},{"type":"behaviour","title":"Hermes.Server.Component.Resource","doc":"Defines the behaviour for MCP resources.\n\nResources represent data that can be read by the client, such as files,\ndocuments, or any other content. Each resource is identified by a URI\nand can provide content in various formats.","ref":"Hermes.Server.Component.Resource.html"},{"type":"behaviour","title":"Example - Hermes.Server.Component.Resource","doc":"defmodule MyServer.Resources.Documentation do\n      @behaviour Hermes.Server.Behaviour.Resource\n      \n      alias Hermes.Server.Frame\n      \n      @impl true\n      def uri, do: \"file:///docs/readme.md\"\n      \n      @impl true\n      def name, do: \"Project README\"\n      \n      @impl true\n      def description, do: \"The main documentation for this project\"\n      \n      @impl true\n      def mime_type, do: \"text/markdown\"\n      \n      @impl true\n      def read(_params, frame) do\n        case File.read(\"README.md\") do\n          {:ok, content} -> \n            # Can track access in frame\n            new_frame = Frame.assign(frame, :last_resource_access, DateTime.utc_now())\n            {:ok, content, new_frame}\n            \n          {:error, reason} -> \n            {:error, \"Failed to read README: #{inspect(reason)}\"}\n        end\n      end\n    end","ref":"Hermes.Server.Component.Resource.html#module-example"},{"type":"behaviour","title":"Example with dynamic content - Hermes.Server.Component.Resource","doc":"defmodule MyServer.Resources.SystemStatus do\n      @behaviour Hermes.Server.Behaviour.Resource\n      \n      @impl true\n      def uri, do: \"system://status\"\n      \n      @impl true\n      def name, do: \"System Status\"\n      \n      @impl true\n      def description, do: \"Current system status and metrics\"\n      \n      @impl true\n      def mime_type, do: \"application/json\"\n      \n      @impl true\n      def read(_params, frame) do\n        status = %{\n          uptime: System.uptime(),\n          memory: :erlang.memory(),\n          user_id: frame.assigns[:user_id],\n          timestamp: DateTime.utc_now()\n        }\n        \n        {:ok, Jason.encode!(status), frame}\n      end\n    end","ref":"Hermes.Server.Component.Resource.html#module-example-with-dynamic-content"},{"type":"type","title":"Hermes.Server.Component.Resource.content/0","doc":"","ref":"Hermes.Server.Component.Resource.html#t:content/0"},{"type":"callback","title":"Hermes.Server.Component.Resource.mime_type/0","doc":"Returns the MIME type of the resource content.\n\nCommon MIME types:\n- `text/plain` for plain text\n- `text/markdown` for Markdown\n- `application/json` for JSON data\n- `text/html` for HTML\n- `application/octet-stream` for binary data","ref":"Hermes.Server.Component.Resource.html#c:mime_type/0"},{"type":"type","title":"Hermes.Server.Component.Resource.params/0","doc":"","ref":"Hermes.Server.Component.Resource.html#t:params/0"},{"type":"callback","title":"Hermes.Server.Component.Resource.read/2","doc":"Reads the resource content.","ref":"Hermes.Server.Component.Resource.html#c:read/2"},{"type":"callback","title":"Parameters - Hermes.Server.Component.Resource.read/2","doc":"- `params` - Optional parameters from the client (typically empty for resources)\n- `frame` - The server frame containing context and state","ref":"Hermes.Server.Component.Resource.html#c:read/2-parameters"},{"type":"callback","title":"Return Values - Hermes.Server.Component.Resource.read/2","doc":"- `{:ok, content}` - Resource read successfully, frame unchanged\n- `{:ok, content, new_frame}` - Resource read successfully with frame updates\n- `{:error, reason}` - Failed to read resource","ref":"Hermes.Server.Component.Resource.html#c:read/2-return-values"},{"type":"callback","title":"Content Types - Hermes.Server.Component.Resource.read/2","doc":"The content should match the declared MIME type:\n- For text types, return a String\n- For binary types, return binary data\n- For JSON, return the JSON-encoded string","ref":"Hermes.Server.Component.Resource.html#c:read/2-content-types"},{"type":"type","title":"Hermes.Server.Component.Resource.t/0","doc":"","ref":"Hermes.Server.Component.Resource.html#t:t/0"},{"type":"callback","title":"Hermes.Server.Component.Resource.uri/0","doc":"Returns the URI that identifies this resource.\n\nThe URI should be unique within the server and follow standard URI conventions.\nCommon schemes include:\n- `file://` for file-based resources\n- `http://` or `https://` for web resources\n- Custom schemes for application-specific resources","ref":"Hermes.Server.Component.Resource.html#c:uri/0"},{"type":"behaviour","title":"Hermes.Server.Component.Tool","doc":"Defines the behaviour for MCP tools.\n\nTools are functions that can be invoked by the client with specific parameters.\nEach tool must define its name, description, and parameter schema, as well as\nimplement the execution logic.","ref":"Hermes.Server.Component.Tool.html"},{"type":"behaviour","title":"Example - Hermes.Server.Component.Tool","doc":"defmodule MyServer.Tools.Calculator do\n      @behaviour Hermes.Server.Behaviour.Tool\n      \n      alias Hermes.Server.Frame\n      \n      @impl true\n      def name, do: \"calculator\"\n      \n      @impl true\n      def description, do: \"Performs basic arithmetic operations\"\n      \n      @impl true\n      def input_schema do\n        %{\n          \"type\" => \"object\",\n          \"properties\" => %{\n            \"operation\" => %{\n              \"type\" => \"string\",\n              \"enum\" => [\"add\", \"subtract\", \"multiply\", \"divide\"]\n            },\n            \"a\" => %{\"type\" => \"number\"},\n            \"b\" => %{\"type\" => \"number\"}\n          },\n          \"required\" => [\"operation\", \"a\", \"b\"]\n        }\n      end\n      \n      @impl true\n      def execute(%{\"operation\" => \"add\", \"a\" => a, \"b\" => b}, frame) do\n        result = a + b\n        \n        # Can access frame assigns\n        user_id = frame.assigns[:user_id]\n        \n        # Can return updated frame if needed\n        new_frame = Frame.assign(frame, :last_calculation, result)\n        \n        {:ok, result, new_frame}\n      end\n      \n      @impl true\n      def execute(%{\"operation\" => \"divide\", \"a\" => a, \"b\" => 0}, _frame) do\n        {:error, \"Cannot divide by zero\"}\n      end\n    end","ref":"Hermes.Server.Component.Tool.html#module-example"},{"type":"callback","title":"Hermes.Server.Component.Tool.annotations/0","doc":"Returns optional annotations for the tool.\n\nAnnotations provide additional metadata about the tool that may be used\nby clients for enhanced functionality. This is an optional callback.","ref":"Hermes.Server.Component.Tool.html#c:annotations/0"},{"type":"callback","title":"Examples - Hermes.Server.Component.Tool.annotations/0","doc":"def annotations do\n      %{\n        \"confidence\" => 0.95,\n        \"category\" => \"text-processing\",\n        \"tags\" => [\"nlp\", \"text\"]\n      }\n    end","ref":"Hermes.Server.Component.Tool.html#c:annotations/0-examples"},{"type":"type","title":"Hermes.Server.Component.Tool.annotations/0","doc":"","ref":"Hermes.Server.Component.Tool.html#t:annotations/0"},{"type":"callback","title":"Hermes.Server.Component.Tool.execute/2","doc":"Executes the tool with the given parameters.","ref":"Hermes.Server.Component.Tool.html#c:execute/2"},{"type":"callback","title":"Parameters - Hermes.Server.Component.Tool.execute/2","doc":"- `params` - The validated input parameters from the client\n- `frame` - The server frame containing:\n  - `assigns` - Custom data like session_id, client_info, user permissions\n  - `initialized` - Whether the server has been initialized","ref":"Hermes.Server.Component.Tool.html#c:execute/2-parameters"},{"type":"callback","title":"Return Values - Hermes.Server.Component.Tool.execute/2","doc":"- `{:ok, result}` - Tool executed successfully, frame unchanged\n- `{:ok, result, new_frame}` - Tool executed successfully with frame updates\n- `{:error, reason}` - Tool failed with the given reason","ref":"Hermes.Server.Component.Tool.html#c:execute/2-return-values"},{"type":"callback","title":"Frame Usage - Hermes.Server.Component.Tool.execute/2","doc":"The frame provides access to server state and context:\n\n    def execute(params, frame) do\n      # Access assigns\n      user_id = frame.assigns[:user_id]\n      permissions = frame.assigns[:permissions]\n      \n      # Update frame if needed\n      new_frame = Frame.assign(frame, :last_tool_call, DateTime.utc_now())\n      \n      {:ok, \"Result\", new_frame}\n    end","ref":"Hermes.Server.Component.Tool.html#c:execute/2-frame-usage"},{"type":"callback","title":"Hermes.Server.Component.Tool.input_schema/0","doc":"Returns the JSON Schema for the tool's input parameters.\n\nThis schema is used to validate client requests and generate documentation.\nThe schema should follow the JSON Schema specification.","ref":"Hermes.Server.Component.Tool.html#c:input_schema/0"},{"type":"type","title":"Hermes.Server.Component.Tool.params/0","doc":"","ref":"Hermes.Server.Component.Tool.html#t:params/0"},{"type":"type","title":"Hermes.Server.Component.Tool.result/0","doc":"","ref":"Hermes.Server.Component.Tool.html#t:result/0"},{"type":"type","title":"Hermes.Server.Component.Tool.schema/0","doc":"","ref":"Hermes.Server.Component.Tool.html#t:schema/0"},{"type":"type","title":"Hermes.Server.Component.Tool.t/0","doc":"","ref":"Hermes.Server.Component.Tool.html#t:t/0"},{"type":"module","title":"Hermes.Server.Frame","doc":"The Hermes Frame.\n\nThis module defines a struct and functions for working with\nMCP server state throughout the request/response lifecycle.","ref":"Hermes.Server.Frame.html"},{"type":"module","title":"User fields - Hermes.Server.Frame","doc":"These fields contain user-controlled data:\n\n  * `assigns` - shared user data as a map. For HTTP transports, this inherits\n    from `Plug.Conn.assigns`. Users are responsible for populating authentication\n    data through their Plug pipeline before it reaches the MCP server.","ref":"Hermes.Server.Frame.html#module-user-fields"},{"type":"module","title":"Transport fields - Hermes.Server.Frame","doc":"These fields contain transport-specific context. The structure varies by transport type:","ref":"Hermes.Server.Frame.html#module-transport-fields"},{"type":"module","title":"HTTP transport (when transport.type == :http) - Hermes.Server.Frame","doc":"* `req_headers` - the request headers as a list, example: `[{\"content-type\", \"application/json\"}]`.\n    All header names are downcased.\n  * `query_params` - the request query params as a map, example: `%{\"session\" => \"abc123\"}`.\n    Returns `nil` if query params were not fetched by the Plug pipeline.\n  * `remote_ip` - the IP of the client, example: `{151, 236, 219, 228}`. \n    This field is set by the transport layer.\n  * `scheme` - the request scheme as an atom, example: `:https`\n  * `host` - the requested host as a binary, example: `\"api.example.com\"`\n  * `port` - the requested port as an integer, example: `443`\n  * `request_path` - the requested path, example: `\"/mcp\"`","ref":"Hermes.Server.Frame.html#module-http-transport-when-transport-type-http"},{"type":"module","title":"STDIO transport (when transport.type == :stdio) - Hermes.Server.Frame","doc":"* `env` - environment variables as a map, example: `%{\"USER\" => \"alice\", \"HOME\" => \"/home/alice\"}`\n  * `pid` - the OS process ID as a string, example: `\"12345\"`","ref":"Hermes.Server.Frame.html#module-stdio-transport-when-transport-type-stdio"},{"type":"module","title":"MCP protocol fields - Hermes.Server.Frame","doc":"These fields contain MCP-specific data:\n\n  * `request` - the current MCP request being processed, with fields:\n    * `id` - the request ID for correlation\n    * `method` - the MCP method being called, example: `\"tools/call\"`\n    * `params` - the raw request parameters (before validation)\n  * `initialized` - boolean indicating if the MCP session has been initialized","ref":"Hermes.Server.Frame.html#module-mcp-protocol-fields"},{"type":"module","title":"Private fields - Hermes.Server.Frame","doc":"These fields are reserved for framework usage:\n\n  * `private` - shared framework data as a map. Contains MCP session context:\n    * `session_id` - unique identifier for the client session\n    * `client_info` - client information from initialization, example: `%{\"name\" => \"my-client\", \"version\" => \"1.0.0\"}`\n    * `client_capabilities` - negotiated client capabilities\n    * `protocol_version` - active MCP protocol version, example: `\"2025-03-26\"`","ref":"Hermes.Server.Frame.html#module-private-fields"},{"type":"function","title":"Hermes.Server.Frame.assign/2","doc":"Assigns a value or multiple values to the frame.","ref":"Hermes.Server.Frame.html#assign/2"},{"type":"function","title":"Examples - Hermes.Server.Frame.assign/2","doc":"# Single assignment\n    frame = Frame.assign(frame, :status, :active)\n\n    # Multiple assignments via map\n    frame = Frame.assign(frame, %{status: :active, count: 5})\n\n    # Multiple assignments via keyword list\n    frame = Frame.assign(frame, status: :active, count: 5)","ref":"Hermes.Server.Frame.html#assign/2-examples"},{"type":"function","title":"Hermes.Server.Frame.assign/3","doc":"","ref":"Hermes.Server.Frame.html#assign/3"},{"type":"function","title":"Hermes.Server.Frame.assign_new/3","doc":"Assigns a value to the frame only if the key doesn't already exist.\n\nThe value is computed lazily using the provided function, which is only\ncalled if the key is not present in assigns.","ref":"Hermes.Server.Frame.html#assign_new/3"},{"type":"function","title":"Examples - Hermes.Server.Frame.assign_new/3","doc":"# Only assigns if :timestamp doesn't exist\n    frame = Frame.assign_new(frame, :timestamp, fn -> DateTime.utc_now() end)\n\n    # Function is not called if key exists\n    frame = frame |> Frame.assign(:count, 5)\n                  |> Frame.assign_new(:count, fn -> expensive_computation() end)\n    # count remains 5","ref":"Hermes.Server.Frame.html#assign_new/3-examples"},{"type":"function","title":"Hermes.Server.Frame.clear_request/1","doc":"Clears the current request from the frame.\n\nThis should be called after processing a request to ensure the frame doesn't\nretain stale request data.","ref":"Hermes.Server.Frame.html#clear_request/1"},{"type":"function","title":"Examples - Hermes.Server.Frame.clear_request/1","doc":"frame = Frame.clear_request(frame)","ref":"Hermes.Server.Frame.html#clear_request/1-examples"},{"type":"function","title":"Hermes.Server.Frame.clear_session/1","doc":"Clears all session-specific private data from the frame.\n\nThis should be called when a session ends to ensure the frame doesn't\nretain stale session data.","ref":"Hermes.Server.Frame.html#clear_session/1"},{"type":"function","title":"Examples - Hermes.Server.Frame.clear_session/1","doc":"frame = Frame.clear_session(frame)","ref":"Hermes.Server.Frame.html#clear_session/1-examples"},{"type":"function","title":"Hermes.Server.Frame.get_client_capabilities/1","doc":"Gets the client capabilities from the frame's private data.","ref":"Hermes.Server.Frame.html#get_client_capabilities/1"},{"type":"function","title":"Examples - Hermes.Server.Frame.get_client_capabilities/1","doc":"capabilities = Frame.get_client_capabilities(frame)\n    # => %{\"tools\" => %{}, \"resources\" => %{}}","ref":"Hermes.Server.Frame.html#get_client_capabilities/1-examples"},{"type":"function","title":"Hermes.Server.Frame.get_client_info/1","doc":"Gets the client info from the frame's private data.","ref":"Hermes.Server.Frame.html#get_client_info/1"},{"type":"function","title":"Examples - Hermes.Server.Frame.get_client_info/1","doc":"client_info = Frame.get_client_info(frame)\n    # => %{\"name\" => \"my-client\", \"version\" => \"1.0.0\"}","ref":"Hermes.Server.Frame.html#get_client_info/1-examples"},{"type":"function","title":"Hermes.Server.Frame.get_mcp_session_id/1","doc":"Gets the MCP session ID from the frame's private data.","ref":"Hermes.Server.Frame.html#get_mcp_session_id/1"},{"type":"function","title":"Examples - Hermes.Server.Frame.get_mcp_session_id/1","doc":"session_id = Frame.get_mcp_session_id(frame)\n    # => \"session_abc123\"","ref":"Hermes.Server.Frame.html#get_mcp_session_id/1-examples"},{"type":"function","title":"Hermes.Server.Frame.get_protocol_version/1","doc":"Gets the protocol version from the frame's private data.","ref":"Hermes.Server.Frame.html#get_protocol_version/1"},{"type":"function","title":"Examples - Hermes.Server.Frame.get_protocol_version/1","doc":"version = Frame.get_protocol_version(frame)\n    # => \"2025-03-26\"","ref":"Hermes.Server.Frame.html#get_protocol_version/1-examples"},{"type":"function","title":"Hermes.Server.Frame.get_query_param/2","doc":"Gets a query parameter value from HTTP transport.\n\nReturns the parameter value, or nil if the transport is not HTTP,\nquery params weren't fetched, or the parameter doesn't exist.","ref":"Hermes.Server.Frame.html#get_query_param/2"},{"type":"function","title":"Examples - Hermes.Server.Frame.get_query_param/2","doc":"# HTTP transport with query params\n    session = Frame.get_query_param(frame, \"session\")\n    # => \"abc123\"\n\n    # Missing parameter or non-HTTP transport\n    missing = Frame.get_query_param(frame, \"nonexistent\")\n    # => nil","ref":"Hermes.Server.Frame.html#get_query_param/2-examples"},{"type":"function","title":"Hermes.Server.Frame.get_req_header/2","doc":"Gets a request header value from HTTP transport.\n\nReturns the first value for the header, or nil if the transport\nis not HTTP or the header is not present.","ref":"Hermes.Server.Frame.html#get_req_header/2"},{"type":"function","title":"Examples - Hermes.Server.Frame.get_req_header/2","doc":"# HTTP transport\n    auth_header = Frame.get_req_header(frame, \"authorization\")\n    # => \"Bearer token123\"\n\n    # Non-HTTP transport or missing header\n    auth_header = Frame.get_req_header(frame, \"authorization\")\n    # => nil","ref":"Hermes.Server.Frame.html#get_req_header/2-examples"},{"type":"type","title":"Hermes.Server.Frame.http_t/0","doc":"","ref":"Hermes.Server.Frame.html#t:http_t/0"},{"type":"function","title":"Hermes.Server.Frame.new/1","doc":"Creates a new frame with optional initial assigns.","ref":"Hermes.Server.Frame.html#new/1"},{"type":"function","title":"Examples - Hermes.Server.Frame.new/1","doc":"iex> Frame.new()\n    %Frame{assigns: %{}, initialized: false}\n\n    iex> Frame.new(%{user: \"alice\"})\n    %Frame{assigns: %{user: \"alice\"}, initialized: false}","ref":"Hermes.Server.Frame.html#new/1-examples"},{"type":"type","title":"Hermes.Server.Frame.private_t/0","doc":"","ref":"Hermes.Server.Frame.html#t:private_t/0"},{"type":"function","title":"Hermes.Server.Frame.put_private/2","doc":"","ref":"Hermes.Server.Frame.html#put_private/2"},{"type":"function","title":"Hermes.Server.Frame.put_private/3","doc":"Sets or updates private session data in the frame.\n\nPrivate data is used for framework-internal session context that persists\nacross requests, similar to Plug.Conn.private.","ref":"Hermes.Server.Frame.html#put_private/3"},{"type":"function","title":"Examples - Hermes.Server.Frame.put_private/3","doc":"# Set single private value\n    frame = Frame.put_private(frame, :session_id, \"abc123\")\n\n    # Set multiple private values\n    frame = Frame.put_private(frame, %{\n      session_id: \"abc123\",\n      client_info: %{name: \"my-client\", version: \"1.0.0\"}\n    })","ref":"Hermes.Server.Frame.html#put_private/3-examples"},{"type":"function","title":"Hermes.Server.Frame.put_request/2","doc":"Sets the current request being processed.\n\nThe request includes the request ID, method, and raw parameters before validation.","ref":"Hermes.Server.Frame.html#put_request/2"},{"type":"function","title":"Examples - Hermes.Server.Frame.put_request/2","doc":"frame = Frame.put_request(frame, %{\n      id: \"req_123\",\n      method: \"tools/call\",\n      params: %{\"name\" => \"calculator\", \"arguments\" => %{}}\n    })","ref":"Hermes.Server.Frame.html#put_request/2-examples"},{"type":"function","title":"Hermes.Server.Frame.put_transport/2","doc":"","ref":"Hermes.Server.Frame.html#put_transport/2"},{"type":"function","title":"Hermes.Server.Frame.put_transport/3","doc":"Sets or updates transport data in the frame.\n\nCheck `transport_t()` for reference.","ref":"Hermes.Server.Frame.html#put_transport/3"},{"type":"function","title":"Examples - Hermes.Server.Frame.put_transport/3","doc":"# Set single transport value\n    frame = Frame.put_transport(frame, :session_id, \"abc123\")\n\n    # Set multiple transport values\n    frame = Frame.put_transport(frame, %{\n      session_id: \"abc123\",\n      client_info: %{name: \"my-client\", version: \"1.0.0\"}\n    })","ref":"Hermes.Server.Frame.html#put_transport/3-examples"},{"type":"function","title":"Hermes.Server.Frame.register_prompt/3","doc":"Registers a prompt definition.","ref":"Hermes.Server.Frame.html#register_prompt/3"},{"type":"function","title":"Hermes.Server.Frame.register_resource/3","doc":"Registers a resource definition.","ref":"Hermes.Server.Frame.html#register_resource/3"},{"type":"function","title":"Hermes.Server.Frame.register_tool/3","doc":"Registers a tool definition.","ref":"Hermes.Server.Frame.html#register_tool/3"},{"type":"type","title":"Hermes.Server.Frame.request_t/0","doc":"","ref":"Hermes.Server.Frame.html#t:request_t/0"},{"type":"type","title":"Hermes.Server.Frame.server_component_t/0","doc":"","ref":"Hermes.Server.Frame.html#t:server_component_t/0"},{"type":"type","title":"Hermes.Server.Frame.stdio_t/0","doc":"","ref":"Hermes.Server.Frame.html#t:stdio_t/0"},{"type":"type","title":"Hermes.Server.Frame.t/0","doc":"","ref":"Hermes.Server.Frame.html#t:t/0"},{"type":"type","title":"Hermes.Server.Frame.transport_t/0","doc":"","ref":"Hermes.Server.Frame.html#t:transport_t/0"},{"type":"module","title":"Hermes.Server.Registry","doc":"Registry for MCP server and transport processes.\n\nThis module provides a safe way to manage process names without creating\natoms dynamically at runtime. It uses a via tuple pattern with Registry.","ref":"Hermes.Server.Registry.html"},{"type":"module","title":"Usage - Hermes.Server.Registry","doc":"# Register a server process\n    {:ok, _pid} = GenServer.start_link(MyServer, arg, name: Registry.server(MyModule))\n    \n    # Register a transport process\n    {:ok, _pid} = GenServer.start_link(Transport, arg, name: Registry.transport(MyModule, :stdio))\n    \n    # Look up a process\n    GenServer.call(Registry.server(MyModule), :ping)","ref":"Hermes.Server.Registry.html#module-usage"},{"type":"function","title":"Hermes.Server.Registry.child_spec/1","doc":"","ref":"Hermes.Server.Registry.html#child_spec/1"},{"type":"function","title":"Hermes.Server.Registry.server/1","doc":"Returns a via tuple for naming a server process.","ref":"Hermes.Server.Registry.html#server/1"},{"type":"function","title":"Hermes.Server.Registry.server_session/2","doc":"Returns a via tuple for naming a server session process.","ref":"Hermes.Server.Registry.html#server_session/2"},{"type":"function","title":"Hermes.Server.Registry.supervisor/2","doc":"Returns a via tuple for naming a supervisor process.","ref":"Hermes.Server.Registry.html#supervisor/2"},{"type":"function","title":"Hermes.Server.Registry.transport/2","doc":"Returns a via tuple for naming a transport process.","ref":"Hermes.Server.Registry.html#transport/2"},{"type":"function","title":"Hermes.Server.Registry.whereis_server/1","doc":"Gets the PID of a registered server.","ref":"Hermes.Server.Registry.html#whereis_server/1"},{"type":"function","title":"Hermes.Server.Registry.whereis_server_session/2","doc":"Gets the PID of a session-specific server.","ref":"Hermes.Server.Registry.html#whereis_server_session/2"},{"type":"function","title":"Hermes.Server.Registry.whereis_supervisor/2","doc":"Gets the PID of a supervisor process.","ref":"Hermes.Server.Registry.html#whereis_supervisor/2"},{"type":"function","title":"Hermes.Server.Registry.whereis_transport/2","doc":"Gets the PID of a registered transport.","ref":"Hermes.Server.Registry.html#whereis_transport/2"},{"type":"behaviour","title":"Hermes.Server.Registry.Adapter","doc":"Behaviour for registry adapters in MCP servers.\n\nThis module defines the interface that registry implementations must follow\nto be pluggable into the Hermes MCP server architecture. It allows users\nto provide custom registry implementations (e.g., using Horde for cluster-wide\ndistribution) while maintaining compatibility with the existing API.","ref":"Hermes.Server.Registry.Adapter.html"},{"type":"behaviour","title":"Implementing a Custom Registry - Hermes.Server.Registry.Adapter","doc":"To implement a custom registry adapter, create a module that implements\nall the callbacks defined in this behaviour:\n\n    defmodule MyApp.HordeRegistry do\n      @behaviour Hermes.Server.Registry.Adapter\n\n      def child_spec(opts) do\n        %{\n          id: __MODULE__,\n          start: {Horde.Registry, :start_link, [\n            [\n              name: __MODULE__,\n              keys: :unique,\n              members: :auto\n            ] ++ opts\n          ]}\n        }\n      end\n\n      def server(module) do\n        {:via, Horde.Registry, {__MODULE__, {:server, module}}}\n      end\n\n      # ... implement other callbacks\n    end","ref":"Hermes.Server.Registry.Adapter.html#module-implementing-a-custom-registry"},{"type":"behaviour","title":"Using a Custom Registry - Hermes.Server.Registry.Adapter","doc":"You can configure a custom registry at multiple levels:\n\n  Hermes.Server.start_link(MyServer, :ok, transport: :stdio, registry: MyApp.HordeRegistry)","ref":"Hermes.Server.Registry.Adapter.html#module-using-a-custom-registry"},{"type":"behaviour","title":"Default Implementation - Hermes.Server.Registry.Adapter","doc":"The default implementation uses Elixir's built-in Registry module.\nSee `Hermes.Server.Registry` for the reference implementation.","ref":"Hermes.Server.Registry.Adapter.html#module-default-implementation"},{"type":"callback","title":"Hermes.Server.Registry.Adapter.child_spec/1","doc":"Returns a child specification for the registry.\n\nThis is used when starting the registry as part of a supervision tree.\nThe implementation should return a valid child specification map or tuple.","ref":"Hermes.Server.Registry.Adapter.html#c:child_spec/1"},{"type":"callback","title":"Hermes.Server.Registry.Adapter.server/1","doc":"Returns a name for a server process.\n\nThe returned value must be a valid GenServer name that can be passed\nto `GenServer.start_link/3` and similar functions.","ref":"Hermes.Server.Registry.Adapter.html#c:server/1"},{"type":"callback","title":"Parameters - Hermes.Server.Registry.Adapter.server/1","doc":"* `module` - The module implementing the server","ref":"Hermes.Server.Registry.Adapter.html#c:server/1-parameters"},{"type":"callback","title":"Hermes.Server.Registry.Adapter.server_session/2","doc":"Returns a name for a server session process.","ref":"Hermes.Server.Registry.Adapter.html#c:server_session/2"},{"type":"callback","title":"Parameters - Hermes.Server.Registry.Adapter.server_session/2","doc":"* `server_module` - The module implementing the server\n  * `session_id` - The unique session identifier","ref":"Hermes.Server.Registry.Adapter.html#c:server_session/2-parameters"},{"type":"callback","title":"Hermes.Server.Registry.Adapter.supervisor/2","doc":"Returns a name for a supervisor process.","ref":"Hermes.Server.Registry.Adapter.html#c:supervisor/2"},{"type":"callback","title":"Parameters - Hermes.Server.Registry.Adapter.supervisor/2","doc":"* `kind` - The kind of supervisor (e.g., :supervisor, :session_supervisor)\n  * `server_module` - The module implementing the server","ref":"Hermes.Server.Registry.Adapter.html#c:supervisor/2-parameters"},{"type":"callback","title":"Hermes.Server.Registry.Adapter.transport/2","doc":"Returns a name for a transport process.","ref":"Hermes.Server.Registry.Adapter.html#c:transport/2"},{"type":"callback","title":"Parameters - Hermes.Server.Registry.Adapter.transport/2","doc":"* `server_module` - The module implementing the server\n  * `transport_type` - The type of transport (e.g., :stdio, :sse, :websocket)","ref":"Hermes.Server.Registry.Adapter.html#c:transport/2-parameters"},{"type":"callback","title":"Hermes.Server.Registry.Adapter.whereis_server/1","doc":"Gets the PID of a registered server.\n\nReturns the PID if the server is registered, nil otherwise.","ref":"Hermes.Server.Registry.Adapter.html#c:whereis_server/1"},{"type":"callback","title":"Parameters - Hermes.Server.Registry.Adapter.whereis_server/1","doc":"* `server_module` - The module implementing the server","ref":"Hermes.Server.Registry.Adapter.html#c:whereis_server/1-parameters"},{"type":"callback","title":"Hermes.Server.Registry.Adapter.whereis_server_session/2","doc":"Gets the PID of a server session process.\n\nReturns the PID if the session is registered, nil otherwise.","ref":"Hermes.Server.Registry.Adapter.html#c:whereis_server_session/2"},{"type":"callback","title":"Parameters - Hermes.Server.Registry.Adapter.whereis_server_session/2","doc":"* `server_module` - The module implementing the server\n  * `session_id` - The unique session identifier","ref":"Hermes.Server.Registry.Adapter.html#c:whereis_server_session/2-parameters"},{"type":"callback","title":"Hermes.Server.Registry.Adapter.whereis_supervisor/2","doc":"Gets the PID of a supervisor process.\n\nReturns the PID if the supervisor is registered, nil otherwise.","ref":"Hermes.Server.Registry.Adapter.html#c:whereis_supervisor/2"},{"type":"callback","title":"Parameters - Hermes.Server.Registry.Adapter.whereis_supervisor/2","doc":"* `kind` - The kind of supervisor\n  * `server_module` - The module implementing the server","ref":"Hermes.Server.Registry.Adapter.html#c:whereis_supervisor/2-parameters"},{"type":"callback","title":"Hermes.Server.Registry.Adapter.whereis_transport/2","doc":"Gets the PID of a transport process.\n\nReturns the PID if the transport is registered, nil otherwise.","ref":"Hermes.Server.Registry.Adapter.html#c:whereis_transport/2"},{"type":"callback","title":"Parameters - Hermes.Server.Registry.Adapter.whereis_transport/2","doc":"* `server_module` - The module implementing the server\n  * `transport_type` - The type of transport","ref":"Hermes.Server.Registry.Adapter.html#c:whereis_transport/2-parameters"},{"type":"module","title":"Hermes.Server.Response","doc":"Fluent interface for building MCP component responses.\n\nThis module provides builders for tool, prompt, and resource responses\nthat integrate seamlessly with the component system.","ref":"Hermes.Server.Response.html"},{"type":"module","title":"Examples - Hermes.Server.Response","doc":"# Tool response\n    Response.tool()\n    |> Response.text(\"Result: \" <> result)\n    |> Response.build()\n    \n    # Resource response (uri and mime_type come from component)\n    Response.resource()\n    |> Response.text(file_contents)\n    |> Response.build()\n    \n    # Prompt response\n    Response.prompt()\n    |> Response.user_message(\"What is the weather?\")\n    |> Response.assistant_message(\"Let me check...\")\n    |> Response.build()","ref":"Hermes.Server.Response.html#module-examples"},{"type":"function","title":"Hermes.Server.Response.assistant_message/2","doc":"Add an assistant message to a prompt response.","ref":"Hermes.Server.Response.html#assistant_message/2"},{"type":"function","title":"Parameters - Hermes.Server.Response.assistant_message/2","doc":"* `response` - A prompt response struct\n  * `content` - The message content (string or structured content)","ref":"Hermes.Server.Response.html#assistant_message/2-parameters"},{"type":"function","title":"Examples - Hermes.Server.Response.assistant_message/2","doc":"iex> Response.prompt() |> Response.assistant_message(\"Let me check the weather for you.\")\n    %Response{\n      type: :prompt,\n      messages: [%{\"role\" => \"assistant\", \"content\" => \"Let me check the weather for you.\"}]\n    }","ref":"Hermes.Server.Response.html#assistant_message/2-examples"},{"type":"function","title":"Hermes.Server.Response.audio/4","doc":"Add audio content to a tool response.","ref":"Hermes.Server.Response.html#audio/4"},{"type":"function","title":"Parameters - Hermes.Server.Response.audio/4","doc":"* `response` - A tool response struct\n  * `data` - Base64 encoded audio data\n  * `mime_type` - MIME type of the audio (e.g., \"audio/wav\")\n  * `opts` - Optional keyword list with:\n    * `:transcription` - Optional text transcription of the audio","ref":"Hermes.Server.Response.html#audio/4-parameters"},{"type":"function","title":"Examples - Hermes.Server.Response.audio/4","doc":"iex> Response.tool() |> Response.audio(audio_data, \"audio/wav\")\n    %Response{\n      type: :tool,\n      content: [%{\"type\" => \"audio\", \"data\" => audio_data, \"mimeType\" => \"audio/wav\"}],\n      isError: false\n    }\n    \n    iex> Response.tool() |> Response.audio(audio_data, \"audio/wav\", transcription: \"Hello\")\n    %Response{\n      type: :tool,\n      content: [%{\n        \"type\" => \"audio\",\n        \"data\" => audio_data,\n        \"mimeType\" => \"audio/wav\",\n        \"transcription\" => \"Hello\"\n      }],\n      isError: false\n    }","ref":"Hermes.Server.Response.html#audio/4-examples"},{"type":"function","title":"Hermes.Server.Response.blob/2","doc":"Set blob (base64) content for a resource response.","ref":"Hermes.Server.Response.html#blob/2"},{"type":"function","title":"Parameters - Hermes.Server.Response.blob/2","doc":"* `response` - A resource response struct\n  * `data` - Base64 encoded binary data","ref":"Hermes.Server.Response.html#blob/2-parameters"},{"type":"function","title":"Examples - Hermes.Server.Response.blob/2","doc":"iex> Response.resource() |> Response.blob(base64_data)\n    %Response{type: :resource, contents: %{\"blob\" => base64_data}}","ref":"Hermes.Server.Response.html#blob/2-examples"},{"type":"function","title":"Hermes.Server.Response.description/2","doc":"Set optional description for a resource response.","ref":"Hermes.Server.Response.html#description/2"},{"type":"function","title":"Parameters - Hermes.Server.Response.description/2","doc":"* `response` - A resource response struct\n  * `desc` - Description of the resource","ref":"Hermes.Server.Response.html#description/2-parameters"},{"type":"function","title":"Examples - Hermes.Server.Response.description/2","doc":"iex> Response.resource() |> Response.description(\"Application configuration settings\")\n    %Response{type: :resource, metadata: %{description: \"Application configuration settings\"}}","ref":"Hermes.Server.Response.html#description/2-examples"},{"type":"function","title":"Hermes.Server.Response.embedded_resource/3","doc":"Add an embedded resource reference to a tool response.","ref":"Hermes.Server.Response.html#embedded_resource/3"},{"type":"function","title":"Parameters - Hermes.Server.Response.embedded_resource/3","doc":"* `response` - A tool response struct\n  * `uri` - The resource URI\n  * `opts` - Optional keyword list with:\n    * `:name` - Human-readable name\n    * `:description` - Resource description\n    * `:mime_type` - MIME type\n    * `:text` - Text content (for text resources)\n    * `:blob` - Base64 data (for binary resources)","ref":"Hermes.Server.Response.html#embedded_resource/3-parameters"},{"type":"function","title":"Examples - Hermes.Server.Response.embedded_resource/3","doc":"iex> Response.tool() |> Response.embedded_resource(\"file://example.txt\",\n    ...>   name: \"Example File\",\n    ...>   mime_type: \"text/plain\",\n    ...>   text: \"File contents\"\n    ...> )","ref":"Hermes.Server.Response.html#embedded_resource/3-examples"},{"type":"function","title":"Hermes.Server.Response.error/2","doc":"Mark a tool response as an error and add error message.","ref":"Hermes.Server.Response.html#error/2"},{"type":"function","title":"Parameters - Hermes.Server.Response.error/2","doc":"* `response` - A tool response struct\n  * `message` - The error message","ref":"Hermes.Server.Response.html#error/2-parameters"},{"type":"function","title":"Examples - Hermes.Server.Response.error/2","doc":"iex> Response.tool() |> Response.error(\"Division by zero\")\n    %Response{\n      type: :tool,\n      content: [%{\"type\" => \"text\", \"text\" => \"Error: Division by zero\"}],\n      isError: true\n    }","ref":"Hermes.Server.Response.html#error/2-examples"},{"type":"function","title":"Hermes.Server.Response.image/3","doc":"Add image content to a tool response.","ref":"Hermes.Server.Response.html#image/3"},{"type":"function","title":"Parameters - Hermes.Server.Response.image/3","doc":"* `response` - A tool response struct\n  * `data` - Base64 encoded image data\n  * `mime_type` - MIME type of the image (e.g., \"image/png\")","ref":"Hermes.Server.Response.html#image/3-parameters"},{"type":"function","title":"Examples - Hermes.Server.Response.image/3","doc":"iex> Response.tool() |> Response.image(base64_data, \"image/png\")\n    %Response{\n      type: :tool,\n      content: [%{\"type\" => \"image\", \"data\" => base64_data, \"mimeType\" => \"image/png\"}],\n      isError: false\n    }","ref":"Hermes.Server.Response.html#image/3-examples"},{"type":"function","title":"Hermes.Server.Response.json/2","doc":"Add JSON-encoded content to a tool response.\n\nThis is a convenience function that automatically encodes data as JSON\nand adds it as text content. Useful for returning structured data from tools.","ref":"Hermes.Server.Response.html#json/2"},{"type":"function","title":"Parameters - Hermes.Server.Response.json/2","doc":"* `response` - A tool response struct\n  * `data` - Any JSON-encodable data structure","ref":"Hermes.Server.Response.html#json/2-parameters"},{"type":"function","title":"Examples - Hermes.Server.Response.json/2","doc":"iex> Response.tool() |> Response.json(%{status: \"ok\", count: 42})\n    %Response{\n      type: :tool,\n      content: [%{\"type\" => \"text\", \"text\" => \"{\\\"status\\\":\\\"ok\\\",\\\"count\\\":42}\"}],\n      isError: false\n    }\n    \n    iex> Response.tool() |> Response.json([1, 2, 3])\n    %Response{\n      type: :tool,\n      content: [%{\"type\" => \"text\", \"text\" => \"[1,2,3]\"}],\n      isError: false\n    }","ref":"Hermes.Server.Response.html#json/2-examples"},{"type":"function","title":"Hermes.Server.Response.name/2","doc":"Set optional name for a resource response.","ref":"Hermes.Server.Response.html#name/2"},{"type":"function","title":"Parameters - Hermes.Server.Response.name/2","doc":"* `response` - A resource response struct\n  * `name` - Human-readable name for the resource","ref":"Hermes.Server.Response.html#name/2-parameters"},{"type":"function","title":"Examples - Hermes.Server.Response.name/2","doc":"iex> Response.resource() |> Response.name(\"Configuration File\")\n    %Response{type: :resource, metadata: %{name: \"Configuration File\"}}","ref":"Hermes.Server.Response.html#name/2-examples"},{"type":"function","title":"Hermes.Server.Response.prompt/1","doc":"Start building a prompt response with optional description.","ref":"Hermes.Server.Response.html#prompt/1"},{"type":"function","title":"Parameters - Hermes.Server.Response.prompt/1","doc":"* `description` - Optional description of the prompt","ref":"Hermes.Server.Response.html#prompt/1-parameters"},{"type":"function","title":"Examples - Hermes.Server.Response.prompt/1","doc":"iex> Response.prompt()\n    %Response{type: :prompt, messages: []}\n    \n    iex> Response.prompt(\"Weather assistant prompt\")\n    %Response{type: :prompt, messages: [], description: \"Weather assistant prompt\"}","ref":"Hermes.Server.Response.html#prompt/1-examples"},{"type":"function","title":"Hermes.Server.Response.resource/0","doc":"Start building a resource response.\n\nThe uri and mimeType are automatically injected from the component's\nuri/0 and mime_type/0 callbacks when the response is built by the server.","ref":"Hermes.Server.Response.html#resource/0"},{"type":"function","title":"Examples - Hermes.Server.Response.resource/0","doc":"iex> Response.resource()\n    %Response{type: :resource, contents: nil}","ref":"Hermes.Server.Response.html#resource/0-examples"},{"type":"function","title":"Hermes.Server.Response.system_message/2","doc":"Add a system message to a prompt response.","ref":"Hermes.Server.Response.html#system_message/2"},{"type":"function","title":"Parameters - Hermes.Server.Response.system_message/2","doc":"* `response` - A prompt response struct\n  * `content` - The message content (string or structured content)","ref":"Hermes.Server.Response.html#system_message/2-parameters"},{"type":"function","title":"Examples - Hermes.Server.Response.system_message/2","doc":"iex> Response.prompt() |> Response.system_message(\"You are a helpful weather assistant.\")\n    %Response{\n      type: :prompt,\n      messages: [%{\"role\" => \"system\", \"content\" => \"You are a helpful weather assistant.\"}]\n    }","ref":"Hermes.Server.Response.html#system_message/2-examples"},{"type":"type","title":"Hermes.Server.Response.t/0","doc":"","ref":"Hermes.Server.Response.html#t:t/0"},{"type":"function","title":"Hermes.Server.Response.text/2","doc":"Add text content to a tool or resource response.\n\nFor tool responses, adds text to the content array.\nFor resource responses, sets the text content.","ref":"Hermes.Server.Response.html#text/2"},{"type":"function","title":"Parameters - Hermes.Server.Response.text/2","doc":"* `response` - A tool or resource response struct\n  * `text` - The text content","ref":"Hermes.Server.Response.html#text/2-parameters"},{"type":"function","title":"Examples - Hermes.Server.Response.text/2","doc":"iex> Response.tool() |> Response.text(\"Hello world\")\n    %Response{\n      type: :tool,\n      content: [%{\"type\" => \"text\", \"text\" => \"Hello world\"}],\n      isError: false\n    }\n    \n    iex> Response.resource() |> Response.text(\"File contents\")\n    %Response{type: :resource, contents: %{\"text\" => \"File contents\"}}","ref":"Hermes.Server.Response.html#text/2-examples"},{"type":"function","title":"Hermes.Server.Response.to_protocol/1","doc":"Build the final response structure.\n\nTransforms the response struct into the appropriate format for the MCP protocol.","ref":"Hermes.Server.Response.html#to_protocol/1"},{"type":"function","title":"Parameters - Hermes.Server.Response.to_protocol/1","doc":"* `response` - A response struct of any type","ref":"Hermes.Server.Response.html#to_protocol/1-parameters"},{"type":"function","title":"Examples - Hermes.Server.Response.to_protocol/1","doc":"iex> Response.tool() |> Response.text(\"Hello\") |> Response.to_protocol()\n    %{\"content\" => [%{\"type\" => \"text\", \"text\" => \"Hello\"}], \"isError\" => false}\n    \n    iex> Response.prompt() |> Response.user_message(\"Hi\") |> Response.to_protocol()\n    %{\"messages\" => [%{\"role\" => \"user\", \"content\" => \"Hi\"}]}\n    \n    iex> Response.resource() |> Response.text(\"data\") |> Response.to_protocol()\n    %{\"text\" => \"data\"}","ref":"Hermes.Server.Response.html#to_protocol/1-examples"},{"type":"function","title":"Hermes.Server.Response.to_protocol/3","doc":"","ref":"Hermes.Server.Response.html#to_protocol/3"},{"type":"function","title":"Hermes.Server.Response.tool/0","doc":"Start building a tool response.","ref":"Hermes.Server.Response.html#tool/0"},{"type":"function","title":"Examples - Hermes.Server.Response.tool/0","doc":"iex> Response.tool()\n    %Response{type: :tool, content: [], isError: false}","ref":"Hermes.Server.Response.html#tool/0-examples"},{"type":"function","title":"Hermes.Server.Response.user_message/2","doc":"Add a user message to a prompt response.","ref":"Hermes.Server.Response.html#user_message/2"},{"type":"function","title":"Parameters - Hermes.Server.Response.user_message/2","doc":"* `response` - A prompt response struct\n  * `content` - The message content (string or structured content)","ref":"Hermes.Server.Response.html#user_message/2-parameters"},{"type":"function","title":"Examples - Hermes.Server.Response.user_message/2","doc":"iex> Response.prompt() |> Response.user_message(\"What's the weather?\")\n    %Response{\n      type: :prompt,\n      messages: [%{\"role\" => \"user\", \"content\" => \"What's the weather?\"}]\n    }","ref":"Hermes.Server.Response.html#user_message/2-examples"},{"type":"module","title":"Hermes.Server.Session.Supervisor","doc":"Dynamic supervisor for managing per-session server processes.\n\nThis module provides a clean API for starting and stopping session-specific\nserver instances without creating atoms dynamically. It enables multi-session\nsupport for transports like StreamableHTTP that handle multiple concurrent\nclient connections.","ref":"Hermes.Server.Session.Supervisor.html"},{"type":"module","title":"Architecture - Hermes.Server.Session.Supervisor","doc":"The session supervisor is part of the server supervision tree:\n```\nServer.Supervisor\n├── Server.Base (message handler)\n├── Server.Session.Supervisor (this module)\n│   ├── Session Agent 1\n│   ├── Session Agent 2\n│   └── Session Agent N\n└── Transport (STDIO, StreamableHTTP, etc.)\n```","ref":"Hermes.Server.Session.Supervisor.html#module-architecture"},{"type":"module","title":"Session Lifecycle - Hermes.Server.Session.Supervisor","doc":"1. **Creation**: When a new client connects, a session is created via `create_session/2`\n2. **State Management**: Each session maintains its own state (protocol version,\n   client info, initialization status)\n3. **Isolation**: Sessions are isolated from each other, preventing cross-contamination\n4. **Cleanup**: Sessions are terminated when clients disconnect or on server shutdown","ref":"Hermes.Server.Session.Supervisor.html#module-session-lifecycle"},{"type":"module","title":"Usage - Hermes.Server.Session.Supervisor","doc":"Sessions are typically managed automatically by the Base server:\n\n    # Base server creates a session when needed\n    {:ok, pid} = Session.Supervisor.create_session(MyServer, session_id)\n\n    # Sessions can be explicitly closed\n    :ok = Session.Supervisor.close_session(MyServer, session_id)","ref":"Hermes.Server.Session.Supervisor.html#module-usage"},{"type":"module","title":"Benefits - Hermes.Server.Session.Supervisor","doc":"- **Resource Management**: Automatic cleanup of disconnected sessions\n- **Fault Tolerance**: Session crashes don't affect other sessions\n- **Scalability**: Support for many concurrent clients\n- **State Isolation**: Each client has independent state","ref":"Hermes.Server.Session.Supervisor.html#module-benefits"},{"type":"function","title":"Hermes.Server.Session.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Hermes.Server.Session.Supervisor.html#child_spec/1"},{"type":"function","title":"Hermes.Server.Session.Supervisor.close_session/3","doc":"Terminates a session and cleans up its resources.","ref":"Hermes.Server.Session.Supervisor.html#close_session/3"},{"type":"function","title":"Parameters - Hermes.Server.Session.Supervisor.close_session/3","doc":"* `registry` - The registry module to use to retrieve processes names\n  * `server` - The server module atom\n  * `session_id` - The session identifier to terminate","ref":"Hermes.Server.Session.Supervisor.html#close_session/3-parameters"},{"type":"function","title":"Returns - Hermes.Server.Session.Supervisor.close_session/3","doc":"* `:ok` - Session terminated successfully\n  * `{:error, :not_found}` - Session does not exist","ref":"Hermes.Server.Session.Supervisor.html#close_session/3-returns"},{"type":"function","title":"Examples - Hermes.Server.Session.Supervisor.close_session/3","doc":"# Close an existing session\n    :ok = Session.Supervisor.close_session(MyRegistry, MyServer, \"session-123\")\n\n    # Attempting to close non-existent session\n    {:error, :not_found} = Session.Supervisor.close_session(MyRegistry, MyServer, \"unknown\")","ref":"Hermes.Server.Session.Supervisor.html#close_session/3-examples"},{"type":"function","title":"Hermes.Server.Session.Supervisor.create_session/3","doc":"Creates a new session for a client connection.","ref":"Hermes.Server.Session.Supervisor.html#create_session/3"},{"type":"function","title":"Parameters - Hermes.Server.Session.Supervisor.create_session/3","doc":"* `registry` - The registry module to use to retrieve processes names\n  * `server` - The server module atom\n  * `session_id` - Unique identifier for the session (typically from transport)","ref":"Hermes.Server.Session.Supervisor.html#create_session/3-parameters"},{"type":"function","title":"Returns - Hermes.Server.Session.Supervisor.create_session/3","doc":"* `{:ok, pid}` - Session created successfully\n  * `{:error, {:already_started, pid}}` - Session already exists\n  * `{:error, reason}` - Failed to create session","ref":"Hermes.Server.Session.Supervisor.html#create_session/3-returns"},{"type":"function","title":"Examples - Hermes.Server.Session.Supervisor.create_session/3","doc":"# Create a new session for a client\n    {:ok, session_pid} = Session.Supervisor.create_session(MyRegistry, MyServer, \"session-123\")\n\n    # Attempting to create duplicate session\n    {:error, {:already_started, ^session_pid}} = \n      Session.Supervisor.create_session(MyRegistry, MyServer, \"session-123\")","ref":"Hermes.Server.Session.Supervisor.html#create_session/3-examples"},{"type":"function","title":"Hermes.Server.Session.Supervisor.start_link/1","doc":"Starts the session supervisor.","ref":"Hermes.Server.Session.Supervisor.html#start_link/1"},{"type":"function","title":"Parameters - Hermes.Server.Session.Supervisor.start_link/1","doc":"* `server` - The server module atom","ref":"Hermes.Server.Session.Supervisor.html#start_link/1-parameters"},{"type":"function","title":"Returns - Hermes.Server.Session.Supervisor.start_link/1","doc":"* `{:ok, pid}` - Supervisor started successfully\n  * `{:error, reason}` - Failed to start supervisor","ref":"Hermes.Server.Session.Supervisor.html#start_link/1-returns"},{"type":"function","title":"Examples - Hermes.Server.Session.Supervisor.start_link/1","doc":"{:ok, _pid} = Session.Supervisor.start_link(MyServer)","ref":"Hermes.Server.Session.Supervisor.html#start_link/1-examples"},{"type":"module","title":"Hermes.Server.Supervisor","doc":"Supervisor for MCP server processes.\n\nThis supervisor manages the lifecycle of an MCP server, including:\n- The Base server process that handles MCP protocol\n- The transport layer (STDIO, StreamableHTTP, or SSE)\n- Session supervisors for StreamableHTTP transport\n\nThe supervision strategy is `:one_for_all`, meaning if any child\nprocess crashes, all processes are restarted to maintain consistency.","ref":"Hermes.Server.Supervisor.html"},{"type":"module","title":"Conditional Startup - Hermes.Server.Supervisor","doc":"The supervisor intelligently handles startup based on transport type:\n\n- **STDIO transport**: Always starts\n- **StreamableHTTP/SSE transport**: Conditional startup based on:\n  1. Explicit `:start` option in transport config (highest priority)\n  2. `HERMES_MCP_SERVER` environment variable present\n  3. `PHX_SERVER` environment variable present, for phoenix apps using releases\n  4. Phoenix `:serve_endpoints` internalr runtime config (usage with `mix phx.server`)\n  5. Default: `true` (starts by default in production releases)\n\nThis ensures MCP servers start correctly in all environments:\n- Mix releases: Use `PHX_SERVER=true` or `HERMES_MCP_SERVER=true`\n- Development: Use `mix phx.server`\n- Tests/Tasks: Servers don't start unless explicitly configured","ref":"Hermes.Server.Supervisor.html#module-conditional-startup"},{"type":"module","title":"Configuration Examples - Hermes.Server.Supervisor","doc":"```elixir\n# Force start with transport option (highest priority)\n{MyServer, transport: {:streamable_http, start: true}}\n\n# Control via environment variables (works in releases)\nPHX_SERVER=true ./my_app start\n```","ref":"Hermes.Server.Supervisor.html#module-configuration-examples"},{"type":"module","title":"Supervision Tree - Hermes.Server.Supervisor","doc":"For STDIO transport:\n```\nSupervisor\n├── Base Server\n└── STDIO Transport\n```\n\nFor StreamableHTTP transport:\n```\nSupervisor\n├── Session.Supervisor\n├── Base Server\n└── StreamableHTTP Transport\n```","ref":"Hermes.Server.Supervisor.html#module-supervision-tree"},{"type":"function","title":"Hermes.Server.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Hermes.Server.Supervisor.html#child_spec/1"},{"type":"type","title":"Hermes.Server.Supervisor.sse/0","doc":"","ref":"Hermes.Server.Supervisor.html#t:sse/0"},{"type":"function","title":"Hermes.Server.Supervisor.start_link/2","doc":"Starts the server supervisor.","ref":"Hermes.Server.Supervisor.html#start_link/2"},{"type":"function","title":"Parameters - Hermes.Server.Supervisor.start_link/2","doc":"* `server` - The module implementing `Hermes.Server`\n  * `opts` - Options including:\n    * `:transport` - Transport configuration (required)\n    * `:name` - Supervisor name (optional, defaults to registered name)\n    * `:registry` - The custom registry to use to manage processes names (defaults to `Hermes.Server.Registry`)\n    * `:session_idle_timeout` - Time in milliseconds before idle sessions expire (default: 30 minutes)","ref":"Hermes.Server.Supervisor.html#start_link/2-parameters"},{"type":"function","title":"Examples - Hermes.Server.Supervisor.start_link/2","doc":"# Start with STDIO transport\n    Hermes.Server.Supervisor.start_link(MyServer, [], transport: :stdio)\n\n    # Start with StreamableHTTP transport\n    Hermes.Server.Supervisor.start_link(MyServer, [],\n      transport: {:streamable_http, port: 8080}\n    )\n    \n    # With custom session timeout (15 minutes)\n    Hermes.Server.Supervisor.start_link(MyServer, [],\n      transport: {:streamable_http, port: 8080},\n      session_idle_timeout: :timer.minutes(15)\n    )","ref":"Hermes.Server.Supervisor.html#start_link/2-examples"},{"type":"type","title":"Hermes.Server.Supervisor.start_option/0","doc":"","ref":"Hermes.Server.Supervisor.html#t:start_option/0"},{"type":"type","title":"Hermes.Server.Supervisor.stream_http/0","doc":"","ref":"Hermes.Server.Supervisor.html#t:stream_http/0"},{"type":"type","title":"Hermes.Server.Supervisor.transport/0","doc":"","ref":"Hermes.Server.Supervisor.html#t:transport/0"},{"type":"module","title":"Hermes.Server.Transport.SSE","doc":"SSE (Server-Sent Events) transport implementation for MCP servers.\n\nThis module provides backward compatibility with the HTTP+SSE transport\nfrom MCP protocol version 2024-11-05. It supports multiple concurrent\nclient sessions through Server-Sent Events for server-to-client communication\nand HTTP POST for client-to-server messages.","ref":"Hermes.Server.Transport.SSE.html"},{"type":"module","title":"Features - Hermes.Server.Transport.SSE","doc":"- Multiple concurrent client sessions\n- Server-Sent Events for real-time server-to-client communication\n- Separate SSE and POST endpoints (as per 2024-11-05 spec)\n- Automatic session cleanup on disconnect\n- Integration with existing Phoenix/Plug applications","ref":"Hermes.Server.Transport.SSE.html#module-features"},{"type":"module","title":"Usage - Hermes.Server.Transport.SSE","doc":"SSE transport is typically started through the server supervisor:\n\n    Hermes.Server.start_link(MyServer, [],\n      transport: :sse,\n      sse: [port: 8080, sse_path: \"/sse\", post_path: \"/messages\"]\n    )\n\nFor integration with existing Phoenix/Plug applications:\n\n    # In your router\n    forward \"/sse\", Hermes.Server.Transport.SSE.Plug,\n      server: MyApp.MCPServer,\n      mode: :sse\n\n    forward \"/messages\", Hermes.Server.Transport.SSE.Plug,\n      server: MyApp.MCPServer,\n      mode: :post","ref":"Hermes.Server.Transport.SSE.html#module-usage"},{"type":"module","title":"Message Flow - Hermes.Server.Transport.SSE","doc":"1. Client connects to SSE endpoint, receives \"endpoint\" event with POST URL\n2. Client sends messages via POST to the endpoint URL\n3. Server responses are pushed through the SSE connection\n4. Connection closes on client disconnect or server shutdown","ref":"Hermes.Server.Transport.SSE.html#module-message-flow"},{"type":"module","title":"Configuration - Hermes.Server.Transport.SSE","doc":"- `:port` - HTTP server port (default: 8080)\n- `:sse_path` - Path for SSE connections (default: \"/sse\")\n- `:post_path` - Path for POST messages (default: \"/messages\")\n- `:server` - The MCP server process to connect to\n- `:name` - Process registration name","ref":"Hermes.Server.Transport.SSE.html#module-configuration"},{"type":"function","title":"Hermes.Server.Transport.SSE.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Hermes.Server.Transport.SSE.html#child_spec/1"},{"type":"function","title":"Hermes.Server.Transport.SSE.get_endpoint_url/1","doc":"Gets the endpoint URL that should be sent to clients.\n\nThis constructs the URL that clients should use for POST requests.","ref":"Hermes.Server.Transport.SSE.html#get_endpoint_url/1"},{"type":"function","title":"Hermes.Server.Transport.SSE.get_schema/1","doc":"","ref":"Hermes.Server.Transport.SSE.html#get_schema/1"},{"type":"function","title":"Hermes.Server.Transport.SSE.get_sse_handler/2","doc":"Gets the SSE handler process for a session.\n\nReturns the pid of the process handling SSE for this session,\nor nil if no SSE connection exists.","ref":"Hermes.Server.Transport.SSE.html#get_sse_handler/2"},{"type":"function","title":"Hermes.Server.Transport.SSE.handle_message/4","doc":"Handles an incoming message from a client with request context.\n\nCalled by the Plug when a message is received via HTTP POST.","ref":"Hermes.Server.Transport.SSE.html#handle_message/4"},{"type":"type","title":"Hermes.Server.Transport.SSE.option/0","doc":"SSE transport options\n\n- `:server` - The server process (required)\n- `:name` - Name for registering the GenServer (required)\n- `:base_url` - Base URL for constructing endpoint URLs\n- `:post_path` - Path for POST endpoint (default: \"/messages\")","ref":"Hermes.Server.Transport.SSE.html#t:option/0"},{"type":"function","title":"Hermes.Server.Transport.SSE.parse_options/1","doc":"","ref":"Hermes.Server.Transport.SSE.html#parse_options/1"},{"type":"function","title":"Hermes.Server.Transport.SSE.parse_options!/1","doc":"","ref":"Hermes.Server.Transport.SSE.html#parse_options!/1"},{"type":"function","title":"Hermes.Server.Transport.SSE.register_sse_handler/2","doc":"Registers an SSE handler process for a session.\n\nCalled by the Plug when establishing an SSE connection.\nThe calling process becomes the SSE handler for the session.","ref":"Hermes.Server.Transport.SSE.html#register_sse_handler/2"},{"type":"function","title":"Hermes.Server.Transport.SSE.route_to_session/3","doc":"Routes a message to a specific session's SSE handler.\n\nUsed for targeted server notifications to specific clients.","ref":"Hermes.Server.Transport.SSE.html#route_to_session/3"},{"type":"function","title":"Hermes.Server.Transport.SSE.send_message/2","doc":"Sends a message to the client via the active SSE connection.\n\nThis broadcasts to all active SSE connections for the session.","ref":"Hermes.Server.Transport.SSE.html#send_message/2"},{"type":"function","title":"Parameters - Hermes.Server.Transport.SSE.send_message/2","doc":"* `transport` - The transport process\n  * `message` - The message to send","ref":"Hermes.Server.Transport.SSE.html#send_message/2-parameters"},{"type":"function","title":"Returns - Hermes.Server.Transport.SSE.send_message/2","doc":"* `:ok` if message was sent successfully\n  * `{:error, reason}` otherwise","ref":"Hermes.Server.Transport.SSE.html#send_message/2-returns"},{"type":"function","title":"Hermes.Server.Transport.SSE.shutdown/1","doc":"Shuts down the transport connection.\n\nThis terminates all active sessions managed by this transport.","ref":"Hermes.Server.Transport.SSE.html#shutdown/1"},{"type":"function","title":"Parameters - Hermes.Server.Transport.SSE.shutdown/1","doc":"* `transport` - The transport process","ref":"Hermes.Server.Transport.SSE.html#shutdown/1-parameters"},{"type":"function","title":"Hermes.Server.Transport.SSE.start_link/1","doc":"Starts the SSE transport.","ref":"Hermes.Server.Transport.SSE.html#start_link/1"},{"type":"type","title":"Hermes.Server.Transport.SSE.t/0","doc":"","ref":"Hermes.Server.Transport.SSE.html#t:t/0"},{"type":"function","title":"Hermes.Server.Transport.SSE.unregister_sse_handler/2","doc":"Unregisters an SSE handler process for a session.\n\nCalled when the SSE connection is closed.","ref":"Hermes.Server.Transport.SSE.html#unregister_sse_handler/2"},{"type":"module","title":"Hermes.Server.Transport.SSE.Plug","doc":"A Plug implementation for the SSE (Server-Sent Events) transport.\n\nThis plug handles the MCP HTTP+SSE protocol as specified in MCP 2024-11-05.\nIt provides two separate endpoints:\n\n- SSE endpoint: Opens an SSE stream and sends \"endpoint\" event\n- POST endpoint: Handles JSON-RPC messages from client to server","ref":"Hermes.Server.Transport.SSE.Plug.html"},{"type":"module","title":"SSE Streaming Architecture - Hermes.Server.Transport.SSE.Plug","doc":"This Plug handles SSE streaming by keeping the request process alive \nand managing the streaming loop for server-to-client communication.","ref":"Hermes.Server.Transport.SSE.Plug.html#module-sse-streaming-architecture"},{"type":"module","title":"Usage in Phoenix Router - Hermes.Server.Transport.SSE.Plug","doc":"pipeline :mcp do\n      plug :accepts, [\"json\", \"event-stream\"]\n    end\n\n    scope \"/mcp\" do\n      pipe_through :mcp\n\n      # SSE endpoint\n      get \"/sse\", Hermes.Server.Transport.SSE.Plug,\n        server: :your_server_name, mode: :sse\n\n      # POST endpoint\n      post \"/messages\", Hermes.Server.Transport.SSE.Plug,\n        server: :your_server_name, mode: :post\n    end","ref":"Hermes.Server.Transport.SSE.Plug.html#module-usage-in-phoenix-router"},{"type":"module","title":"Usage in Plug Router (Standalone) - Hermes.Server.Transport.SSE.Plug","doc":"# When using in a standalone Plug.Router app\n    plug Hermes.Server.Transport.SSE.Plug,\n      server: :your_server_name,\n      mode: :sse,\n      at: \"/sse\",\n      method_whitelist: [\"GET\"]\n\n    plug Hermes.Server.Transport.SSE.Plug,\n      server: :your_server_name,\n      mode: :post,\n      at: \"/messages\",\n      method_whitelist: [\"POST\"]","ref":"Hermes.Server.Transport.SSE.Plug.html#module-usage-in-plug-router-standalone"},{"type":"module","title":"Configuration Options - Hermes.Server.Transport.SSE.Plug","doc":"- `:server` - The server process name (required)\n- `:mode` - Either `:sse` or `:post` to determine endpoint behavior (required)\n- `:timeout` - Request timeout in milliseconds (default: 30000)\n- `:registry` - The registry to use. See `Hermes.Server.Registry.Adapter` for more information (default: `Hermes.Server.Registry`)","ref":"Hermes.Server.Transport.SSE.Plug.html#module-configuration-options"},{"type":"module","title":"Security Features - Hermes.Server.Transport.SSE.Plug","doc":"- Origin header validation for DNS rebinding protection\n- Session-based request validation\n- Automatic session cleanup on connection loss","ref":"Hermes.Server.Transport.SSE.Plug.html#module-security-features"},{"type":"module","title":"HTTP Response Codes - Hermes.Server.Transport.SSE.Plug","doc":"- 200: Successful request or SSE stream established\n- 202: Accepted (for notifications)\n- 400: Bad request (malformed JSON-RPC)\n- 405: Method not allowed\n- 500: Internal server error","ref":"Hermes.Server.Transport.SSE.Plug.html#module-http-response-codes"},{"type":"module","title":"Hermes.Server.Transport.STDIO","doc":"STDIO transport implementation for MCP servers.\n\nThis module handles communication with MCP clients via standard input/output streams,\nprocessing incoming JSON-RPC messages and forwarding responses.","ref":"Hermes.Server.Transport.STDIO.html"},{"type":"function","title":"Hermes.Server.Transport.STDIO.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Hermes.Server.Transport.STDIO.html#child_spec/1"},{"type":"function","title":"Hermes.Server.Transport.STDIO.get_schema/1","doc":"","ref":"Hermes.Server.Transport.STDIO.html#get_schema/1"},{"type":"type","title":"Hermes.Server.Transport.STDIO.option/0","doc":"STDIO transport options\n\n- `:server` - The server process (required)\n- `:name` - Optional name for registering the GenServer","ref":"Hermes.Server.Transport.STDIO.html#t:option/0"},{"type":"function","title":"Hermes.Server.Transport.STDIO.parse_options/1","doc":"","ref":"Hermes.Server.Transport.STDIO.html#parse_options/1"},{"type":"function","title":"Hermes.Server.Transport.STDIO.parse_options!/1","doc":"","ref":"Hermes.Server.Transport.STDIO.html#parse_options!/1"},{"type":"function","title":"Hermes.Server.Transport.STDIO.send_message/2","doc":"Sends a message to the client via stdout.","ref":"Hermes.Server.Transport.STDIO.html#send_message/2"},{"type":"function","title":"Parameters - Hermes.Server.Transport.STDIO.send_message/2","doc":"* `transport` - The transport process\n  * `message` - The message to send","ref":"Hermes.Server.Transport.STDIO.html#send_message/2-parameters"},{"type":"function","title":"Returns - Hermes.Server.Transport.STDIO.send_message/2","doc":"* `:ok` if message was sent successfully\n  * `{:error, reason}` otherwise","ref":"Hermes.Server.Transport.STDIO.html#send_message/2-returns"},{"type":"function","title":"Hermes.Server.Transport.STDIO.shutdown/1","doc":"Shuts down the transport connection.","ref":"Hermes.Server.Transport.STDIO.html#shutdown/1"},{"type":"function","title":"Parameters - Hermes.Server.Transport.STDIO.shutdown/1","doc":"* `transport` - The transport process","ref":"Hermes.Server.Transport.STDIO.html#shutdown/1-parameters"},{"type":"function","title":"Hermes.Server.Transport.STDIO.start_link/1","doc":"Starts a new STDIO transport process.","ref":"Hermes.Server.Transport.STDIO.html#start_link/1"},{"type":"function","title":"Parameters - Hermes.Server.Transport.STDIO.start_link/1","doc":"* `opts` - Options\n    * `:server` - (required) The server to forward messages to\n    * `:name` - Optional name for the GenServer process","ref":"Hermes.Server.Transport.STDIO.html#start_link/1-parameters"},{"type":"function","title":"Examples - Hermes.Server.Transport.STDIO.start_link/1","doc":"iex> Hermes.Server.Transport.STDIO.start_link(server: my_server)\n    {:ok, pid}","ref":"Hermes.Server.Transport.STDIO.html#start_link/1-examples"},{"type":"type","title":"Hermes.Server.Transport.STDIO.t/0","doc":"","ref":"Hermes.Server.Transport.STDIO.html#t:t/0"},{"type":"module","title":"Hermes.Server.Transport.StreamableHTTP","doc":"StreamableHTTP transport implementation for MCP servers.\n\nThis module provides an HTTP-based transport layer that supports multiple\nconcurrent client sessions through Server-Sent Events (SSE). It enables\nweb-based MCP clients to communicate with the server using standard HTTP\nprotocols.","ref":"Hermes.Server.Transport.StreamableHTTP.html"},{"type":"module","title":"Features - Hermes.Server.Transport.StreamableHTTP","doc":"- Multiple concurrent client sessions\n- Server-Sent Events for real-time server-to-client communication\n- HTTP POST endpoint for client-to-server messages\n- Automatic session cleanup on disconnect\n- Integration with Phoenix/Plug applications","ref":"Hermes.Server.Transport.StreamableHTTP.html#module-features"},{"type":"module","title":"Usage - Hermes.Server.Transport.StreamableHTTP","doc":"StreamableHTTP is typically started through the server supervisor:\n\n    Hermes.Server.start_link(MyServer, [],\n      transport: :streamable_http,\n      streamable_http: [port: 4000]\n    )\n\nFor integration with existing Phoenix/Plug applications:\n\n    # In your router\n    forward \"/mcp\", Hermes.Server.Transport.StreamableHTTP.Plug,\n      server: MyApp.MCPServer","ref":"Hermes.Server.Transport.StreamableHTTP.html#module-usage"},{"type":"module","title":"Message Flow - Hermes.Server.Transport.StreamableHTTP","doc":"1. Client connects to `/sse` endpoint, receives a session ID\n2. Client sends messages via POST to `/messages` with session ID header\n3. Server responses are pushed through the SSE connection\n4. Connection closes on client disconnect or server shutdown","ref":"Hermes.Server.Transport.StreamableHTTP.html#module-message-flow"},{"type":"module","title":"Configuration - Hermes.Server.Transport.StreamableHTTP","doc":"- `:port` - HTTP server port (default: 4000)\n- `:server` - The MCP server process to connect to\n- `:name` - Process registration name","ref":"Hermes.Server.Transport.StreamableHTTP.html#module-configuration"},{"type":"function","title":"Hermes.Server.Transport.StreamableHTTP.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Hermes.Server.Transport.StreamableHTTP.html#child_spec/1"},{"type":"function","title":"Hermes.Server.Transport.StreamableHTTP.get_schema/1","doc":"","ref":"Hermes.Server.Transport.StreamableHTTP.html#get_schema/1"},{"type":"function","title":"Hermes.Server.Transport.StreamableHTTP.get_sse_handler/2","doc":"Gets the SSE handler process for a session.\n\nReturns the pid of the process handling SSE for this session,\nor nil if no SSE connection exists.","ref":"Hermes.Server.Transport.StreamableHTTP.html#get_sse_handler/2"},{"type":"function","title":"Hermes.Server.Transport.StreamableHTTP.handle_message/4","doc":"Handles an incoming message from a client with request context.\n\nCalled by the Plug when a message is received via HTTP POST.","ref":"Hermes.Server.Transport.StreamableHTTP.html#handle_message/4"},{"type":"function","title":"Hermes.Server.Transport.StreamableHTTP.handle_message_for_sse/4","doc":"Handles an incoming message with context and returns {:sse, response} if SSE handler exists.\n\nThis allows the Plug to know whether to stream the response via SSE\nor return it as a regular HTTP response.","ref":"Hermes.Server.Transport.StreamableHTTP.html#handle_message_for_sse/4"},{"type":"type","title":"Hermes.Server.Transport.StreamableHTTP.option/0","doc":"StreamableHTTP transport options\n\n- `:server` - The server process (required)\n- `:name` - Name for registering the GenServer (required)","ref":"Hermes.Server.Transport.StreamableHTTP.html#t:option/0"},{"type":"function","title":"Hermes.Server.Transport.StreamableHTTP.parse_options/1","doc":"","ref":"Hermes.Server.Transport.StreamableHTTP.html#parse_options/1"},{"type":"function","title":"Hermes.Server.Transport.StreamableHTTP.parse_options!/1","doc":"","ref":"Hermes.Server.Transport.StreamableHTTP.html#parse_options!/1"},{"type":"function","title":"Hermes.Server.Transport.StreamableHTTP.register_sse_handler/2","doc":"Registers an SSE handler process for a session.\n\nCalled by the Plug when establishing an SSE connection.\nThe calling process becomes the SSE handler for the session.","ref":"Hermes.Server.Transport.StreamableHTTP.html#register_sse_handler/2"},{"type":"function","title":"Hermes.Server.Transport.StreamableHTTP.route_to_session/3","doc":"Routes a message to a specific session's SSE handler.\n\nUsed for targeted server notifications to specific clients.","ref":"Hermes.Server.Transport.StreamableHTTP.html#route_to_session/3"},{"type":"function","title":"Hermes.Server.Transport.StreamableHTTP.send_message/2","doc":"Sends a message to the client via the active SSE connection.\n\nThis function is used for server-initiated notifications.\nIt will broadcast to all active SSE connections.","ref":"Hermes.Server.Transport.StreamableHTTP.html#send_message/2"},{"type":"function","title":"Parameters - Hermes.Server.Transport.StreamableHTTP.send_message/2","doc":"* `transport` - The transport process\n  * `message` - The message to send","ref":"Hermes.Server.Transport.StreamableHTTP.html#send_message/2-parameters"},{"type":"function","title":"Returns - Hermes.Server.Transport.StreamableHTTP.send_message/2","doc":"* `:ok` if message was sent successfully\n  * `{:error, reason}` otherwise","ref":"Hermes.Server.Transport.StreamableHTTP.html#send_message/2-returns"},{"type":"function","title":"Hermes.Server.Transport.StreamableHTTP.shutdown/1","doc":"Shuts down the transport connection.\n\nThis terminates all active sessions managed by this transport.","ref":"Hermes.Server.Transport.StreamableHTTP.html#shutdown/1"},{"type":"function","title":"Parameters - Hermes.Server.Transport.StreamableHTTP.shutdown/1","doc":"* `transport` - The transport process","ref":"Hermes.Server.Transport.StreamableHTTP.html#shutdown/1-parameters"},{"type":"function","title":"Hermes.Server.Transport.StreamableHTTP.start_link/1","doc":"Starts the StreamableHTTP transport.","ref":"Hermes.Server.Transport.StreamableHTTP.html#start_link/1"},{"type":"type","title":"Hermes.Server.Transport.StreamableHTTP.t/0","doc":"","ref":"Hermes.Server.Transport.StreamableHTTP.html#t:t/0"},{"type":"function","title":"Hermes.Server.Transport.StreamableHTTP.unregister_sse_handler/2","doc":"Unregisters an SSE handler process for a session.\n\nCalled when the SSE connection is closed.","ref":"Hermes.Server.Transport.StreamableHTTP.html#unregister_sse_handler/2"},{"type":"module","title":"Hermes.Server.Transport.StreamableHTTP.Plug","doc":"A Plug implementation for the Streamable HTTP transport.\n\nThis plug handles the MCP Streamable HTTP protocol as specified in MCP 2025-03-26.\nIt provides a single endpoint that supports both GET and POST methods:\n\n- GET: Opens an SSE stream for server-to-client communication\n- POST: Handles JSON-RPC messages from client to server\n- DELETE: Closes a session","ref":"Hermes.Server.Transport.StreamableHTTP.Plug.html"},{"type":"module","title":"SSE Streaming Architecture - Hermes.Server.Transport.StreamableHTTP.Plug","doc":"This Plug handles SSE streaming by keeping the request process alive \nand managing the streaming loop for server-to-client communication.","ref":"Hermes.Server.Transport.StreamableHTTP.Plug.html#module-sse-streaming-architecture"},{"type":"module","title":"Usage in Phoenix Router - Hermes.Server.Transport.StreamableHTTP.Plug","doc":"pipeline :mcp do\n      plug :accepts, [\"json\"]\n    end\n\n    scope \"/mcp\" do\n      pipe_through :mcp\n      forward \"/\", to: Hermes.Server.Transport.StreamableHTTP.Plug, server: :your_server_name\n    end","ref":"Hermes.Server.Transport.StreamableHTTP.Plug.html#module-usage-in-phoenix-router"},{"type":"module","title":"Usage in Plug Router - Hermes.Server.Transport.StreamableHTTP.Plug","doc":"forward \"/mcp\", to: Hermes.Server.Transport.StreamableHTTP.Plug, init_opts: [server: :your_server_name]","ref":"Hermes.Server.Transport.StreamableHTTP.Plug.html#module-usage-in-plug-router"},{"type":"module","title":"Configuration Options - Hermes.Server.Transport.StreamableHTTP.Plug","doc":"- `:server` - The server process name (required)\n- `:session_header` - Custom header name for session ID (default: \"mcp-session-id\")\n- `:timeout` - Request timeout in milliseconds (default: 30000)\n- `:registry` - The registry to use. See `Hermes.Server.Registry.Adapter` for more information (default: `Hermes.Server.Registry`)","ref":"Hermes.Server.Transport.StreamableHTTP.Plug.html#module-configuration-options"},{"type":"module","title":"Security Features - Hermes.Server.Transport.StreamableHTTP.Plug","doc":"- Origin header validation for DNS rebinding protection\n- Session-based request validation\n- Automatic session cleanup on connection loss\n- Rate limiting support (when configured)","ref":"Hermes.Server.Transport.StreamableHTTP.Plug.html#module-security-features"},{"type":"module","title":"HTTP Response Codes - Hermes.Server.Transport.StreamableHTTP.Plug","doc":"- 200: Successful request\n- 202: Accepted (for notifications and responses)\n- 400: Bad request (malformed JSON-RPC)\n- 404: Session not found\n- 405: Method not allowed\n- 500: Internal server error","ref":"Hermes.Server.Transport.StreamableHTTP.Plug.html#module-http-response-codes"},{"type":"behaviour","title":"Hermes.Transport.Behaviour","doc":"Defines the behavior that all transport implementations must follow.","ref":"Hermes.Transport.Behaviour.html"},{"type":"type","title":"Hermes.Transport.Behaviour.message/0","doc":"The JSON-RPC message encoded","ref":"Hermes.Transport.Behaviour.html#t:message/0"},{"type":"type","title":"Hermes.Transport.Behaviour.reason/0","doc":"","ref":"Hermes.Transport.Behaviour.html#t:reason/0"},{"type":"callback","title":"Hermes.Transport.Behaviour.send_message/2","doc":"","ref":"Hermes.Transport.Behaviour.html#c:send_message/2"},{"type":"callback","title":"Hermes.Transport.Behaviour.shutdown/1","doc":"","ref":"Hermes.Transport.Behaviour.html#c:shutdown/1"},{"type":"callback","title":"Hermes.Transport.Behaviour.start_link/1","doc":"","ref":"Hermes.Transport.Behaviour.html#c:start_link/1"},{"type":"callback","title":"Hermes.Transport.Behaviour.supported_protocol_versions/0","doc":"Returns the list of MCP protocol versions supported by this transport.","ref":"Hermes.Transport.Behaviour.html#c:supported_protocol_versions/0"},{"type":"callback","title":"Examples - Hermes.Transport.Behaviour.supported_protocol_versions/0","doc":"iex> MyTransport.supported_protocol_versions()\n    [\"2024-11-05\", \"2025-03-26\"]","ref":"Hermes.Transport.Behaviour.html#c:supported_protocol_versions/0-examples"},{"type":"type","title":"Hermes.Transport.Behaviour.t/0","doc":"","ref":"Hermes.Transport.Behaviour.html#t:t/0"},{"type":"module","title":"Hermes.Transport.SSE","doc":"A transport implementation that uses Server-Sent Events (SSE) for receiving messages\nand HTTP POST requests for sending messages back to the server.\n\n>","ref":"Hermes.Transport.SSE.html"},{"type":"module","title":"Notes - Hermes.Transport.SSE","doc":">\n> For initialization and setup, check our [Installation & Setup](./installation.html) and\n> the [Transport options](./transport_options.html) guides for reference.","ref":"Hermes.Transport.SSE.html#module-notes"},{"type":"function","title":"Hermes.Transport.SSE.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Hermes.Transport.SSE.html#child_spec/1"},{"type":"function","title":"Hermes.Transport.SSE.get_schema/1","doc":"","ref":"Hermes.Transport.SSE.html#get_schema/1"},{"type":"type","title":"Hermes.Transport.SSE.option/0","doc":"The options for the SSE transport.\n\n- `:name` - The name of the transport process, respecting the `GenServer` \"Name Registration\" section.\n- `:client` - The client to send the messages to, respecting the `GenServer` \"Name Registration\" section.\n- `:server` - The server configuration.\n- `:headers` - The headers to send with the HTTP requests.\n- `:transport_opts` - The underlying HTTP transport options to pass to the HTTP client. You can check on the [Mint docs](https://hexdocs.pm/mint/Mint.HTTP.html#connect/4-transport-options)\n- `:http_options` - The underlying HTTP client options to pass to the HTTP client. You can check on the [Finch docs](https://hexdocs.pm/finch/Finch.html#t:request_opt/0)","ref":"Hermes.Transport.SSE.html#t:option/0"},{"type":"function","title":"Hermes.Transport.SSE.options_schema/1","doc":"","ref":"Hermes.Transport.SSE.html#options_schema/1"},{"type":"function","title":"Hermes.Transport.SSE.options_schema!/1","doc":"","ref":"Hermes.Transport.SSE.html#options_schema!/1"},{"type":"type","title":"Hermes.Transport.SSE.params_t/0","doc":"","ref":"Hermes.Transport.SSE.html#t:params_t/0"},{"type":"type","title":"Hermes.Transport.SSE.server/0","doc":"The options for the MCP server.\n\n- `:base_url` - The base URL of the MCP server (e.g. http://localhost:8000) (required).\n- `:base_path` - The base path of the MCP server (e.g. /mcp).\n- `:sse_path` - The path to the SSE endpoint (e.g. /mcp/sse) (default `:base_path` + `/sse`).","ref":"Hermes.Transport.SSE.html#t:server/0"},{"type":"type","title":"Hermes.Transport.SSE.t/0","doc":"","ref":"Hermes.Transport.SSE.html#t:t/0"},{"type":"module","title":"Hermes.Transport.STDIO","doc":"A transport implementation that uses standard input/output.\n\n>","ref":"Hermes.Transport.STDIO.html"},{"type":"module","title":"Notes - Hermes.Transport.STDIO","doc":">\n> For initialization and setup, check our [Installation & Setup](./installation.html) and\n> the [Transport options](./transport_options.html) guides for reference.","ref":"Hermes.Transport.STDIO.html#module-notes"},{"type":"function","title":"Hermes.Transport.STDIO.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Hermes.Transport.STDIO.html#child_spec/1"},{"type":"function","title":"Hermes.Transport.STDIO.get_schema/1","doc":"","ref":"Hermes.Transport.STDIO.html#get_schema/1"},{"type":"type","title":"Hermes.Transport.STDIO.option/0","doc":"The options for the STDIO transport.\n\n- `:command` - The command to run, it will be searched in the system's PATH.\n- `:args` - The arguments to pass to the command, as a list of strings.\n- `:env` - The extra environment variables to set for the command, as a map.\n- `:cwd` - The working directory for the command.\n- `:client` - The client to send the messages to, respecting the `GenServer` \"Name Registration\" section\n\nAnd any other `GenServer` init option.","ref":"Hermes.Transport.STDIO.html#t:option/0"},{"type":"function","title":"Hermes.Transport.STDIO.options_schema/1","doc":"","ref":"Hermes.Transport.STDIO.html#options_schema/1"},{"type":"function","title":"Hermes.Transport.STDIO.options_schema!/1","doc":"","ref":"Hermes.Transport.STDIO.html#options_schema!/1"},{"type":"type","title":"Hermes.Transport.STDIO.params_t/0","doc":"","ref":"Hermes.Transport.STDIO.html#t:params_t/0"},{"type":"type","title":"Hermes.Transport.STDIO.t/0","doc":"","ref":"Hermes.Transport.STDIO.html#t:t/0"},{"type":"module","title":"Hermes.Transport.StreamableHTTP","doc":"A transport implementation that uses Streamable HTTP as specified in MCP 2025-03-26.\n\nThis transport communicates with MCP servers via HTTP POST requests for sending messages\nand optionally uses Server-Sent Events (SSE) for receiving streaming responses.","ref":"Hermes.Transport.StreamableHTTP.html"},{"type":"module","title":"Usage - Hermes.Transport.StreamableHTTP","doc":"# Start the transport with a base URL\n    {:ok, transport} = Hermes.Transport.StreamableHTTP.start_link(\n      client: client_pid,\n      base_url: \"http://localhost:8000\",\n      mcp_path: \"/mcp\"\n    )\n\n    # Send a message\n    :ok = Hermes.Transport.StreamableHTTP.send_message(transport, encoded_message)","ref":"Hermes.Transport.StreamableHTTP.html#module-usage"},{"type":"module","title":"Session Management - Hermes.Transport.StreamableHTTP","doc":"The transport automatically handles MCP session IDs via the `mcp-session-id` header:\n- Extracts session ID from server responses\n- Includes session ID in subsequent requests\n- Maintains session state throughout the connection lifecycle\n- Handles session expiration (404 responses) by reinitializing","ref":"Hermes.Transport.StreamableHTTP.html#module-session-management"},{"type":"module","title":"Response Handling - Hermes.Transport.StreamableHTTP","doc":"Based on the response status and content type:\n- 202 Accepted: Message acknowledged, no immediate response\n- 200 OK with application/json: Single JSON response forwarded to client\n- 200 OK with text/event-stream: SSE stream parsed and events forwarded to client\n- 404 Not Found: Session expired, triggers reinitialization","ref":"Hermes.Transport.StreamableHTTP.html#module-response-handling"},{"type":"module","title":"SSE Support - Hermes.Transport.StreamableHTTP","doc":"The transport can establish a separate GET connection for server-initiated messages.\nThis allows the server to send requests and notifications without a client request.","ref":"Hermes.Transport.StreamableHTTP.html#module-sse-support"},{"type":"function","title":"Hermes.Transport.StreamableHTTP.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Hermes.Transport.StreamableHTTP.html#child_spec/1"},{"type":"function","title":"Hermes.Transport.StreamableHTTP.get_schema/1","doc":"","ref":"Hermes.Transport.StreamableHTTP.html#get_schema/1"},{"type":"type","title":"Hermes.Transport.StreamableHTTP.option/0","doc":"The options for the Streamable HTTP transport.\n\n- `:base_url` - The base URL of the MCP server (e.g. http://localhost:8000) (required).\n- `:mcp_path` - The MCP endpoint path (e.g. /mcp) (default \"/mcp\").\n- `:client` - The client to send the messages to.\n- `:headers` - The headers to send with the HTTP requests.\n- `:transport_opts` - The underlying HTTP transport options.\n- `:http_options` - The underlying HTTP client options.\n- `:enable_sse` - Whether to establish a GET connection for server-initiated messages (default false).","ref":"Hermes.Transport.StreamableHTTP.html#t:option/0"},{"type":"function","title":"Hermes.Transport.StreamableHTTP.options_schema/1","doc":"","ref":"Hermes.Transport.StreamableHTTP.html#options_schema/1"},{"type":"function","title":"Hermes.Transport.StreamableHTTP.options_schema!/1","doc":"","ref":"Hermes.Transport.StreamableHTTP.html#options_schema!/1"},{"type":"type","title":"Hermes.Transport.StreamableHTTP.params_t/0","doc":"","ref":"Hermes.Transport.StreamableHTTP.html#t:params_t/0"},{"type":"type","title":"Hermes.Transport.StreamableHTTP.t/0","doc":"","ref":"Hermes.Transport.StreamableHTTP.html#t:t/0"},{"type":"module","title":"Hermes.Transport.WebSocket","doc":"A transport implementation that uses WebSockets for bidirectional communication\nwith the MCP server.\n\n>","ref":"Hermes.Transport.WebSocket.html"},{"type":"module","title":"Notes - Hermes.Transport.WebSocket","doc":">\n> For initialization and setup, check our [Installation & Setup](./installation.html) and\n> the [Transport options](./transport_options.html) guides for reference.","ref":"Hermes.Transport.WebSocket.html#module-notes"},{"type":"function","title":"Hermes.Transport.WebSocket.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Hermes.Transport.WebSocket.html#child_spec/1"},{"type":"function","title":"Hermes.Transport.WebSocket.get_schema/1","doc":"","ref":"Hermes.Transport.WebSocket.html#get_schema/1"},{"type":"type","title":"Hermes.Transport.WebSocket.option/0","doc":"The options for the WebSocket transport.\n\n- `:name` - The name of the transport process, respecting the `GenServer` \"Name Registration\" section.\n- `:client` - The client to send the messages to, respecting the `GenServer` \"Name Registration\" section.\n- `:server` - The server configuration.\n- `:headers` - The headers to send with the HTTP requests.\n- `:transport_opts` - The underlying transport options to pass to Gun.","ref":"Hermes.Transport.WebSocket.html#t:option/0"},{"type":"function","title":"Hermes.Transport.WebSocket.options_schema/1","doc":"","ref":"Hermes.Transport.WebSocket.html#options_schema/1"},{"type":"function","title":"Hermes.Transport.WebSocket.options_schema!/1","doc":"","ref":"Hermes.Transport.WebSocket.html#options_schema!/1"},{"type":"type","title":"Hermes.Transport.WebSocket.params_t/0","doc":"","ref":"Hermes.Transport.WebSocket.html#t:params_t/0"},{"type":"type","title":"Hermes.Transport.WebSocket.server/0","doc":"The options for the MCP server.\n\n- `:base_url` - The base URL of the MCP server (e.g. http://localhost:8000) (required).\n- `:base_path` - The base path of the MCP server (e.g. /mcp).\n- `:ws_path` - The path to the WebSocket endpoint (e.g. /mcp/ws) (default `:base_path` + `/ws`).","ref":"Hermes.Transport.WebSocket.html#t:server/0"},{"type":"type","title":"Hermes.Transport.WebSocket.t/0","doc":"","ref":"Hermes.Transport.WebSocket.html#t:t/0"},{"type":"task","title":"mix hermes.sse.interactive","doc":"Mix task to test the SSE transport implementation, interactively sending commands.","ref":"Mix.Tasks.Hermes.Sse.Interactive.html"},{"type":"task","title":"Options - mix hermes.sse.interactive","doc":"* `--base-url` - Base URL for the SSE server (default: http://localhost:8000)\n* `--base-path` - Base path to append to the base URL\n* `--sse-path` - Specific SSE endpoint path","ref":"Mix.Tasks.Hermes.Sse.Interactive.html#module-options"},{"type":"function","title":"Mix.Tasks.Hermes.Sse.Interactive.run/1","doc":"Callback implementation for  Mix.Task.run/1 .","ref":"Mix.Tasks.Hermes.Sse.Interactive.html#run/1"},{"type":"task","title":"mix hermes.stdio.interactive","doc":"Mix task to test the STDIO transport implementation, interactively sending commands.","ref":"Mix.Tasks.Hermes.Stdio.Interactive.html"},{"type":"task","title":"Options - mix hermes.stdio.interactive","doc":"* `--command` - Command to execute for the STDIO transport (default: \"mcp\")\n* `--args` - Comma-separated arguments for the command (default: \"run,priv/dev/echo/index.py\")","ref":"Mix.Tasks.Hermes.Stdio.Interactive.html#module-options"},{"type":"function","title":"Mix.Tasks.Hermes.Stdio.Interactive.run/1","doc":"Callback implementation for  Mix.Task.run/1 .","ref":"Mix.Tasks.Hermes.Stdio.Interactive.html#run/1"},{"type":"task","title":"mix hermes.streamable_http.interactive","doc":"Mix task to test the Streamable HTTP transport implementation, interactively sending commands.","ref":"Mix.Tasks.Hermes.StreamableHttp.Interactive.html"},{"type":"task","title":"Options - mix hermes.streamable_http.interactive","doc":"* `--base-url` - Base URL for the MCP server (default: http://localhost:8000)\n* `--mcp-path` - MCP endpoint path (default: /mcp)","ref":"Mix.Tasks.Hermes.StreamableHttp.Interactive.html#module-options"},{"type":"function","title":"Mix.Tasks.Hermes.StreamableHttp.Interactive.run/1","doc":"Callback implementation for  Mix.Task.run/1 .","ref":"Mix.Tasks.Hermes.StreamableHttp.Interactive.html#run/1"},{"type":"task","title":"mix hermes.websocket.interactive","doc":"Mix task to test the WebSocket transport implementation, interactively sending commands.","ref":"Mix.Tasks.Hermes.Websocket.Interactive.html"},{"type":"task","title":"Options - mix hermes.websocket.interactive","doc":"* `--base-url` - Base URL for the WebSocket server (default: http://localhost:8000)\n* `--base-path` - Base path to append to the base URL\n* `--ws-path` - Specific WebSocket endpoint path (default: /ws)","ref":"Mix.Tasks.Hermes.Websocket.Interactive.html#module-options"},{"type":"function","title":"Mix.Tasks.Hermes.Websocket.Interactive.run/1","doc":"Callback implementation for  Mix.Task.run/1 .","ref":"Mix.Tasks.Hermes.Websocket.Interactive.html#run/1"},{"type":"exception","title":"Hermes.Server.ConfigurationError","doc":"Raised when required MCP server configuration is missing or invalid.\n\nThe MCP specification requires servers to provide:\n- `name`: A human-readable name for the server\n- `version`: The server's version string","ref":"Hermes.Server.ConfigurationError.html"},{"type":"exception","title":"Examples - Hermes.Server.ConfigurationError","doc":"# This will raise an error - missing required options\n    defmodule BadServer do\n      use Hermes.Server  # Raises Hermes.Server.ConfigurationError\n    end\n    \n    # This is correct\n    defmodule GoodServer do\n      use Hermes.Server,\n        name: \"My Server\",\n        version: \"1.0.0\"\n    end","ref":"Hermes.Server.ConfigurationError.html#module-examples"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project are documented in this file.\n\n## [0.11.1](https://github.com/cloudwalk/hermes-mcp/compare/v0.11.0...v0.11.1) (2025-06-30)","ref":"changelog.html"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* loggin should respect the logger config ([#157](https://github.com/cloudwalk/hermes-mcp/issues/157)) ([0fbf6a6](https://github.com/cloudwalk/hermes-mcp/commit/0fbf6a652ed9dd5da64716d47ece675192442ea0))\n\n## [0.11.0](https://github.com/cloudwalk/hermes-mcp/compare/v0.10.5...v0.11.0) (2025-06-30)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Features - Changelog","doc":"* runtime server components, simplified api ([#153](https://github.com/cloudwalk/hermes-mcp/issues/153)) ([8af35d6](https://github.com/cloudwalk/hermes-mcp/commit/8af35d67cd15d125e40ab9b115ff6900c3487ea5))","ref":"changelog.html#features"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* allow users to control external process messages ([#149](https://github.com/cloudwalk/hermes-mcp/issues/149)) ([8fef4eb](https://github.com/cloudwalk/hermes-mcp/commit/8fef4eb753f38a325a1d3a94c310e5fd1c478ede))\n* correctly encode prompt/resource ([#155](https://github.com/cloudwalk/hermes-mcp/issues/155)) ([4249b13](https://github.com/cloudwalk/hermes-mcp/commit/4249b137e43587862de2e59acba4660ac785702a))\n* more genserver mcp server callbacks, plug based startup and documentation ([#152](https://github.com/cloudwalk/hermes-mcp/issues/152)) ([9c26b1c](https://github.com/cloudwalk/hermes-mcp/commit/9c26b1ce4d033e3c69bc872a5ed01a037ec68f59))\n* server behaviour with optional callbacks ([#151](https://github.com/cloudwalk/hermes-mcp/issues/151)) ([91aa191](https://github.com/cloudwalk/hermes-mcp/commit/91aa1916f28da5972f6b100861a2547697c1ddb7))","ref":"changelog.html#bug-fixes-1"},{"type":"extras","title":"Code Refactoring - Changelog","doc":"* improve runtime components schema def ([#154](https://github.com/cloudwalk/hermes-mcp/issues/154)) ([96ff2a9](https://github.com/cloudwalk/hermes-mcp/commit/96ff2a97be02f0f35ee4a288838a1dbf518bbac3))\n\n## [0.10.5](https://github.com/cloudwalk/hermes-mcp/compare/v0.10.4...v0.10.5) (2025-06-25)","ref":"changelog.html#code-refactoring"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* normalize transport api ([#146](https://github.com/cloudwalk/hermes-mcp/issues/146)) ([8a30a34](https://github.com/cloudwalk/hermes-mcp/commit/8a30a34c944ca4d85de622f9063a558bd495c6fc)), closes [#145](https://github.com/cloudwalk/hermes-mcp/issues/145)\n\n## [0.10.4](https://github.com/cloudwalk/hermes-mcp/compare/v0.10.3...v0.10.4) (2025-06-24)","ref":"changelog.html#bug-fixes-2"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* server session expiration on idle (configurable) ([#143](https://github.com/cloudwalk/hermes-mcp/issues/143)) ([d9f7164](https://github.com/cloudwalk/hermes-mcp/commit/d9f7164028b2b6e8ff3d697f888adf655110bc1f))\n\n## [0.10.3](https://github.com/cloudwalk/hermes-mcp/compare/v0.10.2...v0.10.3) (2025-06-23)","ref":"changelog.html#bug-fixes-3"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* include formatter on hex release ([#139](https://github.com/cloudwalk/hermes-mcp/issues/139)) ([d91b244](https://github.com/cloudwalk/hermes-mcp/commit/d91b244aeed4211a6653a82705b03da2247db9a3))\n\n## [0.10.2](https://github.com/cloudwalk/hermes-mcp/compare/v0.10.1...v0.10.2) (2025-06-23)","ref":"changelog.html#bug-fixes-4"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* default implementation for server handle_notification ([#135](https://github.com/cloudwalk/hermes-mcp/issues/135)) ([c958041](https://github.com/cloudwalk/hermes-mcp/commit/c9580410162c31a62190631f6702024ea3458beb))","ref":"changelog.html#bug-fixes-5"},{"type":"extras","title":"Code Refactoring - Changelog","doc":"* cleaner peri integration ([#137](https://github.com/cloudwalk/hermes-mcp/issues/137)) ([43226cc](https://github.com/cloudwalk/hermes-mcp/commit/43226cc9fb2f49edfbf8685fe181eb3093304308)), closes [#123](https://github.com/cloudwalk/hermes-mcp/issues/123)\n\n## [0.10.1](https://github.com/cloudwalk/hermes-mcp/compare/v0.10.0...v0.10.1) (2025-06-21)","ref":"changelog.html#code-refactoring-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* client should send both sse/json headers on POST requests ([#134](https://github.com/cloudwalk/hermes-mcp/issues/134)) ([e906b7f](https://github.com/cloudwalk/hermes-mcp/commit/e906b7f02bf390faecc2b6bd39aab05ef9c500b1))\n* correctly allows macro-based/callback-based server implementations ([#131](https://github.com/cloudwalk/hermes-mcp/issues/131)) ([d7bfc75](https://github.com/cloudwalk/hermes-mcp/commit/d7bfc7541a8c381573f1a20ebc37d4a7dbaaa139))\n* remove last uses of hard-coded Hermes.Server.Registry ([cc0ffd9](https://github.com/cloudwalk/hermes-mcp/commit/cc0ffd95fce771a9c861c6859124dcedb6ceb88e))\n\n## [0.10.0](https://github.com/cloudwalk/hermes-mcp/compare/v0.9.1...v0.10.0) (2025-06-18)","ref":"changelog.html#bug-fixes-6"},{"type":"extras","title":"Features - Changelog","doc":"* batch operations on server-side ([#125](https://github.com/cloudwalk/hermes-mcp/issues/125)) ([28eea7c](https://github.com/cloudwalk/hermes-mcp/commit/28eea7cd15f72c4effccc8475e6b301b2cd9745c))\n* missing notifications handlers ([#129](https://github.com/cloudwalk/hermes-mcp/issues/129)) ([34d5934](https://github.com/cloudwalk/hermes-mcp/commit/34d593499fdd846f430b93b4c52ca986f345646d))\n* support batch operations on client side ([#101](https://github.com/cloudwalk/hermes-mcp/issues/101)) ([fadf28d](https://github.com/cloudwalk/hermes-mcp/commit/fadf28d80068f3f1e77835fd46a276338048f0bc))\n* tools annotations ([#127](https://github.com/cloudwalk/hermes-mcp/issues/127)) ([c83e8f1](https://github.com/cloudwalk/hermes-mcp/commit/c83e8f1b0e721b1a03960ac67cdd0774337675dc))","ref":"changelog.html#features-1"},{"type":"extras","title":"Miscellaneous Chores - Changelog","doc":"* release please correct version on readme ([#128](https://github.com/cloudwalk/hermes-mcp/issues/128)) ([d0125c6](https://github.com/cloudwalk/hermes-mcp/commit/d0125c664b5190b560bb988ff01d17cbdba814bd))\n* update peri ([#126](https://github.com/cloudwalk/hermes-mcp/issues/126)) ([7292615](https://github.com/cloudwalk/hermes-mcp/commit/7292615cf57da0185006df2f6221f8ab93aacd30))\n\n## [0.9.1](https://github.com/cloudwalk/hermes-mcp/compare/v0.9.0...v0.9.1) (2025-06-13)","ref":"changelog.html#miscellaneous-chores"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* allow enum specific type on json-schema ([#121](https://github.com/cloudwalk/hermes-mcp/issues/121)) ([23c9ce2](https://github.com/cloudwalk/hermes-mcp/commit/23c9ce2081ed1099ce1f3afbd9318c8a02480039)), closes [#114](https://github.com/cloudwalk/hermes-mcp/issues/114)\n* correctly escape quoted expressions ([#119](https://github.com/cloudwalk/hermes-mcp/issues/119)) ([0c469c5](https://github.com/cloudwalk/hermes-mcp/commit/0c469c552d8d5fd2498706b4b1f41100e8561e2f)), closes [#118](https://github.com/cloudwalk/hermes-mcp/issues/118)\n\n## [0.9.0](https://github.com/cloudwalk/hermes-mcp/compare/v0.8.2...v0.9.0) (2025-06-12)","ref":"changelog.html#bug-fixes-7"},{"type":"extras","title":"Features - Changelog","doc":"* add independently configurable logs ([#113](https://github.com/cloudwalk/hermes-mcp/issues/113)) ([bb0be27](https://github.com/cloudwalk/hermes-mcp/commit/bb0be2716ac41bf43814a16497b209fa55cc811b))","ref":"changelog.html#features-2"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* allow registering a name for the client supervisor ([#117](https://github.com/cloudwalk/hermes-mcp/issues/117)) ([d356511](https://github.com/cloudwalk/hermes-mcp/commit/d356511e2fbcf47b6a85ed733671e96d800ac693))\n\n## [0.8.2](https://github.com/cloudwalk/hermes-mcp/compare/v0.8.1...v0.8.2) (2025-06-11)","ref":"changelog.html#bug-fixes-8"},{"type":"extras","title":"Code Refactoring - Changelog","doc":"* higher-level client implementation ([#111](https://github.com/cloudwalk/hermes-mcp/issues/111)) ([5de2162](https://github.com/cloudwalk/hermes-mcp/commit/5de2162b166e390b4dacfd7b9960fab59c81b4d7))\n\n## [0.8.1](https://github.com/cloudwalk/hermes-mcp/compare/v0.8.0...v0.8.1) (2025-06-10)","ref":"changelog.html#code-refactoring-2"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* hermes should respect mix releases startup ([#109](https://github.com/cloudwalk/hermes-mcp/issues/109)) ([f42d476](https://github.com/cloudwalk/hermes-mcp/commit/f42d476e1dc05c57479170bd58aaca9028ef1e66))\n\n## [0.8.0](https://github.com/cloudwalk/hermes-mcp/compare/v0.7.0...v0.8.0) (2025-06-10)","ref":"changelog.html#bug-fixes-9"},{"type":"extras","title":"Features - Changelog","doc":"* inject user and transport data on mcp server frame ([#106](https://github.com/cloudwalk/hermes-mcp/issues/106)) ([feb2ce3](https://github.com/cloudwalk/hermes-mcp/commit/feb2ce308e9fd0cde4118b294dd47ce64d8db18f))\n* legacy sse server transport ([#102](https://github.com/cloudwalk/hermes-mcp/issues/102)) ([4a71088](https://github.com/cloudwalk/hermes-mcp/commit/4a71088713071a726bde03bf1385c3c794d2134b))","ref":"changelog.html#features-3"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* allow empty capabilities on incoming JSON-RPC messages ([#105](https://github.com/cloudwalk/hermes-mcp/issues/105)) ([f0ad4cf](https://github.com/cloudwalk/hermes-mcp/commit/f0ad4cf1a5a85cc8a56baed875d2d2d200bb5860)), closes [#96](https://github.com/cloudwalk/hermes-mcp/issues/96)","ref":"changelog.html#bug-fixes-10"},{"type":"extras","title":"Miscellaneous Chores - Changelog","doc":"* release please should include all files ([#108](https://github.com/cloudwalk/hermes-mcp/issues/108)) ([d0a25b9](https://github.com/cloudwalk/hermes-mcp/commit/d0a25b968c83ae1023ffffc8ee07b5b490122c03))\n\n## [0.7.0](https://github.com/cloudwalk/hermes-mcp/compare/v0.6.0...v0.7.0) (2025-06-09)","ref":"changelog.html#miscellaneous-chores-1"},{"type":"extras","title":"Features - Changelog","doc":"* allow json schema fields on tools/prompts definition ([#99](https://github.com/cloudwalk/hermes-mcp/issues/99)) ([0345f12](https://github.com/cloudwalk/hermes-mcp/commit/0345f122484a0169645c5da07e50c2d64fd6c5f5))\n\n## [0.6.0](https://github.com/cloudwalk/hermes-mcp/compare/v0.5.0...v0.6.0) (2025-06-09)","ref":"changelog.html#features-4"},{"type":"extras","title":"Features - Changelog","doc":"* allow customize server registry impl ([#94](https://github.com/cloudwalk/hermes-mcp/issues/94)) ([f3ac087](https://github.com/cloudwalk/hermes-mcp/commit/f3ac08749a7c361466a7a619f9782e8d8706a7b6))\n* mcp high level server components definition ([#91](https://github.com/cloudwalk/hermes-mcp/issues/91)) ([007f41d](https://github.com/cloudwalk/hermes-mcp/commit/007f41d33874fd9f1b5e340ecbe16317dc3576b7))\n* mcp server handlers refactored ([#92](https://github.com/cloudwalk/hermes-mcp/issues/92)) ([e213e04](https://github.com/cloudwalk/hermes-mcp/commit/e213e046b1360b24ff9e42835cdf80f5fe2ae4fa))","ref":"changelog.html#features-5"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* correctly handle mcp requests on phoenix apps ([#88](https://github.com/cloudwalk/hermes-mcp/issues/88)) ([09f4235](https://github.com/cloudwalk/hermes-mcp/commit/09f42359f0daac694013f0be4f6a74de2be7f4ff)), closes [#86](https://github.com/cloudwalk/hermes-mcp/issues/86)","ref":"changelog.html#bug-fixes-11"},{"type":"extras","title":"Miscellaneous Chores - Changelog","doc":"* upcate automatic version ([#98](https://github.com/cloudwalk/hermes-mcp/issues/98)) ([0c08233](https://github.com/cloudwalk/hermes-mcp/commit/0c08233371338af24ea66047b4e1a8e9fa5cb055))","ref":"changelog.html#miscellaneous-chores-2"},{"type":"extras","title":"Code Refactoring - Changelog","doc":"* tests ([#93](https://github.com/cloudwalk/hermes-mcp/issues/93)) ([ca31feb](https://github.com/cloudwalk/hermes-mcp/commit/ca31febee7aec1d45dcb32398b33228d1399ae39))\n\n## [0.5.0](https://github.com/cloudwalk/hermes-mcp/compare/v0.4.1...v0.5.0) (2025-06-05)","ref":"changelog.html#code-refactoring-3"},{"type":"extras","title":"Features - Changelog","doc":"* client support new mcp spec ([#83](https://github.com/cloudwalk/hermes-mcp/issues/83)) ([73d14f7](https://github.com/cloudwalk/hermes-mcp/commit/73d14f77522cef0f7212230c05cdac23ee2d93e2))\n* enable log disabling ([#78](https://github.com/cloudwalk/hermes-mcp/issues/78)) ([fa1453f](https://github.com/cloudwalk/hermes-mcp/commit/fa1453fee9b015c0ad7f9ac223749a9c9f1fcf6a))\n* low level genservy mcp server implementation (stdio + stremable http) ([#77](https://github.com/cloudwalk/hermes-mcp/issues/77)) ([e6606b4](https://github.com/cloudwalk/hermes-mcp/commit/e6606b4d66a2d7ddeb6c32e0041c22d4f0036ac5))\n* mvp higher level mcp server definition ([#84](https://github.com/cloudwalk/hermes-mcp/issues/84)) ([a5fec1c](https://github.com/cloudwalk/hermes-mcp/commit/a5fec1c976595c3363d4eec83e0cbc382eac9207))","ref":"changelog.html#features-6"},{"type":"extras","title":"Code Refactoring - Changelog","doc":"* base mcp server implementation correctly uses streamable http ([#85](https://github.com/cloudwalk/hermes-mcp/issues/85)) ([29060fd](https://github.com/cloudwalk/hermes-mcp/commit/29060fd2d2e383c58c727a9085b30162c6b8179a))\n\n## [0.4.0](https://github.com/cloudwalk/hermes-mcp) - 2025-05-06","ref":"changelog.html#code-refactoring-4"},{"type":"extras","title":"Added - Changelog","doc":"- Implemented WebSocket transport (#70)\n- Emit `telemetry` events (#54)\n- Implement client feature `completion` request (#72)\n- Implement client feature roots, server requests (#73)\n\n## [0.3.12](https://github.com/cloudwalk/hermes-mcp) - 2025-04-24","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Correctly handles \"nested\" timeouts (genserver vs MCP) (#71)\n\n## [0.3.11](https://github.com/cloudwalk/hermes-mcp) - 2025-04-17","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Improved core library logging and added verbosity level on interactive/CLI (#68)\n\n## [0.3.10](https://github.com/cloudwalk/hermes-mcp) - 2025-04-17","ref":"changelog.html#added-1"},{"type":"extras","title":"Fixed - Changelog","doc":"- Handle SSE ping and reconnect events from server (#65)\n\n## [0.3.9](https://github.com/cloudwalk/hermes-mcp) - 2025-04-15","ref":"changelog.html#fixed-1"},{"type":"extras","title":"Fixed - Changelog","doc":"- Improved and simplified SSE endpoint event URI merging (#64)","ref":"changelog.html#fixed-2"},{"type":"extras","title":"Added - Changelog","doc":"- Added internal client/transport state inspection on CLI/mix tasks (#61)\n\n## [0.3.8](https://github.com/cloudwalk/hermes-mcp) - 2025-04-10","ref":"changelog.html#added-2"},{"type":"extras","title":"Added - Changelog","doc":"- Created `Operation` struct to standardize client API calls (#56)\n- Fixed ERTS version to avoid release errors","ref":"changelog.html#added-3"},{"type":"extras","title":"Fixed - Changelog","doc":"- Resolved client timeout confusion by standardizing timeout handling (#42)\n\n## [0.3.7](https://github.com/cloudwalk/hermes-mcp) - 2025-04-01","ref":"changelog.html#fixed-3"},{"type":"extras","title":"Fixed - Changelog","doc":"- Client reinitialization from interactive CLI (#55)\n\n## [0.3.6](https://github.com/cloudwalk/hermes-mcp) - 2025-03-28","ref":"changelog.html#fixed-4"},{"type":"extras","title":"Added - Changelog","doc":"- New roadmap and protocol update proposal (#53)\n- Added documentation for the 2025-03-26 protocol update\n\n## [0.3.5](https://github.com/cloudwalk/hermes-mcp) - 2025-03-25","ref":"changelog.html#added-4"},{"type":"extras","title":"Documentation - Changelog","doc":"- Added Roadmap to README (#47)\n\n## [0.3.4](https://github.com/cloudwalk/hermes-mcp) - 2025-03-20","ref":"changelog.html#documentation"},{"type":"extras","title":"Added - Changelog","doc":"- `help` command and flag on the interactive CLI (#37)\n- improve SSE connection status on interactive task/cli (#37)\n\n## [0.3.3](https://github.com/cloudwalk/hermes-mcp) - 2025-03-20","ref":"changelog.html#added-5"},{"type":"extras","title":"Added - Changelog","doc":"- Client request cancellation support (#35)\n- Improved URI path handling for SSE transport (#36)\n- Enhanced interactive mix tasks for testing MCP servers (#34)\n\n## [0.3.2](https://github.com/cloudwalk/hermes-mcp) - 2025-03-19","ref":"changelog.html#added-6"},{"type":"extras","title":"Added - Changelog","doc":"- Ship static binaries to use hermes-mcp as standalone application\n\n## [0.3.1](https://github.com/cloudwalk/hermes-mcp) - 2025-03-19","ref":"changelog.html#added-7"},{"type":"extras","title":"Added - Changelog","doc":"- Ship interactive mix tasks `stdio.interactive` and `sse.interactive` to test MCP servers\n\n## [0.3.0](https://github.com/cloudwalk/hermes-mcp) - 2025-03-18","ref":"changelog.html#added-8"},{"type":"extras","title":"Added - Changelog","doc":"- Structured server-client logging support (#27)\n- Progress notification tracking (#26)\n- MCP domain model implementation (#28)\n- Comprehensive SSE unit tests (#20)\n- Centralized state management (#31)\n- Standardized error response handling (#32)","ref":"changelog.html#added-9"},{"type":"extras","title":"Fixed - Changelog","doc":"- Improved domain error handling (#33)\n\n## [0.2.3](https://github.com/cloudwalk/hermes-mcp) - 2025-03-12","ref":"changelog.html#fixed-5"},{"type":"extras","title":"Added - Changelog","doc":"- Enhanced SSE transport with graceful shutdown capabilities (#25)\n- Improved SSE streaming with automatic reconnection handling (#25)\n\n## [0.2.2](https://github.com/cloudwalk/hermes-mcp) - 2025-03-05","ref":"changelog.html#added-10"},{"type":"extras","title":"Added - Changelog","doc":"- Support for multiple concurrent client <> transport pairs (#24)\n- Improved client resource management\n\n## [0.2.1](https://github.com/cloudwalk/hermes-mcp) - 2025-02-28","ref":"changelog.html#added-11"},{"type":"extras","title":"Added - Changelog","doc":"- Support for custom base and SSE paths in HTTP/SSE client (#19)\n- Enhanced configuration options for SSE endpoints\n\n## [0.2.0](https://github.com/cloudwalk/hermes-mcp) - 2025-02-27","ref":"changelog.html#added-12"},{"type":"extras","title":"Added - Changelog","doc":"- Implemented HTTP/SSE transport (#7)\n  - Support for server-sent events communication\n  - HTTP client integration for MCP protocol\n  - Streaming response handling","ref":"changelog.html#added-13"},{"type":"extras","title":"Documentation - Changelog","doc":"- Extensive guides and documentation improvements\n\n## [0.1.0](https://github.com/cloudwalk/hermes-mcp) - 2025-02-26","ref":"changelog.html#documentation-1"},{"type":"extras","title":"Added - Changelog","doc":"- Implemented STDIO transport (#1) for MCP communication\n  - Support for bidirectional communication via standard I/O\n  - Automatic process monitoring and recovery\n  - Environment variable handling for cross-platform support\n  - Integration test utilities in Mix tasks\n\n- Created stateful client interface (#6)\n  - Robust GenServer implementation for MCP client\n  - Automatic initialization and protocol handshake\n  - Synchronous-feeling API over asynchronous transport\n  - Support for all MCP operations (ping, resources, prompts, tools)\n  - Proper error handling and logging\n  - Capability negotiation and management\n\n- Developed JSON-RPC message parsing (#5)\n  - Schema-based validation of MCP messages\n  - Support for requests, responses, notifications, and errors\n  - Comprehensive test suite for message handling\n  - Encoding/decoding functions with proper validation\n\n- Established core architecture and client API\n  - MCP protocol implementation following specification\n  - Client struct for maintaining connection state\n  - Request/response correlation with unique IDs\n  - Initial transport abstraction layer","ref":"changelog.html#added-14"},{"type":"extras","title":"Documentation - Changelog","doc":"- Added detailed RFC document describing the library architecture\n- Enhanced README with project overview and installation instructions","ref":"changelog.html#documentation-2"},{"type":"extras","title":"Contributing to Hermes MCP","doc":"# Contributing to Hermes MCP\n\nThank you for your interest in contributing to Hermes MCP! This document provides guidelines and instructions for contributing to the project.\n\nFirstly, have sure to follow the official MCP (Model Context Protocol) [specification](https://spec.modelcontextprotocol.io/specification/2024-11-05/)!","ref":"contributing.html"},{"type":"extras","title":"Development Setup - Contributing to Hermes MCP","doc":"","ref":"contributing.html#development-setup"},{"type":"extras","title":"Prerequisites - Contributing to Hermes MCP","doc":"- Elixir 1.18+\n- Erlang/OTP 26+\n- Python 3.11+ with uv (for echo server)\n- Go 1.21+ (for calculator server)\n- Just command runner","ref":"contributing.html#prerequisites"},{"type":"extras","title":"Getting Started - Contributing to Hermes MCP","doc":"1. Clone the repository\n   ```bash\n   git clone https://github.com/cloudwalk/hermes-mcp.git\n   cd hermes-mcp\n   ```\n\n2. Install dependencies\n   ```bash\n   mix setup\n   ```\n\n3. Run tests to ensure everything is working\n   ```bash\n   mix test\n   ```","ref":"contributing.html#getting-started"},{"type":"extras","title":"Development Workflow - Contributing to Hermes MCP","doc":"","ref":"contributing.html#development-workflow"},{"type":"extras","title":"Running MCP Servers - Contributing to Hermes MCP","doc":"For development and testing, you can use the provided MCP server implementations:\n\n```bash\n# Start the Echo server (Python)\n# For now only support stdio transport\njust echo-server\n\n# Start the Calculator server (Go)\n# Supports both stdio and http/sse transport\njust calculator-server sse\n```","ref":"contributing.html#running-mcp-servers"},{"type":"extras","title":"Code Quality - Contributing to Hermes MCP","doc":"Before submitting a pull request, ensure your code passes all quality checks:\n\n```bash\n# Run all code quality checks\nmix lint\n\n# Individual checks\nmix format        # Code formatting\nmix credo         # Linting\nmix dialyzer      # Type checking\n```","ref":"contributing.html#code-quality"},{"type":"extras","title":"Testing - Contributing to Hermes MCP","doc":"Write tests for all new features and bug fixes:\n\n```bash\n# Run all tests\nmix test\n```","ref":"contributing.html#testing"},{"type":"extras","title":"Submitting Contributions - Contributing to Hermes MCP","doc":"1. Create a new branch for your feature or bugfix\n   ```bash\n   git checkout -b feature/your-feature-name\n   ```\n\n2. Make your changes and commit them with clear, descriptive messages\n\n3. Push your branch to GitHub\n   ```bash\n   git push origin feature/your-feature-name\n   ```\n\n4. Open a pull request against the main branch","ref":"contributing.html#submitting-contributions"},{"type":"extras","title":"Pull Request Guidelines - Contributing to Hermes MCP","doc":"- Follow the existing code style and conventions\n- Include tests for new functionality\n- Update documentation as needed\n- Keep pull requests focused on a single topic\n- Reference any related issues in your PR description","ref":"contributing.html#pull-request-guidelines"},{"type":"extras","title":"Documentation - Contributing to Hermes MCP","doc":"Update documentation for any user-facing changes:\n\n- Update relevant sections in `pages/` directory\n- Add examples for new features\n- Document any breaking changes","ref":"contributing.html#documentation"},{"type":"extras","title":"Release Process - Contributing to Hermes MCP","doc":"Releases are managed by the maintainers. Version numbers follow [Semantic Versioning](https://semver.org/).","ref":"contributing.html#release-process"},{"type":"extras","title":"License - Contributing to Hermes MCP","doc":"By contributing to Hermes MCP, you agree that your contributions will be licensed under the project's [MIT License](./LICENSE).","ref":"contributing.html#license"},{"type":"extras","title":"LICENSE","doc":"Copyright 2025 zoedsoupe  \n\nPermission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"},{"type":"extras","title":"Hermes MCP","doc":"# Hermes MCP\n\n[![hex.pm](https://img.shields.io/hexpm/v/hermes_mcp.svg)](https://hex.pm/packages/hermes_mcp)\n[![docs](https://img.shields.io/badge/hex-docs-blue.svg)](https://hexdocs.pm/hermes_mcp)\n[![ci](https://github.com/cloudwalk/hermes-mcp/actions/workflows/ci.yml/badge.svg)](https://github.com/cloudwalk/hermes-mcp/actions/workflows/ci.yml)\n\nA high-performance Model Context Protocol (MCP) implementation in Elixir.","ref":"readme.html"},{"type":"extras","title":"Overview - Hermes MCP","doc":"Hermes MCP is a comprehensive Elixir SDK for the [Model Context Protocol](https://spec.modelcontextprotocol.io/), providing complete client and server implementations with Elixir's exceptional concurrency model and fault tolerance.","ref":"readme.html#overview"},{"type":"extras","title":"Installation - Hermes MCP","doc":"```elixir\ndef deps do\n  [\n    {:hermes_mcp, \"~> 0.11.1\"}  # x-release-please-version\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Quick Start - Hermes MCP","doc":"","ref":"readme.html#quick-start"},{"type":"extras","title":"Server - Hermes MCP","doc":"```elixir\n# Define a server with tools capabilities\ndefmodule MyApp.MCPServer do\n  use Hermes.Server, \n    name: \"My Server\", \n    version: \"1.0.0\", \n    capabilities: [:tools]\n\n  @impl true\n  # this callback will be called when the\n  # MCP initialize lifecycle completes\n  def init(_client_info, frame) do\n    {:ok,frame\n      |> assign(counter: 0)\n      |> register_tool(\"echo\",\n        input_schema: %{\n          text: {:required, :string, max: 150, description: \"the text to be echoed\"}\n        },\n        annotations: %{read_only: true},\n        description: \"echoes everything the user says to the LLM\") }\n  end\n\n  @impl true\n  def handle_tool(\"echo\", %{text: text}, frame) do\n    Logger.info(\"This tool was called #{frame.assigns.counter + 1}\")\n    {:reply, text, assign(frame, counter: frame.assigns.counter + 1)}\n  end\nend\n\n# Add to your application supervisor\nchildren = [\n  Hermes.Server.Registry,\n  {MyApp.MCPServer, transport: :streamable_http}\n]\n\n# Add to your Plug/Phoenix router (if using HTTP)\nforward \"/mcp\", Hermes.Server.Transport.StreamableHTTP.Plug, server: MyApp.MCPServer\n```\n\nNow you can achieve your MCP server on `http://localhost: /mcp`","ref":"readme.html#server"},{"type":"extras","title":"Client - Hermes MCP","doc":"```elixir\n# Define a client module\ndefmodule MyApp.MCPClient do\n  use Hermes.Client,\n    name: \"MyApp\",\n    version: \"1.0.0\",\n    protocol_version: \"2025-03-26\"\nend\n\n# Add to your application supervisor\nchildren = [\n  {MyApp.AnthropicClient, \n   transport: {:streamable_http, base_url: \"http://localhost:4000\"}}\n]\n\n# Use the client\n{:ok, tools} = MyApp.MCPClient.list_tools()\n{:ok, result} = MyApp.MCPClient.call_tool(\"search\", %{query: \"elixir\"})\n```","ref":"readme.html#client"},{"type":"extras","title":"Why Hermes? - Hermes MCP","doc":"Named after Hermes, the Greek god of boundaries and communication, this library facilitates seamless interaction between Large Language Models and external tools - serving as a messenger between AI and data sources.","ref":"readme.html#why-hermes"},{"type":"extras","title":"Documentation - Hermes MCP","doc":"For detailed guides and examples, visit the [official documentation](https://hexdocs.pm/hermes_mcp).","ref":"readme.html#documentation"},{"type":"extras","title":"License - Hermes MCP","doc":"MIT License. See [LICENSE](./LICENSE) for details.","ref":"readme.html#license"},{"type":"extras","title":"Installation","doc":"# Installation\n\nAdd Hermes MCP to your Elixir project.","ref":"installation.html"},{"type":"extras","title":"Add Dependency - Installation","doc":"In `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:hermes_mcp, \"~> 0.11.1\"} # x-release-please-version\n  ]\nend\n```\n\nThen:\n\n```shell\nmix deps.get\n```","ref":"installation.html#add-dependency"},{"type":"extras","title":"Client Setup - Installation","doc":"Define a client module:\n\n```elixir\ndefmodule MyApp.MCPClient do\n  use Hermes.Client,\n    name: \"MyApp\",\n    version: \"1.0.0\",\n    protocol_version: \"2024-11-05\",\n    capabilities: [:roots, :sampling]\nend\n```\n\nAdd to your supervision tree:\n\n```elixir\ndefmodule MyApp.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      # MCP client with STDIO transport\n      {MyApp.MCPClient, \n       transport: {:stdio, command: \"python\", args: [\"-m\", \"mcp.server\", \"my_server.py\"]}}\n    ]\n\n    opts = [strategy: :one_for_all, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```","ref":"installation.html#client-setup"},{"type":"extras","title":"Server Setup - Installation","doc":"For MCP servers, see [Server Quick Start](server_quickstart.md).","ref":"installation.html#server-setup"},{"type":"extras","title":"Client Module Options - Installation","doc":"When defining a client module with `use Hermes.Client`:\n\n| Option | Type | Description | Required |\n|--------|------|-------------|----------|\n| `:name` | string | Client name to advertise | Yes |\n| `:version` | string | Client version | Yes |\n| `:protocol_version` | string | MCP protocol version | Yes |\n| `:capabilities` | list | Client capabilities | Yes |","ref":"installation.html#client-module-options"},{"type":"extras","title":"Transport Options - Installation","doc":"When starting the client:\n\n| Transport | Options | Example |\n|-----------|---------|----------|\n| STDIO | `command`, `args` | `{:stdio, command: \"python\", args: [\"-m\", \"server\"]}` |\n| SSE | `base_url` | `{:sse, base_url: \"http://localhost:8000\"}` |\n| WebSocket | `url` | `{:websocket, url: \"ws://localhost:8000/ws\"}` |\n| HTTP | `url` | `{:streamable_http, url: \"http://localhost:8000/mcp\"}` |","ref":"installation.html#transport-options"},{"type":"extras","title":"Next Steps - Installation","doc":"- [Client Usage](client_usage.md) - Using the client\n- [Transport Layer](transport.md) - Transport details\n- [Server Development](server_quickstart.md) - Build servers","ref":"installation.html#next-steps"},{"type":"extras","title":"Transport Layer","doc":"# Transport Layer\n\nConnect MCP clients to servers using different transport mechanisms.","ref":"transport.html"},{"type":"extras","title":"Available Transports - Transport Layer","doc":"| Transport | Module | Use Case |\n|-----------|--------|----------|\n| **STDIO** | `Hermes.Transport.STDIO` | Local subprocess servers |\n| **SSE** | `Hermes.Transport.SSE` | HTTP servers with Server-Sent Events |\n| **WebSocket** | `Hermes.Transport.WebSocket` | Real-time bidirectional communication |\n| **StreamableHTTP** | `Hermes.Transport.StreamableHTTP` | HTTP with streaming responses |","ref":"transport.html#available-transports"},{"type":"extras","title":"Transport Interface - Transport Layer","doc":"All transports implement:\n\n```elixir\n@callback start_link(keyword()) :: GenServer.on_start()\n@callback send_message(t(), message()) :: :ok | {:error, reason()}\n@callback shutdown(t()) :: :ok | {:error, reason()}\n```","ref":"transport.html#transport-interface"},{"type":"extras","title":"STDIO Transport - Transport Layer","doc":"For local subprocess servers.","ref":"transport.html#stdio-transport"},{"type":"extras","title":"Configuration - Transport Layer","doc":"```elixir\n{Hermes.Transport.STDIO, [\n  name: MyApp.MCPTransport,\n  client: MyApp.MCPClient,\n  command: \"python\",\n  args: [\"-m\", \"mcp.server\", \"my_server.py\"],\n  env: %{\"PYTHONPATH\" => \"/path/to/modules\"},\n  cwd: \"/path/to/server\"\n]}\n```","ref":"transport.html#configuration"},{"type":"extras","title":"Options - Transport Layer","doc":"| Option | Type | Description | Default |\n|--------|------|-------------|---------|\n| `:name` | atom | Process name | Required |\n| `:client` | pid/name | Client process | Required |\n| `:command` | string | Command to run | Required |\n| `:args` | list | Command arguments | `[]` |\n| `:env` | map | Environment vars | System defaults |\n| `:cwd` | string | Working directory | Current dir |","ref":"transport.html#options"},{"type":"extras","title":"SSE Transport - Transport Layer","doc":"For HTTP servers with Server-Sent Events.","ref":"transport.html#sse-transport"},{"type":"extras","title":"Configuration - Transport Layer","doc":"```elixir\n{Hermes.Transport.SSE, [\n  name: MyApp.HTTPTransport,\n  client: MyApp.MCPClient,\n  server: [\n    base_url: \"https://api.example.com\",\n    base_path: \"/mcp\",\n    sse_path: \"/sse\"\n  ],\n  headers: [{\"Authorization\", \"Bearer token\"}]\n]}\n```","ref":"transport.html#configuration-1"},{"type":"extras","title":"Options - Transport Layer","doc":"| Option | Type | Description | Default |\n|--------|------|-------------|---------|\n| `:server.base_url` | string | Server URL | Required |\n| `:server.base_path` | string | Base path | `\"/\"` |\n| `:server.sse_path` | string | SSE endpoint | `\"/sse\"` |\n| `:headers` | list | HTTP headers | `[]` |","ref":"transport.html#options-1"},{"type":"extras","title":"WebSocket Transport - Transport Layer","doc":"For real-time bidirectional communication.","ref":"transport.html#websocket-transport"},{"type":"extras","title":"Configuration - Transport Layer","doc":"```elixir\n{Hermes.Transport.WebSocket, [\n  name: MyApp.WSTransport,\n  client: MyApp.MCPClient,\n  server: [\n    base_url: \"wss://api.example.com\",\n    ws_path: \"/ws\"\n  ]\n]}\n```","ref":"transport.html#configuration-2"},{"type":"extras","title":"Options - Transport Layer","doc":"| Option | Type | Description | Default |\n|--------|------|-------------|---------|\n| `:server.base_url` | string | WebSocket URL | Required |\n| `:server.ws_path` | string | WS endpoint | `\"/ws\"` |\n| `:headers` | list | HTTP headers | `[]` |","ref":"transport.html#options-2"},{"type":"extras","title":"Custom Transport - Transport Layer","doc":"Implement the behaviour:\n\n```elixir\ndefmodule MyTransport do\n  @behaviour Hermes.Transport.Behaviour\n  use GenServer\n\n  def start_link(opts) do\n    GenServer.start_link(__MODULE__, opts, name: opts[:name])\n  end\n\n  def send_message(pid, message) do\n    GenServer.call(pid, {:send, message})\n  end\n\n  def shutdown(pid) do\n    GenServer.stop(pid)\n  end\n\n  # GenServer callbacks...\nend\n```","ref":"transport.html#custom-transport"},{"type":"extras","title":"Supervision - Transport Layer","doc":"Use `:one_for_all` strategy:\n\n```elixir\nchildren = [\n  {Hermes.Transport.STDIO, transport_opts},\n  {Hermes.Client, client_opts}\n]\n\nSupervisor.start_link(children, strategy: :one_for_all)\n```\n\nTransport and client depend on each other - if one fails, restart both.","ref":"transport.html#supervision"},{"type":"extras","title":"References - Transport Layer","doc":"See [MCP transport specification](https://spec.modelcontextprotocol.io/specification/basic/transports/)","ref":"transport.html#references"},{"type":"extras","title":"Client Usage","doc":"# Client Usage\n\nUsing the Hermes MCP client with the new DSL.","ref":"client_usage.html"},{"type":"extras","title":"Basic Setup - Client Usage","doc":"Once your client module is defined and added to the supervision tree, it automatically:\n- Connects to the server\n- Negotiates protocol version\n- Exchanges capabilities\n- Completes handshake","ref":"client_usage.html#basic-setup"},{"type":"extras","title":"Connection Status - Client Usage","doc":"```elixir\n# Using your client module directly\ncase MyApp.MCPClient.ping() do\n  :pong -> \n    IO.puts(\"Server is responsive\")\n  \n  {:error, reason} -> \n    IO.puts(\"Connection error: #{inspect(reason)}\")\nend\n```","ref":"client_usage.html#connection-status"},{"type":"extras","title":"Server Information - Client Usage","doc":"```elixir\n# Get capabilities\ncapabilities = MyApp.MCPClient.get_server_capabilities()\n\n# Get server info\ninfo = MyApp.MCPClient.get_server_info()\n```","ref":"client_usage.html#server-information"},{"type":"extras","title":"Working with Tools - Client Usage","doc":"","ref":"client_usage.html#working-with-tools"},{"type":"extras","title":"List Tools - Client Usage","doc":"```elixir\ncase MyApp.MCPClient.list_tools() do\n  {:ok, %{result: %{\"tools\" => tools}}} ->\n    for tool <- tools do\n      IO.puts(\"#{tool[\"name\"]}: #{tool[\"description\"]}\")\n    end\n\n  {:error, error} ->\n    IO.puts(\"Error: #{inspect(error)}\")\nend\n```","ref":"client_usage.html#list-tools"},{"type":"extras","title":"Call Tool - Client Usage","doc":"```elixir\ncase MyApp.MCPClient.call_tool(\"calculator\", %{expr: \"2+2\"}) do\n  {:ok, %{is_error: false, result: result}} ->\n    IO.puts(\"Result: #{inspect(result)}\")\n\n  {:ok, %{is_error: true, result: error}} ->\n    IO.puts(\"Tool error: #{error[\"message\"]}\")\n\n  {:error, error} ->\n    IO.puts(\"Protocol error: #{inspect(error)}\")\nend\n```","ref":"client_usage.html#call-tool"},{"type":"extras","title":"Working with Resources - Client Usage","doc":"","ref":"client_usage.html#working-with-resources"},{"type":"extras","title":"List Resources - Client Usage","doc":"```elixir\n{:ok, %{result: %{\"resources\" => resources}}} = \n  MyApp.MCPClient.list_resources()\n\nfor resource <- resources do\n  IO.puts(\"#{resource[\"name\"]} (#{resource[\"uri\"]})\")\nend\n```","ref":"client_usage.html#list-resources"},{"type":"extras","title":"Read Resource - Client Usage","doc":"```elixir\ncase MyApp.MCPClient.read_resource(\"file:///data.txt\") do\n  {:ok, %{result: %{\"contents\" => contents}}} ->\n    for content <- contents do\n      case content do\n        %{\"text\" => text} -> IO.puts(text)\n        %{\"blob\" => blob} -> IO.puts(\"Binary: #{byte_size(blob)} bytes\")\n      end\n    end\n\n  {:error, error} ->\n    IO.puts(\"Error: #{inspect(error)}\")\nend\n```","ref":"client_usage.html#read-resource"},{"type":"extras","title":"Pagination - Client Usage","doc":"```elixir\n# First page\n{:ok, %{result: result}} = MyApp.MCPClient.list_resources()\nresources = result[\"resources\"]\ncursor = result[\"nextCursor\"]\n\n# Next page\nif cursor do\n  {:ok, %{result: more}} = \n    MyApp.MCPClient.list_resources(cursor: cursor)\nend\n```","ref":"client_usage.html#pagination"},{"type":"extras","title":"Working with Prompts - Client Usage","doc":"","ref":"client_usage.html#working-with-prompts"},{"type":"extras","title":"List Prompts - Client Usage","doc":"```elixir\n{:ok, %{result: %{\"prompts\" => prompts}}} = \n  MyApp.MCPClient.list_prompts()\n\nfor prompt <- prompts do\n  IO.puts(\"#{prompt[\"name\"]}: #{prompt[\"description\"]}\")\nend\n```","ref":"client_usage.html#list-prompts"},{"type":"extras","title":"Get Prompt - Client Usage","doc":"```elixir\nargs = %{\"language\" => \"elixir\", \"task\" => \"refactor\"}\n\ncase MyApp.MCPClient.get_prompt(\"code_review\", args) do\n  {:ok, %{result: %{\"messages\" => messages}}} ->\n    for msg <- messages do\n      IO.puts(\"#{msg[\"role\"]}: #{msg[\"content\"]}\")\n    end\n\n  {:error, error} ->\n    IO.puts(\"Error: #{inspect(error)}\")\nend\n```","ref":"client_usage.html#get-prompt"},{"type":"extras","title":"Timeouts - Client Usage","doc":"Set custom timeouts per operation:\n\n```elixir\n# 60 second timeout\nopts = [timeout: 60_000]\nMyApp.MCPClient.call_tool(\"slow_tool\", %{}, opts)\n```\n\nDefault timeout is 30 seconds.","ref":"client_usage.html#timeouts"},{"type":"extras","title":"Autocompletion - Client Usage","doc":"Get completion suggestions:\n\n```elixir\n# For prompt arguments\nref = %{\"type\" => \"ref/prompt\", \"name\" => \"code_review\"}\narg = %{\"name\" => \"language\", \"value\" => \"py\"}\n\n{:ok, response} = MyApp.MCPClient.complete(ref, arg)\n# => [\"python\", \"pytorch\", \"pydantic\"]\n\n# For resource URIs\nref = %{\"type\" => \"ref/resource\", \"uri\" => \"file:///\"}\narg = %{\"name\" => \"path\", \"value\" => \"doc\"}\n\n{:ok, response} = MyApp.MCPClient.complete(ref, arg)\n# => [\"docs/\", \"docker-compose.yml\", \"documentation.md\"]\n```","ref":"client_usage.html#autocompletion"},{"type":"extras","title":"Error Handling - Client Usage","doc":"See [Error Handling](error_handling.md) for patterns.","ref":"client_usage.html#error-handling"},{"type":"extras","title":"Graceful Shutdown - Client Usage","doc":"```elixir\nMyApp.MCPClient.close()\n```\n\nThis also shuts down the transport.","ref":"client_usage.html#graceful-shutdown"},{"type":"extras","title":"Advanced Usage - Client Usage","doc":"","ref":"client_usage.html#advanced-usage"},{"type":"extras","title":"Multiple Client Instances - Client Usage","doc":"You can run multiple instances of the same client with different names:\n\n```elixir\n# In your supervision tree\nchildren = [\n  {MyApp.MCPClient, \n   name: :client_one,\n   transport: {:stdio, command: \"server1\", args: []}},\n  {MyApp.MCPClient, \n   name: :client_two,\n   transport: {:stdio, command: \"server2\", args: []}}\n]\n\n# Use specific instances\nMyApp.MCPClient.ping(:client_one)\nMyApp.MCPClient.ping(:client_two)\n```","ref":"client_usage.html#multiple-client-instances"},{"type":"extras","title":"Custom Process Naming - Client Usage","doc":"For distributed systems using registries like Horde:\n\n```elixir\n{MyApp.MCPClient,\n name: {:via, Horde.Registry, {MyCluster, \"client_1\"}},\n transport_name: {:via, Horde.Registry, {MyCluster, \"transport_1\"}},\n transport: {:stdio, command: \"server\", args: []}}\n```","ref":"client_usage.html#custom-process-naming"},{"type":"extras","title":"Manual Client Setup (Advanced) - Client Usage","doc":"For advanced use cases, you can manually start the client base and transport:\n\n```elixir\n# Start client first (it will hibernate waiting for transport)\n{:ok, client_pid} = Hermes.Client.Base.start_link(\n  name: :my_client,\n  transport: [\n    layer: Hermes.Transport.STDIO,\n    name: :my_transport\n  ],\n  client_info: %{\n    \"name\" => \"MyApp\",\n    \"version\" => \"1.0.0\"\n  },\n  capabilities: %{\n    \"roots\" => %{\"listChanged\" => true},\n    \"sampling\" => %{}\n  },\n  protocol_version: \"2024-11-05\"\n)\n\n# Then start the transport (it will send :initialize to the client)\n{:ok, transport_pid} = Hermes.Transport.STDIO.start_link(\n  name: :my_transport,\n  client: :my_client,\n  command: \"python\",\n  args: [\"-m\", \"mcp.server\"]\n)\n\n# Use the client\n:pong = Hermes.Client.Base.ping(:my_client)\n{:ok, tools} = Hermes.Client.Base.list_tools(:my_client)\n```\n\n> #### Important {: .warning}\n>\n> The client must be started before the transport. The client process will hibernate \n> waiting for the `:initialize` message from the transport process. The client name \n> in the transport configuration must match the actual client process name.","ref":"client_usage.html#manual-client-setup-advanced"},{"type":"extras","title":"MCP Server Quick Start Guide","doc":"# MCP Server Quick Start Guide\n\nThis guide will help you create your first MCP server using Hermes in under 5 minutes.","ref":"server_quickstart.html"},{"type":"extras","title":"What is an MCP Server? - MCP Server Quick Start Guide","doc":"An MCP (Model Context Protocol) server provides tools, prompts, and resources to AI assistants. Think of it as an API that AI models can interact with to perform actions or retrieve information.","ref":"server_quickstart.html#what-is-an-mcp-server"},{"type":"extras","title":"Basic Server Setup - MCP Server Quick Start Guide","doc":"","ref":"server_quickstart.html#basic-server-setup"},{"type":"extras","title":"Step 1: Define Your Server Module - MCP Server Quick Start Guide","doc":"Create a new Elixir module that uses `Hermes.Server`:\n\n```elixir\ndefmodule MyApp.MCPServer do\n  use Hermes.Server,\n    name: \"My First MCP Server\",\n    version: \"1.0.0\",\n    capabilities: [:tools]\n\n  def start_link(opts) do\n    Hermes.Server.start_link(__MODULE__, :ok, opts)\n  end\n\n  @impl Hermes.Server.Behaviour\n  def init(:ok, frame) do\n    {:ok, frame}\n  end\nend\n```","ref":"server_quickstart.html#step-1-define-your-server-module"},{"type":"extras","title":"Step 2: Add to Your Application Supervisor - MCP Server Quick Start Guide","doc":"In your `application.ex`:\n\n```elixir\ndefmodule MyApp.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      # Registry to handle processes names\n      Hermes.Server.Registry,\n      # Start with STDIO transport (for local tests)\n      {MyApp.MCPServer, transport: :stdio}\n    ]\n\n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```","ref":"server_quickstart.html#step-2-add-to-your-application-supervisor"},{"type":"extras","title":"Step 3: Create Your First Tool - MCP Server Quick Start Guide","doc":"Tools are functions that AI assistants can call:\n\n```elixir\ndefmodule MyApp.MCPServer.Tools.Greeter do\n  @moduledoc \"Greet someone by name\"\n\n  use Hermes.Server.Component, type: :tool\n\n  alias Hermes.Server.Response\n\n  schema do\n    field :name, :string, required: true\n  end\n\n  @impl true\n  def execute(%{name: name}, frame) do\n    {:reply, Response.text(Response.tool(), \"Hello, #{name}! Welcome to MCP!\"), frame}\n  end\nend\n```","ref":"server_quickstart.html#step-3-create-your-first-tool"},{"type":"extras","title":"Step 4: Register the Tool - MCP Server Quick Start Guide","doc":"Update your server module to include the tool:\n\n```elixir\ndefmodule MyApp.MCPServer do\n  use Hermes.Server,\n    name: \"My First MCP Server\",\n    version: \"1.0.0\",\n    capabilities: [:tools]\n\n  def start_link(opts) do\n    Hermes.Server.start_link(__MODULE__, :ok, opts)\n  end\n\n  # Register your tool\n  component MyApp.MCPServer.Tools.Greeter\n\n  @impl Hermes.Server.Behaviour\n  def init(_arg, frame) do\n    {:ok, frame}\n  end\nend\n```","ref":"server_quickstart.html#step-4-register-the-tool"},{"type":"extras","title":"Testing Your Server - MCP Server Quick Start Guide","doc":"","ref":"server_quickstart.html#testing-your-server"},{"type":"extras","title":"Using the Interactive Shell - MCP Server Quick Start Guide","doc":"Hermes provides interactive mix tasks for testing:\n\n```bash\n# Test with STDIO transport\nmix help hermes.stdio.interactive\n\n# Test with HTTP transport\nmix help hermes.streamable_http.interactive\n```","ref":"server_quickstart.html#using-the-interactive-shell"},{"type":"extras","title":"Next Steps - MCP Server Quick Start Guide","doc":"- Explore the [Component System](server_components.md) for building tools, prompts, and resources\n- Configure different [Transport Options](server_transport.md)","ref":"server_quickstart.html#next-steps"},{"type":"extras","title":"MCP Server Components Guide","doc":"# MCP Server Components Guide\n\nThis guide shows you how to build Tools, Prompts, and Resources using Hermes component system.","ref":"server_components.html"},{"type":"extras","title":"Tools - MCP Server Components Guide","doc":"Tools are functions that AI assistants can call to perform actions.","ref":"server_components.html#tools"},{"type":"extras","title":"Basic Tool - MCP Server Components Guide","doc":"```elixir\ndefmodule MyServer.Tools.Calculator do\n  @moduledoc \"Add two numbers\"\n\n  use Hermes.Server.Component, type: :tool\n\n  alias Hermes.Server.Response\n\n  schema do\n    field :a, :float, required: true\n    field :b, :float, required: true\n  end\n\n  @impl true\n  def execute(%{a: a, b: b}, frame) do\n    {:reply, Response.text(Response.tool(), \"Result: #{a + b}\"), frame}\n  end\nend\n```","ref":"server_components.html#basic-tool"},{"type":"extras","title":"Schema with Field Metadata - MCP Server Components Guide","doc":"The `field` macro allows adding JSON Schema metadata like format and description:\n\n```elixir\ndefmodule MyServer.Tools.UserManager do\n  @moduledoc \"Manage user data\"\n\n  use Hermes.Server.Component, type: :tool\n\n  alias Hermes.Server.Response\n\n  schema do\n    field :email, :string, required: true, format: \"email\", description: \"User's email address\"\n    field :age, {:integer, {:range, {0, 150}}}, description: \"Age in years\"\n    field :website, :string, format: \"uri\"\n    \n    field :address, description: \"Mailing address\" do\n      field :street, :string, required: true\n      field :city, :string, required: true\n      field :postal_code, :string, format: \"postal-code\"\n      field :country, :string, description: \"ISO 3166-1 alpha-2 code\"\n    end\n  end\n\n  @impl true\n  def execute(params, frame) do\n    {:reply, Response.text(Response.tool(), \"User created: #{params.email}\"), frame}\n  end\nend\n```\n\nThe field metadata is included in the JSON Schema exposed to MCP clients, providing better documentation and validation hints.","ref":"server_components.html#schema-with-field-metadata"},{"type":"extras","title":"Tool with Error Handling - MCP Server Components Guide","doc":"```elixir\ndefmodule MyServer.Tools.Divider do\n  @moduledoc \"Divide two numbers\"\n\n  use Hermes.Server.Component, type: :tool\n\n  alias Hermes.Server.Response\n\n  schema do\n    field :a, :float, required: true\n    field :b, :float, required: true\n  end\n\n  @impl true\n  def execute(%{a: a, b: 0}, frame) do    \n    {:reply, Response.error(Response.tool(), \"Cannot divide by zero\"), frame}\n  end\n\n  def execute(%{a: a, b: b}, frame) do\n    {:reply, Response.text(Response.tool(), \"#{a} ÷ #{b} = #{a / b}\"), frame}\n  end\nend\n```","ref":"server_components.html#tool-with-error-handling"},{"type":"extras","title":"Tool with JSON Response - MCP Server Components Guide","doc":"```elixir\ndefmodule MyServer.Tools.SystemInfo do\n  @moduledoc \"Get system information\"\n\n  use Hermes.Server.Component, type: :tool\n\n  alias Hermes.Server.Response\n  \n  @impl true\n  def execute(_params, frame) do  \n    info = %{\n      os: :os.type(),\n      memory: :erlang.memory(:total),\n      processes: :erlang.system_info(:process_count)\n    }\n        \n    {:reply, Response.json(Response.tool(), info), frame}\n  end\nend\n```","ref":"server_components.html#tool-with-json-response"},{"type":"extras","title":"Prompts - MCP Server Components Guide","doc":"Prompts provide reusable message templates for AI conversations.","ref":"server_components.html#prompts"},{"type":"extras","title":"Basic Prompt - MCP Server Components Guide","doc":"```elixir\ndefmodule MyServer.Prompts.Assistant do\n  @moduledoc \"General assistant prompt\"\n\n  use Hermes.Server.Component, type: :prompt\n\n  alias Hermes.Server.Response\n\n  schema do\n    field :style, {:enum, [\"formal\", \"casual\"]}, required: true, type: :string\n  end\n\n  @impl true\n  def get_messages(%{style: \"formal\"}, frame) do    \n    response = \n      Response.prompt()\n      |> Response.system_message(\"You are a formal, professional assistant.\")\n      |> Response.user_message(\"Please help me with my task.\")\n    \n    {:reply, response, frame}\n  end\n\n  def get_messages(%{style: \"casual\"}, frame) do\n    response = \n      Response.prompt()\n      |> Response.system_message(\"You are a friendly, casual assistant.\")\n      |> Response.user_message(\"Hey, can you help me out?\")\n    \n    {:reply, response, frame}\n  end\nend\n```","ref":"server_components.html#basic-prompt"},{"type":"extras","title":"Prompt with Context - MCP Server Components Guide","doc":"```elixir\ndefmodule MyServer.Prompts.CodeReview do\n  @moduledoc \"Code review prompt\"\n\n  use Hermes.Server.Component, type: :prompt\n\n  schema do\n    field :language, :string, required: true, description: \"Programming language\"\n  end\n\n  @impl true\n  def get_messages(%{language: lang}, frame) do\n    response = \n      Response.prompt(\"Code review for #{lang}\")\n      |> Response.system_message(\"You are an expert #{lang} code reviewer.\")\n      |> Response.user_message(\"Please review the following #{lang} code for best practices and potential issues.\")\n    \n    {:reply, response, frame}\n  end\nend\n```","ref":"server_components.html#prompt-with-context"},{"type":"extras","title":"Prompt with Field Metadata - MCP Server Components Guide","doc":"When using prompts, you can add descriptions to arguments for better documentation in the MCP protocol:\n\n```elixir\ndefmodule MyServer.Prompts.DocumentAnalyzer do\n  @moduledoc \"Analyze and summarize documents\"\n\n  use Hermes.Server.Component, type: :prompt\n\n  alias Hermes.Server.Response\n\n  schema do\n    field :document, :string, required: true, description: \"The document text to analyze\"\n    field :language, :string, required: true, description: \"Document language (e.g., 'en', 'es', 'fr')\"\n    field :analysis_type, {:enum, [\"summary\", \"sentiment\", \"keywords\"]}, type: :string,\n          description: \"Type of analysis to perform\"\n    field :max_length, {:integer, {:default, 500}}, \n          description: \"Maximum length of the summary in characters\"\n  end\n\n  @impl true\n  def get_messages(params, frame) do\n    %{document: doc, language: lang, analysis_type: type, max_length: max_len} = params\n    \n    response = \n      Response.prompt()\n      |> Response.system_message(\"You are an expert document analyzer specializing in #{type} analysis.\")\n      |> Response.user_message(\"\"\"\n      Please analyze the following #{lang} document and provide a #{type}.\n      Maximum response length: #{max_len} characters.\n      \n      Document:\n      #{doc}\n      \"\"\")\n    \n    {:reply, response, frame}\n  end\nend\n```\n\nThis generates the following MCP arguments:\n\n```json\n{\n  \"arguments\": [\n    {\n      \"name\": \"document\",\n      \"description\": \"The document text to analyze\",\n      \"required\": true\n    },\n    {\n      \"name\": \"language\", \n      \"description\": \"Document language (e.g., 'en', 'es', 'fr')\",\n      \"required\": true\n    },\n    {\n      \"name\": \"analysis_type\",\n      \"description\": \"Type of analysis to perform\",\n      \"required\": false\n    },\n    {\n      \"name\": \"max_length\",\n      \"description\": \"Maximum length of the summary in characters\",\n      \"required\": false\n    }\n  ]\n}\n```","ref":"server_components.html#prompt-with-field-metadata"},{"type":"extras","title":"Resources - MCP Server Components Guide","doc":"Resources provide data that AI can read, identified by URIs.","ref":"server_components.html#resources"},{"type":"extras","title":"Text Resource - MCP Server Components Guide","doc":"```elixir\ndefmodule MyServer.Resources.Config do\n  @moduledoc \"Application configuration\"\n\n  use Hermes.Server.Component,\n    type: :resource,\n    uri: \"config://app\",\n    mime_type: \"application/json\"\n\n  alias Hermes.Server.Response\n\n  @impl true\n  def read(_params, frame) do\n    config = %{version: \"1.0.0\", env: Mix.env()}\n    \n    {:reply, Response.json(Response.resource(), config), frame}\n  end\nend\n```","ref":"server_components.html#text-resource"},{"type":"extras","title":"Binary Resource - MCP Server Components Guide","doc":"```elixir\ndefmodule MyServer.Resources.Logo do\n  @moduledoc \"Company logo\"\n\n  use Hermes.Server.Component,\n    type: :resource,\n    uri: \"assets://logo\",\n    mime_type: \"image/png\"\n\n  alias Hermes.MCP.Error  \n  alias Hermes.Server.Response\n\n  @impl true\n  def read(_params, frame) do\n    case File.read(\"priv/static/logo.png\") do\n      {:ok, binary} ->\n        {:reply, Response.blob(Response.resource(), Base.encode64(binary)), frame}\n      \n      {:error, reason} ->\n        {:error, Error.internal_error(%{reason: reason}), frame}\n    end\n  end\nend\n```","ref":"server_components.html#binary-resource"},{"type":"extras","title":"Response Builder - MCP Server Components Guide","doc":"The `Hermes.Server.Response` module provides a fluent API for building responses.","ref":"server_components.html#response-builder"},{"type":"extras","title":"Tool Responses - MCP Server Components Guide","doc":"```elixir\nimport Hermes.Server.Response\n\n# Text response\ntool() |> text(\"Hello!\") |> build()\n# => %{\"content\" => [%{\"type\" => \"text\", \"text\" => \"Hello!\"}], \"isError\" => false}\n\n# JSON response\ntool() |> json(%{status: \"ok\"}) |> build()\n# => %{\"content\" => [%{\"type\" => \"text\", \"text\" => \"{\\\"status\\\":\\\"ok\\\"}\"}], \"isError\" => false}\n\n# Error response\ntool() |> error(\"Something went wrong\") |> build()\n# => %{\"content\" => [%{\"type\" => \"text\", \"text\" => \"Something went wrong\"}], \"isError\" => true}\n\n# Multiple content items\ntool() \n|> text(\"Processing complete\")\n|> json(%{items: 10, processed: 10})\n|> build()\n```","ref":"server_components.html#tool-responses"},{"type":"extras","title":"Prompt Responses - MCP Server Components Guide","doc":"```elixir\nimport Hermes.Server.Response\n\n# Simple conversation\nprompt()\n|> user_message(\"What's the weather?\")\n|> assistant_message(\"Let me check that for you.\")\n|> build()\n# => %{\"messages\" => [\n#      %{\"role\" => \"user\", \"content\" => \"What's the weather?\"}, \n#      %{\"role\" => \"assistant\", \"content\" => \"Let me check that for you.\"}\n#    ]}\n\n# With system context\nprompt(\"Weather Assistant\")\n|> system_message(\"You are a helpful weather assistant.\")\n|> user_message(\"What's the forecast for tomorrow?\")\n|> build()\n```","ref":"server_components.html#prompt-responses"},{"type":"extras","title":"Resource Responses - MCP Server Components Guide","doc":"```elixir\nimport Hermes.Server.Response\n\n# Text resource\nresource() |> text(\"File contents here\") |> build()\n# => %{\"text\" => \"File contents here\"}\n\n# Binary resource  \nresource() |> blob(base64_data) |> build()\n# => %{\"blob\" => base64_data}\n\n# With metadata\nresource()\n|> text(\"Config data\")\n|> name(\"Application Config\")\n|> description(\"Current app configuration\")\n|> build()\n```","ref":"server_components.html#resource-responses"},{"type":"extras","title":"Registering Components - MCP Server Components Guide","doc":"In your server module:\n\n```elixir\ndefmodule MyServer do\n  use Hermes.Server,\n    name: \"My Server\",\n    version: \"1.0.0\",\n    capabilities: [:tools, :prompts, :resources]\n\n  # Register components\n  component MyServer.Tools.Calculator\n  component MyServer.Tools.Divider\n  component MyServer.Prompts.Assistant\n  component MyServer.Resources.Config\n  \n  @impl true\n  def init(_arg, frame) do\n    {:ok, frame}\n  end\nend\n```","ref":"server_components.html#registering-components"},{"type":"extras","title":"Using Frame State - MCP Server Components Guide","doc":"The `frame` parameter carries state through requests:\n\n```elixir\ndefmodule MyServer.Tools.Counter do\n  use Hermes.Server.Component, type: :tool\n\n  alias Hermes.Server.Response\n\n  schema do\n    field :increment, {:integer, {:default, 1}},\n      description: \"how much to increment\",\n      required: false\n  end\n\n  @impl true\n  def execute(%{increment: inc}, frame) do\n    # Get current count\n    count = frame.assigns[:count] || 0\n    new_count = count + inc\n    \n    # Update frame, if current count == 1, in the next tool call\n    # assuming inc == 2, then could would have value 3\n    new_frame = assign(frame, :count, new_count)\n    \n    {:reply, Response.text(Response.tool(), \"Count: #{new_count}\"), new_frame}\n  end\nend\n```","ref":"server_components.html#using-frame-state"},{"type":"extras","title":"Frame and Authorization - MCP Server Components Guide","doc":"For HTTP transports, `frame.assigns` inherits from `Plug.Conn.assigns`. Users are responsible for populating it with authentication data through their own Plug pipeline:\n\n```elixir\n# In your Phoenix/Plug pipeline\npipeline :authenticated_api do\n  plug MyApp.AuthPlug        # Sets conn.assigns[:current_user]\n  plug MyApp.PermissionsPlug # Sets conn.assigns[:permissions]\nend\n\nscope \"/mcp\" do\n  pipe_through :authenticated_api\n\n  forward \"/\", Hermes.Server.Transport.StreamableHTTP, server: MyServer\nend\n\n# In your MCP component\ndefmodule MyServer.Tools.SecureTool do\n  use Hermes.Server.Component, type: :tool\n\n  @impl true\n  def execute(params, frame) do\n    # Access auth data populated by your plugs\n    current_user = frame.assigns[:current_user]\n    permissions = frame.assigns[:permissions]\n    \n    if authorized?(current_user, permissions) do\n      # Tool logic here\n    else\n      {:error, \"Unauthorized\", frame}\n    end\n  end\nend\n```\n\nThe Frame provides access to:\n- `assigns` - User data from `conn.assigns` (authentication, business context)\n- `transport` - Request metadata (headers, query params, IP address)\n- `private` - MCP session data (session ID, client info, protocol version)\n- `request` - Current MCP request being processed","ref":"server_components.html#frame-and-authorization"},{"type":"extras","title":"Return Types - MCP Server Components Guide","doc":"All component callbacks must return one of:\n\n- `{:reply, response, frame}` - Success with response\n- `{:noreply, frame}` - Success without response or delayed response\n- `{:error, error, frame}` - Error response\n\nWhere:\n- `response` is built using `Hermes.Server.Response`\n- `error` can be a string or `Hermes.MCP.Error`\n- `frame` is the updated frame state","ref":"server_components.html#return-types"},{"type":"extras","title":"Schema Definition - MCP Server Components Guide","doc":"","ref":"server_components.html#schema-definition"},{"type":"extras","title":"Field Macro with Metadata - MCP Server Components Guide","doc":"For richer JSON Schema output, use the `field` macro:\n\n```elixir\nschema do\n  field :email, :string, required: true, format: \"email\", description: \"Contact email\"\n  field :phone, :string, format: \"phone\"\n  field :birth_date, :date, required: true, description: \"Date of birth\"\n  \n  field :preferences do\n    field :theme, {:enum, [\"light\", \"dark\"]}, type: :string, description: \"UI theme\"\n    field :notifications, :boolean, description: \"Email notifications\"\n  end\nend\n```","ref":"server_components.html#field-macro-with-metadata"},{"type":"extras","title":"Enum Fields with Type - MCP Server Components Guide","doc":"When using enum fields, you can specify the underlying type for proper JSON Schema generation:\n\n```elixir\nschema do\n  field :weight, :integer, required: true\n  field :unit, {:enum, [\"kg\", \"lb\"]}, required: true, type: :string\n  field :status, {:enum, [\"active\", \"inactive\", \"pending\"]}, type: :string, description: \"Current status\"\nend\n```\n\nThis generates the following JSON Schema:\n\n```json\n{\n  \"properties\": {\n    \"weight\": {\"type\": \"integer\"},\n    \"unit\": {\n      \"type\": \"string\",\n      \"enum\": [\"kg\", \"lb\"]\n    },\n    \"status\": {\n      \"type\": \"string\", \n      \"enum\": [\"active\", \"inactive\", \"pending\"],\n      \"description\": \"Current status\"\n    }\n  },\n  \"required\": [\"weight\", \"unit\"]\n}\n```\n\nSupported metadata options:\n- `format`: JSON Schema format hint (email, uri, date, date-time, phone, etc.)\n- `description`: Human-readable field description\n- `type`: Explicit type for enum fields (string, integer, etc.)\n- `required`: Boolean indicating if field is required (cleaner than `{:required, type}`)\n\nBoth schema styles work together - choose based on whether you need JSON Schema metadata.","ref":"server_components.html#enum-fields-with-type"},{"type":"extras","title":"Supported Types - MCP Server Components Guide","doc":"Hermes provides automatic type conversion between JSON and Elixir types through its schema system.","ref":"server_components.html#supported-types"},{"type":"extras","title":"Basic Types - MCP Server Components Guide","doc":"| Type | JSON Input | Elixir Output | Example |\n|------|------------|---------------|---------|\n| `:string` | `\"hello\"` | `\"hello\"` | Basic string |\n| `:integer` | `42` | `42` | Number without decimals |\n| `:float` | `3.14` | `3.14` | Number with decimals |\n| `:boolean` | `true` | `true` | Boolean value |\n| `:any` | Any valid JSON | As-is | Accepts any value |","ref":"server_components.html#basic-types"},{"type":"extras","title":"Date and Time Types - MCP Server Components Guide","doc":"Hermes automatically parses ISO 8601 formatted strings into Elixir date/time structs:\n\n| Type | JSON Input | Elixir Output | Example |\n|------|------------|---------------|---------|\n| `:date` | `\"2024-01-15\"` | `~D[2024-01-15]` | ISO 8601 date |\n| `:time` | `\"14:30:00\"` | `~T[14:30:00]` | ISO 8601 time |\n| `:datetime` | `\"2024-01-15T14:30:00Z\"` | `~U[2024-01-15 14:30:00Z]` | ISO 8601 datetime with timezone |\n| `:naive_datetime` | `\"2024-01-15T14:30:00\"` | `~N[2024-01-15 14:30:00]` | ISO 8601 datetime without timezone |\n\nExample usage:\n\n```elixir\ndefmodule MyServer.Tools.EventScheduler do\n  use Hermes.Server.Component, type: :tool\n\n  schema do\n    field :event_date, :date, required: true, description: \"Date of the event\"\n    field :start_time, :time, required: true, description: \"Event start time\"\n    field :created_at, :datetime, description: \"When the event was created\"\n  end\n\n  @impl true\n  def execute(%{event_date: date, start_time: time} = params, frame) do\n    # date is already a Date struct: ~D[2024-01-15]\n    # time is already a Time struct: ~T[14:30:00]\n    \n    {:reply, Response.text(Response.tool(), \"Event scheduled for #{date} at #{time}\"), frame}\n  end\nend\n```","ref":"server_components.html#date-and-time-types"},{"type":"extras","title":"Collection Types - MCP Server Components Guide","doc":"| Type | JSON Input | Elixir Output | Example |\n|------|------------|---------------|---------|\n| `{:list, type}` | `[...]` | `[...]` | List of specified type |\n| `{:map, type}` | `{...}` | `%{...}` | Map with string keys |","ref":"server_components.html#collection-types"},{"type":"extras","title":"Constraint Types - MCP Server Components Guide","doc":"| Type | Description | Example |\n|------|-------------|---------|\n| `{:enum, choices}` | Value must be one of the choices | `{:enum, [\"active\", \"inactive\"]}` * |\n| `{:string, {:min, n}}` | String with minimum length | `{:string, {:min, 3}}` |\n| `{:string, {:max, n}}` | String with maximum length | `{:string, {:max, 100}}` |\n| `{:integer, {:min, n}}` | Integer >= n | `{:integer, {:min, 0}}` |\n| `{:integer, {:max, n}}` | Integer <= n | `{:integer, {:max, 100}}` |\n| `{:integer, {:range, {min, max}}}` | Integer in range | `{:integer, {:range, {1, 100}}}` |\n\n\\* Note: When using enum with the field macro, add `type: :string` for proper JSON Schema generation.","ref":"server_components.html#constraint-types"},{"type":"extras","title":"Error Handling - MCP Server Components Guide","doc":"When validation fails, Hermes provides clear error messages:\n\n- Invalid date format: `\"invalid ISO 8601 date format\"`\n- Missing required field: `\"is required\"`\n- Type mismatch: Detailed error with expected type","ref":"server_components.html#error-handling"},{"type":"extras","title":"Best Practices - MCP Server Components Guide","doc":"1. Use specific types (`:date`, `:datetime`) instead of generic `:string` when dealing with temporal data\n2. Add descriptions to fields for better API documentation\n3. Use the `required: true` option instead of wrapping types with `{:required, type}`\n4. Leverage enums for fields with known valid values\n5. Use appropriate format hints for strings (email, uri, etc.)","ref":"server_components.html#best-practices"},{"type":"extras","title":"Next Steps - MCP Server Components Guide","doc":"- Configure [Transport Options](server_transport.md) for different connection types","ref":"server_components.html#next-steps"},{"type":"extras","title":"Server Transport Configuration","doc":"# Server Transport Configuration\n\nHermes MCP servers support multiple transport mechanisms to accept connections from clients. This page covers the available options and how to configure them.","ref":"server_transport.html"},{"type":"extras","title":"Available Transports - Server Transport Configuration","doc":"| Transport | Module | Use Case | Multi-Client |\n|-----------|--------|----------|--------------|\n| **STDIO** | `Hermes.Server.Transport.STDIO` | CLI tools, local scripts | No |\n| **StreamableHTTP** | `Hermes.Server.Transport.StreamableHTTP` | Web apps, HTTP APIs | Yes |","ref":"server_transport.html#available-transports"},{"type":"extras","title":"STDIO Transport - Server Transport Configuration","doc":"The STDIO transport enables communication through standard input/output streams. It's suitable for local integrations and CLI tools.","ref":"server_transport.html#stdio-transport"},{"type":"extras","title":"Configuration - Server Transport Configuration","doc":"```elixir\n# Start server with STDIO transport\n{MyServer, transport: :stdio}\n\n# With explicit configuration\n{MyServer, transport: {:stdio, name: :my_stdio_server}}\n```","ref":"server_transport.html#configuration"},{"type":"extras","title":"Example: CLI Tool Server - Server Transport Configuration","doc":"```elixir\ndefmodule MyApp.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      {MyApp.MCPServer, transport: :stdio}\n    ]\n\n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```","ref":"server_transport.html#example-cli-tool-server"},{"type":"extras","title":"StreamableHTTP Transport - Server Transport Configuration","doc":"The StreamableHTTP transport enables communication over HTTP using Server-Sent Events (SSE) for responses. It supports multiple concurrent clients.","ref":"server_transport.html#streamablehttp-transport"},{"type":"extras","title":"Configuration - Server Transport Configuration","doc":"```elixir\n# Basic configuration\n{MyServer, transport: :streamable_http}\n\n# With port configuration\n{MyServer, transport: {:streamable_http, port: 8080}}\n\n# Full configuration\n{MyServer, transport: {:streamable_http,\n  port: 8080,\n  path: \"/mcp\",\n  start: true  # Force start even without HTTP server\n}}\n```","ref":"server_transport.html#configuration-1"},{"type":"extras","title":"Configuration Options - Server Transport Configuration","doc":"| Option | Type | Description | Default |\n|--------|------|-------------|---------|\n| `:port` | integer | Port number for HTTP server | `8080` |\n| `:path` | string | URL path for MCP endpoint | `\"/mcp\"` |\n| `:start` | boolean/`:auto` | Start behavior | `:auto` |","ref":"server_transport.html#configuration-options"},{"type":"extras","title":"Conditional Startup - Server Transport Configuration","doc":"StreamableHTTP transport has smart startup behavior:\n\n```elixir\n# Auto-detect (default) - starts only if HTTP server is running\n{MyServer, transport: :streamable_http}\n\n# Force start - always start HTTP server\n{MyServer, transport: {:streamable_http, start: true}}\n\n# Prevent start - never start, useful for tests\n{MyServer, transport: {:streamable_http, start: false}}\n```","ref":"server_transport.html#conditional-startup"},{"type":"extras","title":"Integration with Phoenix - Server Transport Configuration","doc":"If you're using Phoenix, you can integrate the MCP server with your existing endpoint:\n\n```elixir\n# In lib/my_app_web/endpoint.ex\ndefmodule MyAppWeb.Endpoint do\n  use Phoenix.Endpoint, otp_app: :my_app\n\n  # Add the MCP plug\n  plug Hermes.Server.Transport.StreamableHTTP.Plug,\n    server: MyApp.MCPServer,\n    path: \"/mcp\"\n  \n  # Other plugs...\nend\n```","ref":"server_transport.html#integration-with-phoenix"},{"type":"extras","title":"Transport Selection - Server Transport Configuration","doc":"","ref":"server_transport.html#transport-selection"},{"type":"extras","title":"Use STDIO when: - Server Transport Configuration","doc":"- Building CLI tools\n- Local development and testing\n- Single-client scenarios\n- Subprocess communication","ref":"server_transport.html#use-stdio-when"},{"type":"extras","title":"Use StreamableHTTP when: - Server Transport Configuration","doc":"- Building web applications\n- Need multiple concurrent clients\n- Integrating with existing HTTP services\n- Production deployments","ref":"server_transport.html#use-streamablehttp-when"},{"type":"extras","title":"Supervision - Server Transport Configuration","doc":"Transports are supervised as part of the server supervision tree:\n\n```elixir\n# The server supervisor handles both the server and transport\nchildren = [\n  {MyApp.MCPServer, transport: :stdio}\n]\n\nSupervisor.start_link(children, strategy: :one_for_one)\n```","ref":"server_transport.html#supervision"},{"type":"extras","title":"Custom Transport Options - Server Transport Configuration","doc":"For advanced configurations, you can specify the transport module directly:\n\n```elixir\n{MyServer, transport: [\n  layer: Hermes.Server.Transport.STDIO,\n  name: :custom_stdio\n  # Additional options...\n]}\n```","ref":"server_transport.html#custom-transport-options"},{"type":"extras","title":"References - Server Transport Configuration","doc":"For more information about MCP transport layers, see the official [MCP specification](https://spec.modelcontextprotocol.io/specification/basic/transports/)","ref":"server_transport.html#references"},{"type":"extras","title":"Hermes Server Architecture","doc":"# Hermes Server Architecture\n\nThis guide provides a comprehensive overview of the Hermes server architecture, its design principles, and how components interact to implement the Model Context Protocol (MCP).","ref":"server_architecture.html"},{"type":"extras","title":"Overview - Hermes Server Architecture","doc":"The Hermes server architecture follows a layered design that separates concerns and provides flexibility for different transport mechanisms. At its core, it implements the MCP specification while providing an ergonomic API for Elixir developers.","ref":"server_architecture.html#overview"},{"type":"extras","title":"Architecture Layers - Hermes Server Architecture","doc":"The server architecture is organized into distinct layers:","ref":"server_architecture.html#architecture-layers"},{"type":"extras","title":"User Layer - Hermes Server Architecture","doc":"- **User Code**: Your server implementation using `Hermes.Server`\n- **Components**: Tools, prompts, and resources that define server capabilities","ref":"server_architecture.html#user-layer"},{"type":"extras","title":"API Layer - Hermes Server Architecture","doc":"- **Hermes.Server**: Macro DSL that simplifies server creation with declarative syntax","ref":"server_architecture.html#api-layer"},{"type":"extras","title":"Core Layer - Hermes Server Architecture","doc":"- **Hermes.Server.Base**: The protocol engine that handles MCP compliance\n- **Registry**: Process registry for looking up server components\n- **Session.Supervisor**: Manages session lifecycles for HTTP transports\n- **Session Agents**: Individual session state containers","ref":"server_architecture.html#core-layer"},{"type":"extras","title":"Transport Layer - Hermes Server Architecture","doc":"- **STDIO Transport**: For CLI tools and subprocess communication\n- **StreamableHTTP Transport**: For web applications with multiple clients\n- **WebSocket Transport**: For real-time bidirectional communication","ref":"server_architecture.html#transport-layer"},{"type":"extras","title":"Core Components - Hermes Server Architecture","doc":"","ref":"server_architecture.html#core-components"},{"type":"extras","title":"Hermes.Server (API Layer) - Hermes Server Architecture","doc":"The `Hermes.Server` module is the primary developer interface. It provides:\n\n- **Macro DSL**: Simplifies server implementation with declarative syntax\n- **Component Registration**: Manages tools, prompts, and resources\n- **Default Implementations**: Provides sensible defaults for common MCP methods\n- **Compile-time Validation**: Ensures server configuration is valid\n\nWhen you `use Hermes.Server`, it:\n1. Injects the behavior callbacks\n2. Sets up component registration\n3. Provides import conveniences\n4. Generates metadata functions (server_info, capabilities, etc.)\n5. Delegates actual request handling to the Base server","ref":"server_architecture.html#hermes-server-api-layer"},{"type":"extras","title":"Hermes.Server.Base (Protocol Engine) - Hermes Server Architecture","doc":"The Base server is the protocol implementation engine. It handles:\n\n- **Message Processing**: Decodes JSON-RPC messages and routes them appropriately\n- **Protocol Compliance**: Ensures MCP specification adherence\n- **Session Orchestration**: Creates and manages sessions for HTTP transports\n- **Error Handling**: Provides consistent error responses\n- **Lifecycle Management**: Handles initialization, shutdown, and state transitions\n\nThe Base server maintains minimal state:\n- Reference to the user's server module\n- Transport configuration\n- Session registry (for HTTP transports)\n- Server metadata (capabilities, versions)","ref":"server_architecture.html#hermes-server-base-protocol-engine"},{"type":"extras","title":"Transport Layer - Hermes Server Architecture","doc":"Transports handle the actual communication with clients:\n\n```mermaid\nsequenceDiagram\n    participant C as Client\n    participant T as Transport\n    participant B as Base Server\n    participant U as User Server\n    \n    C->>T: Raw message\n    T->>T: Parse/Frame message\n    T->>B: {:request, decoded, session_id, context}\n    B->>B: Validate & Session lookup\n    B->>U: handle_request(request, frame)\n    U->>B: {:reply, response, frame}\n    B->>T: Encoded response\n    T->>C: Raw response\n```","ref":"server_architecture.html#transport-layer-1"},{"type":"extras","title":"Message Flow - Hermes Server Architecture","doc":"","ref":"server_architecture.html#message-flow"},{"type":"extras","title":"Request Processing Pipeline - Hermes Server Architecture","doc":"1. **Transport Reception**: Transport receives raw bytes/text from client\n2. **Message Framing**: Transport extracts complete JSON-RPC messages\n3. **Base Server Routing**: Transport calls Base with decoded message and metadata\n4. **Session Association**: Base attaches or creates session (HTTP only)\n5. **Protocol Validation**: Base validates message structure and session state\n6. **Business Logic**: Base calls user's handle_request/handle_notification\n7. **Response Encoding**: Base encodes the response as JSON-RPC\n8. **Transport Delivery**: Transport sends encoded response to client","ref":"server_architecture.html#request-processing-pipeline"},{"type":"extras","title":"Lifecycle Management - Hermes Server Architecture","doc":"The server follows a well-defined lifecycle:\n\n1. **Uninitialized**: Server starts and awaits client connection\n2. **Initializing**: Client sends `initialize` request, protocol negotiation occurs\n3. **Initialized**: Client confirms with `initialized` notification\n4. **Active**: Normal operation handling requests and notifications\n5. **Terminating**: Graceful shutdown with cleanup","ref":"server_architecture.html#lifecycle-management"},{"type":"extras","title":"Session Management - Hermes Server Architecture","doc":"","ref":"server_architecture.html#session-management"},{"type":"extras","title":"Understanding MCP Sessions - Hermes Server Architecture","doc":"Sessions in MCP are **exclusively for HTTP-based transports**. This design reflects fundamental differences between transport types:\n\n#### STDIO Transport\n- **No sessions needed**: Process lifecycle = session boundary\n- **1:1 relationship**: One client launches one server subprocess\n- **Natural isolation**: OS process isolation provides security\n- **State persistence**: Process memory serves as session state\n\n#### HTTP Transport\n- **Sessions required**: HTTP is stateless by design\n- **N:1 relationship**: Multiple clients connect to one server\n- **Explicit boundaries**: Sessions provide isolation between clients\n- **State management**: Sessions persist state across requests","ref":"server_architecture.html#understanding-mcp-sessions"},{"type":"extras","title":"Session Architecture - Hermes Server Architecture","doc":"For HTTP transports, Hermes implements a sophisticated session management system where:\n\n- **Base Server** maintains a session registry mapping session IDs to processes\n- **Session Supervisor** dynamically spawns session agents as clients connect\n- **Session Agents** store per-client state in isolated processes\n- **Request Routing** ensures each request reaches the correct session based on the `Mcp-Session-Id` header","ref":"server_architecture.html#session-architecture"},{"type":"extras","title":"Session Lifecycle - Hermes Server Architecture","doc":"1. **Creation**: First request triggers session creation\n2. **Association**: Session ID returned in `Mcp-Session-Id` header\n3. **Persistence**: Client includes session ID in subsequent requests\n4. **Isolation**: Each session maintains independent state\n5. **Expiration**: Sessions expire after 30 minutes of inactivity (configurable)\n6. **Termination**: Explicit close, timeout, or transport failure","ref":"server_architecture.html#session-lifecycle"},{"type":"extras","title":"Session Expiration - Hermes Server Architecture","doc":"Sessions automatically expire after a period of inactivity to prevent resource leaks:\n\n- **Default timeout**: 30 minutes\n- **Configuration**: Set `session_idle_timeout` when starting server\n- **Timer reset**: Each request/notification resets the session's expiry timer\n- **Cleanup**: Expired sessions are terminated gracefully\n\n```elixir\n# Configure custom session timeout\n{MyServer, \n  transport: {:streamable_http, port: 8080},\n  session_idle_timeout: :timer.minutes(15)\n}\n```","ref":"server_architecture.html#session-expiration"},{"type":"extras","title":"Why Sessions Matter - Hermes Server Architecture","doc":"Sessions enable critical features for production deployments:\n\n- **Multi-tenancy**: Isolate different clients on shared infrastructure\n- **Stateful Operations**: Maintain context for multi-step workflows\n- **Resource Management**: Track and cleanup client-specific resources\n- **Security Boundaries**: Enforce access control per session\n- **Scalability**: Support load balancing with session affinity","ref":"server_architecture.html#why-sessions-matter"},{"type":"extras","title":"Frame: Request Context - Hermes Server Architecture","doc":"The Frame is a data structure that flows through request processing, providing a clean abstraction over transport and session details. It contains:\n\n- **assigns**: User data populated by middleware (authentication, authorization)\n- **transport**: Connection metadata that varies by transport type\n  - HTTP: headers, query params, IP address, scheme, host, port\n  - STDIO: environment variables, process ID\n- **private**: MCP session data (session ID, client info, protocol version)\n- **request**: Current request being processed (ID, method, params)\n\nThe Frame flows from transport → Base Server → User Server → Components, accumulating context at each layer while maintaining immutability.","ref":"server_architecture.html#frame-request-context"},{"type":"extras","title":"Supervision Tree - Hermes Server Architecture","doc":"The supervision strategy ensures fault tolerance through a hierarchical process structure:\n\n**Application Level**\n- Registry (global process registry)\n- Server Supervisor (manages server instances)\n\n**Server Level** (under Server Supervisor)\n- Base Server (protocol engine)\n- Session Supervisor (DynamicSupervisor for sessions)\n- Transport (communication layer)\n\n**Session Level** (under Session Supervisor)\n- Individual session agents spawned dynamically\n- Isolated processes with `:transient` restart strategy\n\nThis architecture provides:\n- **Fault Isolation**: Session crashes don't affect other sessions\n- **Resource Cleanup**: Automatic cleanup on process termination\n- **Restart Strategies**: Transient restarts for recoverable errors\n- **Graceful Shutdown**: Ordered termination sequence","ref":"server_architecture.html#supervision-tree"},{"type":"extras","title":"Error Handling - Hermes Server Architecture","doc":"Hermes provides structured error handling through the `Hermes.MCP.Error` module with standardized categories:\n\n- **Protocol Errors**: JSON-RPC violations, invalid requests, method not found\n- **Transport Errors**: Connection failures, send failures, timeouts\n- **Domain Errors**: Business logic failures from your server implementation\n- **Client Errors**: Invalid parameters, unauthorized access\n\nEach error includes:\n- Standard error code (following JSON-RPC conventions)\n- Human-readable message\n- Optional additional data for debugging\n\nErrors are automatically formatted as proper JSON-RPC error responses and logged with appropriate severity levels.","ref":"server_architecture.html#error-handling"},{"type":"extras","title":"Performance Considerations - Hermes Server Architecture","doc":"","ref":"server_architecture.html#performance-considerations"},{"type":"extras","title":"Optimization Strategies - Hermes Server Architecture","doc":"1. **Process Hibernation**: Reduce memory footprint for idle processes\n2. **Lazy Session Creation**: Only create sessions when needed\n3. **Efficient Message Routing**: Direct routing without intermediate processes\n4. **Batch Processing**: Support for JSON-RPC batch requests (2025-03-26+)","ref":"server_architecture.html#optimization-strategies"},{"type":"extras","title":"Scalability Patterns - Hermes Server Architecture","doc":"For production deployments:\n\n- **Horizontal Scaling**: Multiple server instances behind load balancer\n- **Session Affinity**: Sticky sessions for stateful operations\n- **Resource Pooling**: Shared resources across sessions\n- **Monitoring**: Telemetry events for observability","ref":"server_architecture.html#scalability-patterns"},{"type":"extras","title":"Related Documentation - Hermes Server Architecture","doc":"- [Server Components](server_components.md) - Building tools, resources, and prompts\n- [Server Quick Start](server_quickstart.md) - Getting started guide\n- [Server Transport](server_transport.md) - Transport configuration details\n- [Error Handling](error_handling.md) - Comprehensive error handling guide","ref":"server_architecture.html#related-documentation"},{"type":"extras","title":"Message Handling","doc":"# Message Handling\n\nHow Hermes handles MCP protocol messages.","ref":"message_handling.html"},{"type":"extras","title":"Message Types - Message Handling","doc":"MCP uses JSON-RPC 2.0 with three message types:\n\n1. **Requests** - Expect a response (have ID)\n2. **Responses** - Reply to requests (match ID)\n3. **Notifications** - One-way messages (no ID)","ref":"message_handling.html#message-types"},{"type":"extras","title":"Message Flow - Message Handling","doc":"```\nClient ──request──> Transport ──> Server\n       <─response── Transport <── \n```","ref":"message_handling.html#message-flow"},{"type":"extras","title":"Message Encoding - Message Handling","doc":"Use `Hermes.MCP.Message` for all message operations:\n\n```elixir\n# Encode request\n{:ok, encoded} = Message.encode_request(%{\n  method: \"tools/call\",\n  params: %{name: \"calculator\", arguments: %{}}\n}, \"req_123\")\n\n# Encode response\n{:ok, encoded} = Message.encode_response(%{\n  result: %{answer: 42}\n}, \"req_123\")\n\n# Encode notification\n{:ok, encoded} = Message.encode_notification(%{\n  method: \"cancelled\",\n  params: %{requestId: \"req_123\"}\n})\n\n# Decode any message\n{:ok, [decoded]} = Message.decode(json_string)\n```","ref":"message_handling.html#message-encoding"},{"type":"extras","title":"Message Guards - Message Handling","doc":"Check message types:\n\n```elixir\ncase decoded do\n  msg when Message.is_request(msg) ->\n    handle_request(msg)\n  \n  msg when Message.is_response(msg) ->\n    handle_response(msg)\n    \n  msg when Message.is_notification(msg) ->\n    handle_notification(msg)\nend\n```","ref":"message_handling.html#message-guards"},{"type":"extras","title":"Request IDs - Message Handling","doc":"Hermes generates unique request IDs:\n\n```elixir\nid = Hermes.MCP.ID.generate_request_id()\n# => \"req_hyp_abcd1234efgh5678\"\n```","ref":"message_handling.html#request-ids"},{"type":"extras","title":"Timeouts - Message Handling","doc":"Requests timeout after 30 seconds by default:\n\n```elixir\n# Custom timeout\nMyApp.MCPClient.call_tool(\"slow_tool\", %{}, timeout: 60_000)\n```\n\nOn timeout:\n1. Request removed from pending\n2. Cancellation sent to server\n3. Error returned to caller","ref":"message_handling.html#timeouts"},{"type":"extras","title":"Common Patterns - Message Handling","doc":"","ref":"message_handling.html#common-patterns"},{"type":"extras","title":"Client Side - Message Handling","doc":"```elixir\n# The client handles correlation automatically\n{:ok, response} = MyApp.MCPClient.call_tool(\"tool\", %{})\n```","ref":"message_handling.html#client-side"},{"type":"extras","title":"Server Side - Message Handling","doc":"```elixir\ndef handle_request(%{\"method\" => method, \"id\" => id}, frame) do\n  result = process_method(method)\n  {:reply, result, frame}\nend\n\ndef handle_notification(%{\"method\" => \"cancelled\"}, frame) do\n  # Cancel any running operation\n  {:noreply, frame}\nend\n```","ref":"message_handling.html#server-side"},{"type":"extras","title":"Progress Tracking","doc":"# Progress Tracking\n\nHermes MCP supports progress notifications for long-running operations as specified in the [MCP protocol](https://spec.modelcontextprotocol.io/specification/2024-11-05/basic/utilities/progress/).","ref":"progress_tracking.html"},{"type":"extras","title":"Overview - Progress Tracking","doc":"The MCP specification includes a progress notification mechanism that allows communicating the progress of long-running operations. Progress updates are useful for:\n\n- Providing feedback for long-running server operations\n- Updating users about the status of operations that might take time to complete\n- Enabling client applications to display progress indicators for better UX","ref":"progress_tracking.html#overview"},{"type":"extras","title":"Progress Tokens - Progress Tracking","doc":"Progress is tracked using tokens that uniquely identify a specific operation. Hermes provides a helper function to generate unique tokens:\n\n```elixir\n# Generate a unique progress token\nprogress_token = Hermes.MCP.ID.generate_progress_token()\n```","ref":"progress_tracking.html#progress-tokens"},{"type":"extras","title":"Making Requests with Progress Tracking - Progress Tracking","doc":"Any client API request can include progress tracking options. Internally, these options are encapsulated in the `Operation` struct, which standardizes how client operations are handled:\n\n```elixir\n# Make a request with progress tracking\nMyApp.MCPClient.read_resource(\"resource-uri\",\n  progress: [token: progress_token]\n)\n```","ref":"progress_tracking.html#making-requests-with-progress-tracking"},{"type":"extras","title":"Internal Operation Structure - Progress Tracking","doc":"Behind the scenes, client methods create an `Operation` struct that encapsulates the method, parameters, progress options, and timeout:\n\n```elixir\n# This is what happens internally in client methods\noperation = Operation.new(%{\n  method: \"resources/read\",\n  params: %{\"uri\" => uri},\n  progress_opts: [token: progress_token, callback: callback],\n  timeout: custom_timeout\n})\n```\n\nThe `Operation` struct provides a standardized way to handle all client requests with consistent timeout and progress tracking behavior.","ref":"progress_tracking.html#internal-operation-structure"},{"type":"extras","title":"Receiving Progress Updates - Progress Tracking","doc":"You can combine the token and callback in a single request:\n\n```elixir\ntoken = Hermes.MCP.ID.generate_progress_token()\n\ncallback = fn ^token, progress, total ->\n  IO.puts(\"Progress: #{progress}/#{total || \"unknown\"}\")\nend\n\nMyApp.MCPClient.list_tools(progress: [token: token, callback: callback])\n```","ref":"progress_tracking.html#receiving-progress-updates"},{"type":"extras","title":"Logging","doc":"# Logging\n\nHow does Hermes MCP handle the logging capabilities defined in the MCP protocol? Let's explore both the MCP logging features and how to configure Hermes's own internal logging.","ref":"logging.html"},{"type":"extras","title":"MCP Logging Capability - Logging","doc":"When building MCP clients, you'll often need to receive and process log messages from servers. How might you handle these structured messages in your Elixir application?","ref":"logging.html#mcp-logging-capability"},{"type":"extras","title":"Controlling Server Verbosity - Logging","doc":"```elixir\n{:ok, _} = MyApp.MCPClient.set_log_level(\"info\")\n```\n\nWhat happens behind the scenes? This sends a `logging/setLevel` request to the server, telling it to only send logs at \"info\" level or higher. The server will respect this preference when deciding which notifications to send your way.","ref":"logging.html#controlling-server-verbosity"},{"type":"extras","title":"Processing Incoming Logs - Logging","doc":"Ever wondered how to handle those log notifications when they arrive? You have two approaches:\n\n```elixir\n# Register a custom handler\nMyApp.MCPClient.register_log_callback(fn level, data, logger ->\n  # Send to your monitoring service?\n  # Transform the format?\n  # Filter by logger name?\nend)\n\n# When done, clean up\nMyApp.MCPClient.unregister_log_callback()\n```\n\nWhat's happening by default though? Hermes automatically forwards all MCP log messages to Elixir's Logger, mapping the levels appropriately. This means your existing log infrastructure just works - no additional setup required.","ref":"logging.html#processing-incoming-logs"},{"type":"extras","title":"Configuring Hermes Library Logs - Logging","doc":"Now, what about the logs that Hermes itself generates? How do you control the verbosity of the library's internal logging?","ref":"logging.html#configuring-hermes-library-logs"},{"type":"extras","title":"Fine-Grained Control - Logging","doc":"```elixir\nconfig :hermes_mcp, :logging,\n  client_events: :info,\n  server_events: :debug,\n  transport_events: :warning,\n  protocol_messages: :debug\n```\n\nWhat do these categories mean for your debugging experience?\n- `client_events`: Client lifecycle and operations\n- `server_events`: Server request handling and responses  \n- `transport_events`: Connection and transport layer activity\n- `protocol_messages`: Raw MCP message exchanges","ref":"logging.html#fine-grained-control"},{"type":"extras","title":"Global Toggle - Logging","doc":"Need to silence Hermes completely? Perhaps for performance testing or production deployments?\n\n```elixir\nconfig :hermes_mcp, log: false\n```\n\nThis disables all library-emitted logs, regardless of individual category settings.","ref":"logging.html#global-toggle"},{"type":"extras","title":"MCP Server Logging - Logging","doc":"What if you're building an MCP server? How do you send logs to connected clients? Let's explore the server-side capabilities.","ref":"logging.html#mcp-server-logging"},{"type":"extras","title":"Declaring Logging Support - Logging","doc":"```elixir\ndefmodule MyServer do\n  use Hermes.Server,\n    name: \"my-server\",\n    version: \"1.0.0\",\n    capabilities: [:logging]  # Enable logging capability\nend\n```\n\nWhat does this capability tell clients? It signals that your server can receive `logging/setLevel` requests and will respect the client's verbosity preferences when sending log notifications.","ref":"logging.html#declaring-logging-support"},{"type":"extras","title":"Sending Log Messages - Logging","doc":"How do you actually send logs from your server to clients? You have a straightforward API:\n\n```elixir\n# From within your server callbacks\nsend_log_message(server, \"info\", \"Processing started\", \"worker\")\n\n# Or without a logger name\nsend_log_message(server, \"error\", \"Connection failed\")\n```\n\nWhat happens when you send these? They become `notifications/message` events that clients process according to their configured handlers. The client's minimum log level (set via `set_log_level/1`) filters what they receive.","ref":"logging.html#sending-log-messages"},{"type":"extras","title":"Progress Notifications - Logging","doc":"Need to report progress on long-running operations? How might this improve user experience?\n\n```elixir\n# Start an operation with progress tracking\nsend_progress(server, \"import-123\", 0, 100, \"Starting import...\")\n\n# Update as you go\nsend_progress(server, \"import-123\", 45, 100, \"Processing records...\")\n\n# Complete\nsend_progress(server, \"import-123\", 100, 100, \"Import complete!\")\n```","ref":"logging.html#progress-notifications"},{"type":"extras","title":"Practical Considerations - Logging","doc":"How might you configure logging for different environments? Consider starting with `:debug` everywhere during development, then adjusting based on what you learn:\n\n- Keep `protocol_messages` at `:debug` unless debugging protocol issues\n- Set `transport_events` to `:warning` once your transport is stable\n- Use `:info` for client and server events in production\n\nWhat patterns emerge from your logs? Which categories provide the most value for your use case?","ref":"logging.html#practical-considerations"},{"type":"extras","title":"Error Handling","doc":"# Error Handling\n\nThis guide explains how errors work in Hermes MCP.","ref":"error_handling.html"},{"type":"extras","title":"Error Types - Error Handling","doc":"Hermes distinguishes between:\n\n1. **Protocol errors** - Standard JSON-RPC errors\n2. **Domain errors** - Application-level errors (tool returns `isError: true`)\n3. **Transport errors** - Network/connection failures","ref":"error_handling.html#error-types"},{"type":"extras","title":"Creating Errors - Error Handling","doc":"Hermes provides a fluent API for creating errors:\n\n```elixir\n# Protocol errors\nHermes.MCP.Error.protocol(:parse_error)\nHermes.MCP.Error.protocol(:method_not_found, %{method: \"unknown\"})\n\n# Transport errors\nHermes.MCP.Error.transport(:connection_refused)\nHermes.MCP.Error.transport(:timeout, %{elapsed_ms: 30000})\n\n# Resource errors\nHermes.MCP.Error.resource(:not_found, %{uri: \"file:///missing.txt\"})\n\n# Execution errors\nHermes.MCP.Error.execution(\"Database connection failed\", %{retries: 3})\n```","ref":"error_handling.html#creating-errors"},{"type":"extras","title":"Error Structure - Error Handling","doc":"All errors have:\n- `code` - JSON-RPC error code\n- `reason` - Semantic atom (e.g., `:method_not_found`)\n- `message` - Human-readable message\n- `data` - Additional context map","ref":"error_handling.html#error-structure"},{"type":"extras","title":"Handling Client Responses - Error Handling","doc":"```elixir\ncase MyApp.MCPClient.call_tool(\"search\", %{query: \"test\"}) do\n  # Success\n  {:ok, %Hermes.MCP.Response{is_error: false, result: result}} ->\n    IO.puts(\"Success: #{inspect(result)}\")\n\n  # Domain error (tool returned isError: true)\n  {:ok, %Hermes.MCP.Response{is_error: true, result: result}} ->\n    IO.puts(\"Tool error: #{result[\"message\"]}\")\n\n  # Protocol/transport error\n  {:error, %Hermes.MCP.Error{reason: reason}} ->\n    IO.puts(\"Error: #{reason}\")\nend\n```","ref":"error_handling.html#handling-client-responses"},{"type":"extras","title":"Common Error Patterns - Error Handling","doc":"","ref":"error_handling.html#common-error-patterns"},{"type":"extras","title":"Timeout Handling - Error Handling","doc":"```elixir\n# Set custom timeout (default is 30 seconds)\ncase MyApp.MCPClient.call_tool(\"slow_tool\", %{}, timeout: 60_000) do\n  {:error, %Hermes.MCP.Error{reason: :timeout}} ->\n    IO.puts(\"Request timed out\")\n  \n  other ->\n    handle_response(other)\nend\n```","ref":"error_handling.html#timeout-handling"},{"type":"extras","title":"Transport Errors - Error Handling","doc":"```elixir\n{:error, %Hermes.MCP.Error{reason: reason}} when reason in [\n  :connection_refused,\n  :connection_closed,\n  :timeout\n] ->\n  # Handle network issues\n```","ref":"error_handling.html#transport-errors"},{"type":"extras","title":"Method Not Found - Error Handling","doc":"```elixir\n{:error, %Hermes.MCP.Error{reason: :method_not_found}} ->\n  # Server doesn't support this method\n```","ref":"error_handling.html#method-not-found"},{"type":"extras","title":"Error Codes - Error Handling","doc":"Standard JSON-RPC codes:\n- `-32700` - Parse error\n- `-32600` - Invalid request\n- `-32601` - Method not found\n- `-32602` - Invalid params\n- `-32603` - Internal error\n\nMCP-specific:\n- `-32002` - Resource not found\n- `-32000` - Generic server error","ref":"error_handling.html#error-codes"},{"type":"extras","title":"Debugging - Error Handling","doc":"Errors have clean inspect output:\n\n```\n#MCP.Error \n#MCP.Error \n```","ref":"error_handling.html#debugging"},{"type":"extras","title":"Interactive CLI","doc":"# Interactive CLI\n\nTest and debug MCP servers with Hermes interactive CLI.","ref":"cli_usage.html"},{"type":"extras","title":"Available CLIs - Interactive CLI","doc":"- **STDIO Interactive** - For local subprocess servers\n- **StreamableHTTP Interactive** - For HTTP/SSE servers  \n- **WebSocket Interactive** - For WebSocket servers\n- **SSE Interactive** - For legacy SSE servers","ref":"cli_usage.html#available-clis"},{"type":"extras","title":"Quick Start - Interactive CLI","doc":"","ref":"cli_usage.html#quick-start"},{"type":"extras","title":"STDIO Server - Interactive CLI","doc":"```shell\nmix hermes.stdio.interactive --command=python --args=-m,mcp.server,my_server.py\n```","ref":"cli_usage.html#stdio-server"},{"type":"extras","title":"HTTP Server - Interactive CLI","doc":"```shell\nmix hermes.streamable_http.interactive --base-url=http://localhost:8080\n```","ref":"cli_usage.html#http-server"},{"type":"extras","title":"WebSocket Server - Interactive CLI","doc":"```shell\nmix hermes.websocket.interactive --base-url=ws://localhost:8081\n```","ref":"cli_usage.html#websocket-server"},{"type":"extras","title":"Common Options - Interactive CLI","doc":"| Option | Description |\n|--------|-------------|\n| `-h, --help` | Show help |\n| `-v` | Increase verbosity (can stack: `-vvv`) |\n\nVerbosity levels:\n- No flag: Errors only\n- `-v`: + Warnings\n- `-vv`: + Info\n- `-vvv`: + Debug","ref":"cli_usage.html#common-options"},{"type":"extras","title":"Interactive Commands - Interactive CLI","doc":"| Command | Description |\n|---------|-------------|\n| `help` | Show available commands |\n| `ping` | Check server connection |\n| `list_tools` | List available tools |\n| `call_tool` | Call a tool |\n| `list_prompts` | List prompts |\n| `get_prompt` | Get prompt messages |\n| `list_resources` | List resources |\n| `read_resource` | Read resource content |\n| `show_state` | Debug state info |\n| `clear` | Clear screen |\n| `exit` | Exit CLI |","ref":"cli_usage.html#interactive-commands"},{"type":"extras","title":"Examples - Interactive CLI","doc":"","ref":"cli_usage.html#examples"},{"type":"extras","title":"Test Local Server - Interactive CLI","doc":"```shell\n# Start interactive session\nmix hermes.stdio.interactive --command=./my-server\n\n# In the CLI\nmcp> ping\npong\n\nmcp> list_tools\nAvailable tools:\n- calculator: Perform calculations\n- file_reader: Read files\n\nmcp> call_tool\nTool name: calculator\nTool arguments (JSON): {\"operation\": \"add\", \"a\": 5, \"b\": 3}\nResult: 8\n```","ref":"cli_usage.html#test-local-server"},{"type":"extras","title":"Debug Connection - Interactive CLI","doc":"```shell\n# Verbose mode\nmix hermes.streamable_http.interactive -vvv --base-url=http://api.example.com\n\nmcp> show_state\nClient State:\n  Protocol: 2025-03-26\n  Initialized: true\n  Capabilities: %{\"tools\" => %{}}\n  \nTransport State:\n  Type: StreamableHTTP\n  URL: http://api.example.com\n  Connected: true\n```","ref":"cli_usage.html#debug-connection"},{"type":"extras","title":"Test Tool Execution - Interactive CLI","doc":"```\nmcp> list_tools\nmcp> call_tool\nTool name: search\nTool arguments (JSON): {\"query\": \"elixir\"}\n```","ref":"cli_usage.html#test-tool-execution"},{"type":"extras","title":"Transport-Specific Options - Interactive CLI","doc":"","ref":"cli_usage.html#transport-specific-options"},{"type":"extras","title":"STDIO - Interactive CLI","doc":"| Option | Description | Default |\n|--------|-------------|---------|\n| `--command` | Command to run | `mcp` |\n| `--args` | Comma-separated args | none |","ref":"cli_usage.html#stdio"},{"type":"extras","title":"StreamableHTTP - Interactive CLI","doc":"| Option | Description | Default |\n|--------|-------------|---------|\n| `--base-url` | Server URL | `http://localhost:8080` |\n| `--base-path` | Base path | `/` |","ref":"cli_usage.html#streamablehttp"},{"type":"extras","title":"WebSocket - Interactive CLI","doc":"| Option | Description | Default |\n|--------|-------------|---------|\n| `--base-url` | WebSocket URL | `ws://localhost:8081` |\n| `--ws-path` | WebSocket path | `/ws` |","ref":"cli_usage.html#websocket"},{"type":"extras","title":"Tips - Interactive CLI","doc":"1. Use `-vvv` for debugging connection issues\n2. `show_state` reveals internal state\n3. JSON arguments must be valid JSON\n4. Exit with `exit` or Ctrl+C","ref":"cli_usage.html#tips"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}